@@ -7,8 +7,8 @@ import java.io.OutputStream;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
 import java.security.Provider;
-import org.bouncycastle.jce.cert.CertStore;
-import org.bouncycastle.jce.cert.CertStoreException;
+import java.security.cert.CertStore;
+import java.security.cert.CertStoreException;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashMap;
