@@ -5,8 +5,8 @@ import java.security.AlgorithmParameters;
 import java.security.PrivateKey;
 import java.security.SecureRandom;
 import java.security.Signature;
-import org.bouncycastle.jce.cert.CertStore;
-import org.bouncycastle.jce.cert.CertStoreException;
+import java.security.cert.CertStore;
+import java.security.cert.CertStoreException;
 import java.security.interfaces.DSAPrivateKey;
 import java.security.interfaces.RSAPrivateKey;
 import java.util.ArrayList;
@@ -96,7 +96,7 @@ public class CMSSignedGenerator
     protected List signerGens = new ArrayList();
     protected Map digests = new HashMap();
 
-    protected final SecureRandom rand;
+    protected SecureRandom rand;
 
     /**
      * base constructor
