@@ -7,8 +7,6 @@ import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
 
 import java.security.SecureRandom;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 
 /**
  * this does your basic PKCS 1 v1.5 padding - whether or not you should be using this
@@ -51,7 +49,6 @@ public class PKCS1Encoding
     //
     private boolean useStrict()
     {
-        // required if security manager has been installed.
         String strict = System.getProperty(STRICT_LENGTH_ENABLED_PROPERTY);
 
         return strict == null || strict.equals("true");
@@ -227,7 +224,7 @@ public class PKCS1Encoding
 
         start++;           // data should start at the next byte
 
-        if (start >= block.length || start < HEADER_LENGTH)
+        if (start > block.length || start < HEADER_LENGTH)
         {
             throw new InvalidCipherTextException("no data in block");
         }
