@@ -443,5 +443,9 @@ public class PGPDSAElGamalTest implements Test
         TestResult result = test.perform();
 
         System.out.println(result.toString());
+        if (result.getException() != null)
+        {
+            result.getException().printStackTrace();
+        }
     }
 }
