@@ -21,12 +21,12 @@ import java.util.List;
 import java.util.Set;
 
 import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1Object;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.DERGeneralizedTime;
-import org.bouncycastle.asn1.DERObject;
-import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DEROutputStream;
 import org.bouncycastle.asn1.util.ASN1Dump;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
@@ -36,6 +36,7 @@ import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x509.X509Extensions;
 import org.bouncycastle.asn1.x509.X509Name;
 import org.bouncycastle.jce.PrincipalUtil;
+import org.bouncycastle.util.Integers;
 
 /**
  * A <code>CertSelector</code> that selects
@@ -76,9 +77,9 @@ import org.bouncycastle.jce.PrincipalUtil;
  * <br />
  * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
  * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
- * {@link org.bouncycastle.asn1.DERObjectIdentifier DERObjectIdentifier},
+ * {@link org.bouncycastle.asn1.ASN1ObjectIdentifier ASN1ObjectIdentifier},
  * {@link org.bouncycastle.asn1.DEROutputStream DEROutputStream},
- * {@link org.bouncycastle.asn1.DERObject DERObject},
+ * {@link org.bouncycastle.asn1.ASN1Object ASN1Object},
  * {@link org.bouncycastle.asn1.OIDTokenizer OIDTokenizer},
  * {@link org.bouncycastle.asn1.x509.X509Name X509Name},
  * {@link org.bouncycastle.asn1.x509.X509Extensions X509Extensions},
@@ -130,7 +131,7 @@ public class X509CertSelector implements CertSelector
 
     private Date privateKeyValid = null;
 
-    private DERObjectIdentifier subjectKeyAlgID = null;
+    private ASN1ObjectIdentifier subjectKeyAlgID = null;
 
     private PublicKey subjectPublicKey = null;
 
@@ -282,7 +283,7 @@ public class X509CertSelector implements CertSelector
      * subsequent modifications.<br />
      * <br />
      * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
-     * {@link org.bouncycastle.asn1.DERObject DERObject},
+     * {@link org.bouncycastle.asn1.ASN1Object ASN1Object},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
      * {@link org.bouncycastle.asn1.x509.X509Name X509Name}
      * 
@@ -304,7 +305,7 @@ public class X509CertSelector implements CertSelector
         {
             ByteArrayInputStream inStream = new ByteArrayInputStream(issuerDN);
             ASN1InputStream derInStream = new ASN1InputStream(inStream);
-            DERObject obj = derInStream.readObject();
+            ASN1Object obj = derInStream.readObject();
             if (obj instanceof ASN1Sequence)
             {
                 this.issuerDNX509 = new X509Name((ASN1Sequence)obj);
@@ -369,7 +370,7 @@ public class X509CertSelector implements CertSelector
      * {@link #setIssuer(byte []) setIssuer(byte [] issuerDN)}.<br />
      * <br />
      * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
-     * {@link org.bouncycastle.asn1.DERObject DERObject},
+     * {@link org.bouncycastle.asn1.ASN1Object ASN1Object},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
      * {@link org.bouncycastle.asn1.x509.X509Name X509Name}
      * 
@@ -391,7 +392,7 @@ public class X509CertSelector implements CertSelector
         {
             ByteArrayInputStream inStream = new ByteArrayInputStream(subjectDN);
             ASN1InputStream derInStream = new ASN1InputStream(inStream);
-            DERObject obj = derInStream.readObject();
+            ASN1Object obj = derInStream.readObject();
 
             if (obj instanceof ASN1Sequence)
             {
@@ -584,8 +585,15 @@ public class X509CertSelector implements CertSelector
      */
     public void setSubjectPublicKeyAlgID(String oid) throws IOException
     {
-        CertUtil.parseOID(oid);
-        subjectKeyAlgID = new DERObjectIdentifier(oid);
+        if (oid != null)
+        {
+            CertUtil.parseOID(oid);
+            subjectKeyAlgID = new ASN1ObjectIdentifier(oid);
+        }
+        else
+        {
+            subjectKeyAlgID = null;
+        }
     }
 
     /**
@@ -895,7 +903,7 @@ public class X509CertSelector implements CertSelector
         // TODO full implementation of CertUtil.parseGeneralName
         byte[] encoded = CertUtil.parseGeneralName(type, name);
         List tmpList = new ArrayList();
-        tmpList.add(new Integer(type));
+        tmpList.add(Integers.valueOf(type));
         tmpList.add(name);
         subjectAltNames.add(tmpList);
         tmpList.set(1, encoded);
@@ -954,7 +962,7 @@ public class X509CertSelector implements CertSelector
     {
         // TODO check encoded format
         List tmpList = new ArrayList();
-        tmpList.add(new Integer(type));
+        tmpList.add(Integers.valueOf(type));
         tmpList.add(name.clone());
         subjectAltNames.add(tmpList);
         subjectAltNamesByte.add(tmpList);
@@ -1093,7 +1101,7 @@ public class X509CertSelector implements CertSelector
                 if (item instanceof String)
                 {
                     CertUtil.parseOID((String)item);
-                    policyOID.add(new DERObjectIdentifier((String)item));
+                    policyOID.add(new ASN1ObjectIdentifier((String)item));
                 }
                 else
                 {
@@ -1235,7 +1243,7 @@ public class X509CertSelector implements CertSelector
         // TODO full implementation of CertUtil.parseGeneralName
         byte[] encoded = CertUtil.parseGeneralName(type, name);
         List tmpList = new ArrayList();
-        tmpList.add(new Integer(type));
+        tmpList.add(Integers.valueOf(type));
         tmpList.add(name);
         pathToNames.add(tmpList);
         tmpList.set(1, encoded);
@@ -1276,7 +1284,7 @@ public class X509CertSelector implements CertSelector
     {
         // TODO check encoded format
         List tmpList = new ArrayList();
-        tmpList.add(new Integer(type));
+        tmpList.add(Integers.valueOf(type));
         tmpList.add(name.clone());
         pathToNames.add(tmpList);
         pathToNamesByte.add(tmpList);
@@ -1375,7 +1383,7 @@ public class X509CertSelector implements CertSelector
             ByteArrayOutputStream outStream = new ByteArrayOutputStream();
             DEROutputStream derOutStream = new DEROutputStream(outStream);
 
-            derOutStream.writeObject(issuerDNX509.getDERObject());
+            derOutStream.writeObject(issuerDNX509.toASN1Primitive());
             derOutStream.close();
 
             return outStream.toByteArray();
@@ -1448,7 +1456,7 @@ public class X509CertSelector implements CertSelector
             ByteArrayOutputStream outStream = new ByteArrayOutputStream();
             DEROutputStream derOutStream = new DEROutputStream(outStream);
 
-            derOutStream.writeObject(subjectDNX509.getDERObject());
+            derOutStream.writeObject(subjectDNX509.toASN1Primitive());
             derOutStream.close();
 
             return outStream.toByteArray();
@@ -1860,7 +1868,7 @@ public class X509CertSelector implements CertSelector
      * constraints)</b><br />
      * <br />
      * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
-     * {@link org.bouncycastle.asn1.DERObject DERObject},
+     * {@link org.bouncycastle.asn1.ASN1Object ASN1Object},
      * {@link org.bouncycastle.asn1.x509.KeyPurposeId KeyPurposeId}
      * 
      * @return a <code>String</code> describing the contents of the
@@ -1893,7 +1901,7 @@ public class X509CertSelector implements CertSelector
                 ByteArrayInputStream inStream = new ByteArrayInputStream(
                         subjectKeyID);
                 ASN1InputStream derInStream = new ASN1InputStream(inStream);
-                DERObject derObject = derInStream.readObject();
+                ASN1Object derObject = derInStream.readObject();
                 sb.append("  Subject Key Identifier: ")
                        .append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
@@ -1902,7 +1910,7 @@ public class X509CertSelector implements CertSelector
                 ByteArrayInputStream inStream = new ByteArrayInputStream(
                         authorityKeyID);
                 ASN1InputStream derInStream = new ASN1InputStream(inStream);
-                DERObject derObject = derInStream.readObject();
+                ASN1Object derObject = derInStream.readObject();
                 sb.append("  Authority Key Identifier: ")
                        .append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
@@ -1958,7 +1966,7 @@ public class X509CertSelector implements CertSelector
                     ByteArrayInputStream inStream = new ByteArrayInputStream(
                             (byte[])obj.get(1));
                     ASN1InputStream derInStream = new ASN1InputStream(inStream);
-                    DERObject derObject = derInStream.readObject();
+                    ASN1Object derObject = derInStream.readObject();
                     sb.append("  Type: ").append(obj.get(0)).append(" Data: ")
                            .append(ASN1Dump.dumpAsString(derObject)).append('\n');
                 }
@@ -1982,7 +1990,7 @@ public class X509CertSelector implements CertSelector
                     ByteArrayInputStream inStream = new ByteArrayInputStream(
                             (byte[])obj.get(1));
                     ASN1InputStream derInStream = new ASN1InputStream(inStream);
-                    DERObject derObject = derInStream.readObject();
+                    ASN1Object derObject = derInStream.readObject();
                     sb.append("  Type: ").append(obj.get(0)).append(" Data: ")
                            .append(ASN1Dump.dumpAsString(derObject)).append('\n');
                 }
@@ -2004,8 +2012,8 @@ public class X509CertSelector implements CertSelector
      * <br />
      * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
-     * {@link org.bouncycastle.asn1.DERObjectIdentifier DERObjectIdentifier},
-     * {@link org.bouncycastle.asn1.DERObject DERObject},
+     * {@link org.bouncycastle.asn1.ASN1ObjectIdentifier ASN1ObjectIdentifier},
+     * {@link org.bouncycastle.asn1.ASN1Object ASN1Object},
      * {@link org.bouncycastle.asn1.DERGeneralizedTime DERGeneralizedTime},
      * {@link org.bouncycastle.asn1.x509.X509Name X509Name},
      * {@link org.bouncycastle.asn1.x509.X509Extensions X509Extensions},
@@ -2230,8 +2238,8 @@ public class X509CertSelector implements CertSelector
                     ByteArrayInputStream inStream = new ByteArrayInputStream(
                             data);
                     ASN1InputStream derInputStream = new ASN1InputStream(inStream);
-                    ExtendedKeyUsage extendedKeyUsage = new ExtendedKeyUsage(
-                            (ASN1Sequence)derInputStream.readObject());
+                    ExtendedKeyUsage extendedKeyUsage = ExtendedKeyUsage.getInstance(
+                            derInputStream.readObject());
                     tempIter = keyPurposeSet.iterator();
                     while (tempIter.hasNext())
                     {
@@ -2328,7 +2336,7 @@ public class X509CertSelector implements CertSelector
                 boolean test = false;
                 Set testSet = new HashSet(subjectAltNamesByte);
                 List testList;
-                DERObject derData;
+                ASN1Object derData;
                 ByteArrayOutputStream outStream;
                 DEROutputStream derOutStream;
                 while (altNamesSequence.hasMoreElements() && !test)
@@ -2336,7 +2344,7 @@ public class X509CertSelector implements CertSelector
                     altNameObject = (ASN1TaggedObject)altNamesSequence
                             .nextElement();
                     testList = new ArrayList(2);
-                    testList.add(new Integer(altNameObject.getTagNo()));
+                    testList.add(Integers.valueOf(altNameObject.getTagNo()));
                     derData = altNameObject.getObject();
                     outStream = new ByteArrayOutputStream();
                     derOutStream = new DEROutputStream(outStream);
@@ -2415,8 +2423,8 @@ public class X509CertSelector implements CertSelector
                 Iterator iter = policyOID.iterator();
                 while (iter.hasNext())
                 {
-                    copy.policyOID.add(new DERObjectIdentifier(
-                            ((DERObjectIdentifier)iter.next()).getId()));
+                    copy.policyOID.add(new ASN1ObjectIdentifier(
+                            ((ASN1ObjectIdentifier)iter.next()).getId()));
                 }
             }
             if (subjectAltNames != null)
