@@ -1,39 +1,27 @@
 package org.bouncycastle.asn1;
 
 import java.io.InputStream;
-import java.io.IOException;
-import java.io.ByteArrayOutputStream;
 
 abstract class LimitedInputStream
         extends InputStream
 {
     protected final InputStream _in;
+    private int _limit;
 
     LimitedInputStream(
-        InputStream in)
+        InputStream in,
+        int         limit)
     {
         this._in = in;
+        this._limit = limit;
     }
 
-    byte[] toByteArray()
-        throws IOException
+    int getRemaining()
     {
-        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
-
-        int b;
-        while ((b = this.read()) >= 0)
-        {
-            bOut.write(b);
-        }
-
-        return bOut.toByteArray();
+        // TODO: maybe one day this can become more accurate
+        return _limit;
     }
-
-    InputStream getUnderlyingStream()
-    {
-        return _in;
-    }
-
+    
     protected void setParentEofDetect(boolean on)
     {
         if (_in instanceof IndefiniteLengthInputStream)
