@@ -16,25 +16,25 @@ import org.bouncycastle.asn1.DEROctetString;
  * <p/>
  * <pre>
  *  CertificateBody ::= SEQUENCE {
- *  	// version of the certificate format. Must be 0 (version 1)
- *      CertificateProfileIdentifer 		DERApplicationSpecific,
+ *      // version of the certificate format. Must be 0 (version 1)
+ *      CertificateProfileIdentifer         DERApplicationSpecific,
  *      //uniquely identifies the issuinng CA's signature key pair
  *      // contains the iso3166-1 alpha2 encoded country code, the
  *      // name of issuer and the sequence number of the key pair.
- *      CertificationAuthorityReference		DERApplicationSpecific,
+ *      CertificationAuthorityReference        DERApplicationSpecific,
  *      // stores the encoded public key
- *      PublicKey							Iso7816PublicKey,
+ *      PublicKey                            Iso7816PublicKey,
  *      //associates the public key contained in the certificate with a unique name
  *      // contains the iso3166-1 alpha2 encoded country code, the
  *      // name of the holder and the sequence number of the key pair.
- *      certificateHolderReference			DERApplicationSpecific,
+ *      certificateHolderReference            DERApplicationSpecific,
  *      // Encodes the role of the holder (i.e. CVCA, DV, IS) and assigns read/write
  *      // access rights to data groups storing sensitive data
- *      certificateHolderAuthorization		Iso7816CertificateHolderAuthorization,
+ *      certificateHolderAuthorization        Iso7816CertificateHolderAuthorization,
  *      // the date of the certificate generation
- *      CertificateEffectiveDate			DERApplicationSpecific,
+ *      CertificateEffectiveDate            DERApplicationSpecific,
  *      // the date after wich the certificate expires
- *      certificateExpirationDate			DERApplicationSpecific
+ *      certificateExpirationDate            DERApplicationSpecific
  *  }
  * </pre>
  */
@@ -145,7 +145,7 @@ public class CertificateBody
             EACTags.ISSUER_IDENTIFICATION_NUMBER, certificationAuthorityReference.getEncoded()));
         setPublicKey(publicKey);
         setCertificateHolderReference(new DERApplicationSpecific(
-				EACTags.CARDHOLDER_NAME, certificateHolderReference.getEncoded()));
+            EACTags.CARDHOLDER_NAME, certificateHolderReference.getEncoded()));
         setCertificateHolderAuthorization(certificateHolderAuthorization);
         try
         {
@@ -194,51 +194,66 @@ public class CertificateBody
     }
 
     private void setCertificateProfileIdentifier(DERApplicationSpecific certificateProfileIdentifier)
-	throws IllegalArgumentException {
-		if (certificateProfileIdentifier.getApplicationTag() == EACTags.INTERCHANGE_PROFILE) {
-			this.certificateProfileIdentifier = certificateProfileIdentifier;
-			certificateType |= CPI;
-		}
-		else
-			throw new IllegalArgumentException("Not an Iso7816Tags.INTERCHANGE_PROFILE tag :"+ EACTags.encodeTag(certificateProfileIdentifier));
-	}
+        throws IllegalArgumentException
+    {
+        if (certificateProfileIdentifier.getApplicationTag() == EACTags.INTERCHANGE_PROFILE)
+        {
+            this.certificateProfileIdentifier = certificateProfileIdentifier;
+            certificateType |= CPI;
+        }
+        else
+        {
+            throw new IllegalArgumentException("Not an Iso7816Tags.INTERCHANGE_PROFILE tag :" + EACTags.encodeTag(certificateProfileIdentifier));
+        }
+    }
 
     private void setCertificateHolderReference(DERApplicationSpecific certificateHolderReference)
-	throws IllegalArgumentException {
-		if (certificateHolderReference.getApplicationTag() == EACTags.CARDHOLDER_NAME) {
-			this.certificateHolderReference = certificateHolderReference;
-			certificateType |= CHR;
-		}
-		else
-			throw new IllegalArgumentException("Not an Iso7816Tags.CARDHOLDER_NAME tag");
-	}
-
-    	/**
-	 * set the CertificationAuthorityReference.
-	 * @param certificationAuthorityReference the DERApplicationSpecific containing the CertificationAuthorityReference.
-	 * @throws IllegalArgumentException if the DERApplicationSpecific is not valid.
-	 */
-	private void setCertificationAuthorityReference(
-			DERApplicationSpecific certificationAuthorityReference)
-				throws IllegalArgumentException {
-		if (certificationAuthorityReference.getApplicationTag() == EACTags.ISSUER_IDENTIFICATION_NUMBER) {
-			this.certificationAuthorityReference = certificationAuthorityReference;
-			certificateType |= CAR;
-		}
-		else
-			throw new IllegalArgumentException("Not an Iso7816Tags.ISSUER_IDENTIFICATION_NUMBER tag");
-	}
-
-    	/**
-	 * set the public Key
-	 * @param publicKey : the DERApplicationSpecific containing the public key
-	 * @throws java.io.IOException
-	 */
-	private void setPublicKey(PublicKeyDataObject publicKey)
+        throws IllegalArgumentException
     {
-		this.publicKey = PublicKeyDataObject.getInstance(publicKey);
+        if (certificateHolderReference.getApplicationTag() == EACTags.CARDHOLDER_NAME)
+        {
+            this.certificateHolderReference = certificateHolderReference;
+            certificateType |= CHR;
+        }
+        else
+        {
+            throw new IllegalArgumentException("Not an Iso7816Tags.CARDHOLDER_NAME tag");
+        }
+    }
+
+    /**
+     * set the CertificationAuthorityReference.
+     *
+     * @param certificationAuthorityReference
+     *         the DERApplicationSpecific containing the CertificationAuthorityReference.
+     * @throws IllegalArgumentException if the DERApplicationSpecific is not valid.
+     */
+    private void setCertificationAuthorityReference(
+        DERApplicationSpecific certificationAuthorityReference)
+        throws IllegalArgumentException
+    {
+        if (certificationAuthorityReference.getApplicationTag() == EACTags.ISSUER_IDENTIFICATION_NUMBER)
+        {
+            this.certificationAuthorityReference = certificationAuthorityReference;
+            certificateType |= CAR;
+        }
+        else
+        {
+            throw new IllegalArgumentException("Not an Iso7816Tags.ISSUER_IDENTIFICATION_NUMBER tag");
+        }
+    }
+
+    /**
+     * set the public Key
+     *
+     * @param publicKey : the DERApplicationSpecific containing the public key
+     * @throws java.io.IOException
+     */
+    private void setPublicKey(PublicKeyDataObject publicKey)
+    {
+        this.publicKey = PublicKeyDataObject.getInstance(publicKey);
         this.certificateType |= PK;
-	}
+    }
 
     /**
      * create a request type Iso7816CertificateBody.
@@ -457,4 +472,4 @@ public class CertificateBody
     {
         return publicKey;
     }
-}
\ No newline at end of file
+}
