@@ -1,18 +1,18 @@
 package org.bouncycastle.asn1.sec;
 
-import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.asn1.x9.X9ECParameters;
-import org.bouncycastle.asn1.x9.X9ECParametersHolder;
-import org.bouncycastle.math.ec.ECCurve;
-import org.bouncycastle.math.ec.ECPoint;
-import org.bouncycastle.math.ec.ECConstants;
-import org.bouncycastle.util.Strings;
-import org.bouncycastle.util.encoders.Hex;
-
 import java.math.BigInteger;
 import java.util.Enumeration;
 import java.util.Hashtable;
 
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.x9.X9ECParameters;
+import org.bouncycastle.asn1.x9.X9ECParametersHolder;
+import org.bouncycastle.math.ec.ECConstants;
+import org.bouncycastle.math.ec.ECCurve;
+import org.bouncycastle.math.ec.ECPoint;
+import org.bouncycastle.util.Strings;
+import org.bouncycastle.util.encoders.Hex;
+
 public class SECNamedCurves
 {
     private static BigInteger fromHex(
@@ -427,7 +427,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("0100000000000000D9CCEC8A39E56F");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "009D73616F35F4AB1407D73562C10F"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -454,7 +454,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("010000000000000108789B2496AF93");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "01A57A6A7B26CA5EF52FCDB8164797"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -483,7 +483,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("0400000000000000023123953A9464B54D");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "0081BAF91FDF9833C40F9C181343638399"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -512,7 +512,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("0400000000000000016954A233049BA98F");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "0356DCD8F2F95031AD652D23951BB366A8"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -541,7 +541,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("04000000000000000000020108A2E0CC0D99F8A5EF");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "02FE13C0537BBC11ACAA07D793DE4E6D5E5C94EEE8"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -570,7 +570,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("03FFFFFFFFFFFFFFFFFFFF48AAB689C29CA710279B");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "0369979697AB43897789566789567F787A7876A654"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -599,7 +599,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("040000000000000000000292FE77E70C12A4234C33");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "03F0EBA16286A2D57EA0991168D4994637E8343E36"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -626,7 +626,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("01000000000000000000000000C7F34A778F443ACC920EBA49");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "01F481BC5F0FF84A74AD6CDF6FDEF4BF6179625372D8C0C5E1"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -653,7 +653,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("010000000000000000000000015AAB561B005413CCD4EE99D5");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "00D9B67D192E0367C803F39E1A7E82CA14A651350AAE617E8F"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -680,7 +680,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("8000000000000000000000000000069D5BB915BCD46EFB1AD5F173ABDF");
             BigInteger h = BigInteger.valueOf(4);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("02"
             //+ "017232BA853A7E731AF129F22FF4149563A419C26BF50A4C9D6EEFAD6126"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -707,7 +707,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("01000000000000000000000000000013E974E72F8A6922031D2603CFE0D7");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "00FAC9DFCBAC8313BB2139F1BB755FEF65BC391F8B36F8F8EB7371FD558B"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -734,7 +734,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("2000000000000000000000000000005A79FEC67CB6E91F1C1DA800E478A5");
             BigInteger h = BigInteger.valueOf(4);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "29A0B6A887A983E9730988A68727A8B2D126C44CC2CC7B2A6555193035DC"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -763,7 +763,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("01FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9AE2ED07577265DFF7F94451E061E163C61");
             BigInteger h = BigInteger.valueOf(4);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("02"
             //+ "0503213F78CA44883F1A3B8162F188E553CD265F23C1567A16876913B0C2AC2458492836"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -792,7 +792,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("03FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEF90399660FC938A90165B042A7CEFADB307");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "05F939258DB7DD90E1934F8C70B0DFEC2EED25B8557EAC9C80E2E198F8CDBECD86B12053"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -819,7 +819,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE5F83B2D4EA20400EC4557D5ED3E3E7CA5B4B5C83B8E01E5FCF");
             BigInteger h = BigInteger.valueOf(4);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "0060F05F658F49C1AD3AB1890F7184210EFD0987E307C84C27ACCFB8F9F67CC2C460189EB5AAAA62EE222EB1B35540CFE9023746"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -846,7 +846,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("010000000000000000000000000000000000000000000000000001E2AAD6A612F33307BE5FA47C3C9E052F838164CD37D9A21173");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "015D4860D088DDB3496B0C6064756260441CDE4AF1771D4DB01FFE5B34E59703DC255A868A1180515603AEAB60794E54BB7996A7"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -875,7 +875,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("020000000000000000000000000000000000000000000000000000000000000000000000131850E1F19A63E4B391A8DB917F4138B630D84BE5D639381E91DEB45CFE778F637C1001");
             BigInteger h = BigInteger.valueOf(4);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("02"
             //+ "026EB7A859923FBC82189631F8103FE4AC9CA2970012D5D46024804801841CA44370958493B205E647DA304DB4CEB08CBBD1BA39494776FB988B47174DCA88C7E2945283A01C8972"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -904,7 +904,7 @@ public class SECNamedCurves
             BigInteger n = fromHex("03FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE661CE18FF55987308059B186823851EC7DD9CA1161DE93D5174D66E8382E9BB2FE84E47");
             BigInteger h = BigInteger.valueOf(2);
 
-            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b);
+            ECCurve curve = new ECCurve.F2m(m, k1, k2, k3, a, b, n, h);
             //ECPoint G = curve.decodePoint(Hex.decode("03"
             //+ "0303001D34B856296C16C0D40D3CD7750A93D1D2955FA80AA5F40FC8DB7B2ABDBDE53950F4C0D293CDD711A35B67FB1499AE60038614F1394ABFA3B4C850D927E1E7769C8EEC2D19"));
             ECPoint G = curve.decodePoint(Hex.decode("04"
@@ -920,7 +920,7 @@ public class SECNamedCurves
     static final Hashtable curves = new Hashtable();
     static final Hashtable names = new Hashtable();
 
-    static void defineCurve(String name, DERObjectIdentifier oid, X9ECParametersHolder holder)
+    static void defineCurve(String name, ASN1ObjectIdentifier oid, X9ECParametersHolder holder)
     {
         objIds.put(name, oid);
         names.put(oid, name);
@@ -933,14 +933,14 @@ public class SECNamedCurves
         defineCurve("secp112r2", SECObjectIdentifiers.secp112r2, secp112r2);
         defineCurve("secp128r1", SECObjectIdentifiers.secp128r1, secp128r1);
         defineCurve("secp128r2", SECObjectIdentifiers.secp128r2, secp128r2);
-//        defineCurve("secp160k1", SECObjectIdentifiers.secp160k1, secp160k1);
+        defineCurve("secp160k1", SECObjectIdentifiers.secp160k1, secp160k1);
         defineCurve("secp160r1", SECObjectIdentifiers.secp160r1, secp160r1);
         defineCurve("secp160r2", SECObjectIdentifiers.secp160r2, secp160r2);
-//        defineCurve("secp192k1", SECObjectIdentifiers.secp192k1, secp192k1);
+        defineCurve("secp192k1", SECObjectIdentifiers.secp192k1, secp192k1);
         defineCurve("secp192r1", SECObjectIdentifiers.secp192r1, secp192r1);
-//        defineCurve("secp224k1", SECObjectIdentifiers.secp224k1, secp224k1);
+        defineCurve("secp224k1", SECObjectIdentifiers.secp224k1, secp224k1);
         defineCurve("secp224r1", SECObjectIdentifiers.secp224r1, secp224r1); 
-//        defineCurve("secp256k1", SECObjectIdentifiers.secp256k1, secp256k1);
+        defineCurve("secp256k1", SECObjectIdentifiers.secp256k1, secp256k1);
         defineCurve("secp256r1", SECObjectIdentifiers.secp256r1, secp256r1); 
         defineCurve("secp384r1", SECObjectIdentifiers.secp384r1, secp384r1); 
         defineCurve("secp521r1", SECObjectIdentifiers.secp521r1, secp521r1); 
@@ -968,7 +968,7 @@ public class SECNamedCurves
     public static X9ECParameters getByName(
         String name)
     {
-        DERObjectIdentifier oid = (DERObjectIdentifier)objIds.get(Strings.toLowerCase(name));
+        ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)objIds.get(Strings.toLowerCase(name));
 
         if (oid != null)
         {
@@ -985,7 +985,7 @@ public class SECNamedCurves
      * @param oid an object identifier representing a named curve, if present.
      */
     public static X9ECParameters getByOID(
-        DERObjectIdentifier oid)
+        ASN1ObjectIdentifier oid)
     {
         X9ECParametersHolder holder = (X9ECParametersHolder)curves.get(oid);
 
@@ -1003,17 +1003,17 @@ public class SECNamedCurves
      *
      * @return the object identifier associated with name, if present.
      */
-    public static DERObjectIdentifier getOID(
+    public static ASN1ObjectIdentifier getOID(
         String name)
     {
-        return (DERObjectIdentifier)objIds.get(Strings.toLowerCase(name));
+        return (ASN1ObjectIdentifier)objIds.get(Strings.toLowerCase(name));
     }
 
     /**
      * return the named curve name represented by the given object identifier.
      */
     public static String getName(
-        DERObjectIdentifier oid)
+        ASN1ObjectIdentifier oid)
     {
         return (String)names.get(oid);
     }
