@@ -1,25 +1,28 @@
 package org.bouncycastle.asn1.x509;
 
-import org.bouncycastle.asn1.ASN1Encodable;
-import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.asn1.ASN1Sequence;
-import org.bouncycastle.asn1.ASN1Set;
-import org.bouncycastle.asn1.ASN1TaggedObject;
-import org.bouncycastle.asn1.DEREncodable;
-import org.bouncycastle.asn1.DERObject;
-import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.asn1.DERSequence;
-import org.bouncycastle.asn1.DERSet;
-import org.bouncycastle.asn1.DERString;
-import org.bouncycastle.asn1.DERUniversalString;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.util.Strings;
-import org.bouncycastle.util.encoders.Hex;
-
+import java.io.IOException;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.ASN1Object;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.ASN1Sequence;
+import org.bouncycastle.asn1.ASN1Set;
+import org.bouncycastle.asn1.ASN1String;
+import org.bouncycastle.asn1.ASN1TaggedObject;
+import org.bouncycastle.asn1.DERSequence;
+import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.DERUniversalString;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.x500.X500Name;
+import org.bouncycastle.util.Strings;
+import org.bouncycastle.util.encoders.Hex;
+
 /**
  * <pre>
  *     RDNSequence ::= SEQUENCE OF RelativeDistinguishedName
@@ -30,165 +33,186 @@ import java.util.Vector;
  *                                   type  OBJECT IDENTIFIER,
  *                                   value ANY }
  * </pre>
+ * @deprecated use org.bouncycastle.asn1.x500.X500Name.
  */
 public class X509Name
-    extends ASN1Encodable
+    extends ASN1Object
 {
     /**
      * country code - StringType(SIZE(2))
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier C = new DERObjectIdentifier("2.5.4.6");
+    public static final ASN1ObjectIdentifier C = new ASN1ObjectIdentifier("2.5.4.6");
 
     /**
      * organization - StringType(SIZE(1..64))
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier O = new DERObjectIdentifier("2.5.4.10");
+    public static final ASN1ObjectIdentifier O = new ASN1ObjectIdentifier("2.5.4.10");
 
     /**
      * organizational unit name - StringType(SIZE(1..64))
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier OU = new DERObjectIdentifier("2.5.4.11");
+    public static final ASN1ObjectIdentifier OU = new ASN1ObjectIdentifier("2.5.4.11");
 
     /**
      * Title
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier T = new DERObjectIdentifier("2.5.4.12");
+    public static final ASN1ObjectIdentifier T = new ASN1ObjectIdentifier("2.5.4.12");
 
     /**
      * common name - StringType(SIZE(1..64))
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier CN = new DERObjectIdentifier("2.5.4.3");
+    public static final ASN1ObjectIdentifier CN = new ASN1ObjectIdentifier("2.5.4.3");
 
     /**
      * device serial number name - StringType(SIZE(1..64))
      */
-    public static final DERObjectIdentifier SN = new DERObjectIdentifier("2.5.4.5");
+    public static final ASN1ObjectIdentifier SN = new ASN1ObjectIdentifier("2.5.4.5");
 
     /**
      * street - StringType(SIZE(1..64))
      */
-    public static final DERObjectIdentifier STREET = new DERObjectIdentifier("2.5.4.9");
+    public static final ASN1ObjectIdentifier STREET = new ASN1ObjectIdentifier("2.5.4.9");
     
     /**
      * device serial number name - StringType(SIZE(1..64))
      */
-    public static final DERObjectIdentifier SERIALNUMBER = SN;
+    public static final ASN1ObjectIdentifier SERIALNUMBER = SN;
 
     /**
      * locality name - StringType(SIZE(1..64))
      */
-    public static final DERObjectIdentifier L = new DERObjectIdentifier("2.5.4.7");
+    public static final ASN1ObjectIdentifier L = new ASN1ObjectIdentifier("2.5.4.7");
 
     /**
      * state, or province name - StringType(SIZE(1..64))
      */
-    public static final DERObjectIdentifier ST = new DERObjectIdentifier("2.5.4.8");
+    public static final ASN1ObjectIdentifier ST = new ASN1ObjectIdentifier("2.5.4.8");
 
     /**
      * Naming attributes of type X520name
      */
-    public static final DERObjectIdentifier SURNAME = new DERObjectIdentifier("2.5.4.4");
-    public static final DERObjectIdentifier GIVENNAME = new DERObjectIdentifier("2.5.4.42");
-    public static final DERObjectIdentifier INITIALS = new DERObjectIdentifier("2.5.4.43");
-    public static final DERObjectIdentifier GENERATION = new DERObjectIdentifier("2.5.4.44");
-    public static final DERObjectIdentifier UNIQUE_IDENTIFIER = new DERObjectIdentifier("2.5.4.45");
+    public static final ASN1ObjectIdentifier SURNAME = new ASN1ObjectIdentifier("2.5.4.4");
+    public static final ASN1ObjectIdentifier GIVENNAME = new ASN1ObjectIdentifier("2.5.4.42");
+    public static final ASN1ObjectIdentifier INITIALS = new ASN1ObjectIdentifier("2.5.4.43");
+    public static final ASN1ObjectIdentifier GENERATION = new ASN1ObjectIdentifier("2.5.4.44");
+    public static final ASN1ObjectIdentifier UNIQUE_IDENTIFIER = new ASN1ObjectIdentifier("2.5.4.45");
 
     /**
      * businessCategory - DirectoryString(SIZE(1..128)
      */
-    public static final DERObjectIdentifier BUSINESS_CATEGORY = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier BUSINESS_CATEGORY = new ASN1ObjectIdentifier(
                     "2.5.4.15");
 
     /**
      * postalCode - DirectoryString(SIZE(1..40)
      */
-    public static final DERObjectIdentifier POSTAL_CODE = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier POSTAL_CODE = new ASN1ObjectIdentifier(
                     "2.5.4.17");
     
     /**
      * dnQualifier - DirectoryString(SIZE(1..64)
      */
-    public static final DERObjectIdentifier DN_QUALIFIER = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier DN_QUALIFIER = new ASN1ObjectIdentifier(
                     "2.5.4.46");
 
     /**
      * RFC 3039 Pseudonym - DirectoryString(SIZE(1..64)
      */
-    public static final DERObjectIdentifier PSEUDONYM = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier PSEUDONYM = new ASN1ObjectIdentifier(
                     "2.5.4.65");
 
 
     /**
      * RFC 3039 DateOfBirth - GeneralizedTime - YYYYMMDD000000Z
      */
-    public static final DERObjectIdentifier DATE_OF_BIRTH = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier DATE_OF_BIRTH = new ASN1ObjectIdentifier(
                     "1.3.6.1.5.5.7.9.1");
 
     /**
      * RFC 3039 PlaceOfBirth - DirectoryString(SIZE(1..128)
      */
-    public static final DERObjectIdentifier PLACE_OF_BIRTH = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier PLACE_OF_BIRTH = new ASN1ObjectIdentifier(
                     "1.3.6.1.5.5.7.9.2");
 
     /**
      * RFC 3039 Gender - PrintableString (SIZE(1)) -- "M", "F", "m" or "f"
      */
-    public static final DERObjectIdentifier GENDER = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier GENDER = new ASN1ObjectIdentifier(
                     "1.3.6.1.5.5.7.9.3");
 
     /**
      * RFC 3039 CountryOfCitizenship - PrintableString (SIZE (2)) -- ISO 3166
      * codes only
      */
-    public static final DERObjectIdentifier COUNTRY_OF_CITIZENSHIP = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier COUNTRY_OF_CITIZENSHIP = new ASN1ObjectIdentifier(
                     "1.3.6.1.5.5.7.9.4");
 
     /**
      * RFC 3039 CountryOfResidence - PrintableString (SIZE (2)) -- ISO 3166
      * codes only
      */
-    public static final DERObjectIdentifier COUNTRY_OF_RESIDENCE = new DERObjectIdentifier(
+    public static final ASN1ObjectIdentifier COUNTRY_OF_RESIDENCE = new ASN1ObjectIdentifier(
                     "1.3.6.1.5.5.7.9.5");
 
 
     /**
      * ISIS-MTT NameAtBirth - DirectoryString(SIZE(1..64)
      */
-    public static final DERObjectIdentifier NAME_AT_BIRTH =  new DERObjectIdentifier("1.3.36.8.3.14");
+    public static final ASN1ObjectIdentifier NAME_AT_BIRTH =  new ASN1ObjectIdentifier("1.3.36.8.3.14");
 
     /**
      * RFC 3039 PostalAddress - SEQUENCE SIZE (1..6) OF
      * DirectoryString(SIZE(1..30))
      */
-    public static final DERObjectIdentifier POSTAL_ADDRESS = new DERObjectIdentifier(
-                    "2.5.4.16");
+    public static final ASN1ObjectIdentifier POSTAL_ADDRESS = new ASN1ObjectIdentifier("2.5.4.16");
+
+    /**
+     * RFC 2256 dmdName
+     */
+    public static final ASN1ObjectIdentifier DMD_NAME = new ASN1ObjectIdentifier("2.5.4.54");
+
+    /**
+     * id-at-telephoneNumber
+     */
+    public static final ASN1ObjectIdentifier TELEPHONE_NUMBER = X509ObjectIdentifiers.id_at_telephoneNumber;
+
+    /**
+     * id-at-name
+     */
+    public static final ASN1ObjectIdentifier NAME = X509ObjectIdentifiers.id_at_name;
 
     /**
      * Email address (RSA PKCS#9 extension) - IA5String.
      * <p>Note: if you're trying to be ultra orthodox, don't use this! It shouldn't be in here.
+     * @deprecated use a X500NameStyle
      */
-    public static final DERObjectIdentifier EmailAddress = PKCSObjectIdentifiers.pkcs_9_at_emailAddress;
+    public static final ASN1ObjectIdentifier EmailAddress = PKCSObjectIdentifiers.pkcs_9_at_emailAddress;
     
     /**
      * more from PKCS#9
      */
-    public static final DERObjectIdentifier UnstructuredName = PKCSObjectIdentifiers.pkcs_9_at_unstructuredName;
-    public static final DERObjectIdentifier UnstructuredAddress = PKCSObjectIdentifiers.pkcs_9_at_unstructuredAddress;
+    public static final ASN1ObjectIdentifier UnstructuredName = PKCSObjectIdentifiers.pkcs_9_at_unstructuredName;
+    public static final ASN1ObjectIdentifier UnstructuredAddress = PKCSObjectIdentifiers.pkcs_9_at_unstructuredAddress;
     
     /**
      * email address in Verisign certificates
      */
-    public static final DERObjectIdentifier E = EmailAddress;
+    public static final ASN1ObjectIdentifier E = EmailAddress;
     
     /*
      * others...
      */
-    public static final DERObjectIdentifier DC = new DERObjectIdentifier("0.9.2342.19200300.100.1.25");
+    public static final ASN1ObjectIdentifier DC = new ASN1ObjectIdentifier("0.9.2342.19200300.100.1.25");
 
     /**
      * LDAP User id.
      */
-    public static final DERObjectIdentifier UID = new DERObjectIdentifier("0.9.2342.19200300.100.1.1");
+    public static final ASN1ObjectIdentifier UID = new ASN1ObjectIdentifier("0.9.2342.19200300.100.1.1");
 
     /**
      * determines whether or not strings should be processed and printed
@@ -266,6 +290,8 @@ public class X509Name
         DefaultSymbols.put(DATE_OF_BIRTH, "DateOfBirth");
         DefaultSymbols.put(POSTAL_CODE, "PostalCode");
         DefaultSymbols.put(BUSINESS_CATEGORY, "BusinessCategory");
+        DefaultSymbols.put(TELEPHONE_NUMBER, "TelephoneNumber");
+        DefaultSymbols.put(NAME, "Name");
 
         RFC2253Symbols.put(C, "C");
         RFC2253Symbols.put(O, "O");
@@ -317,6 +343,8 @@ public class X509Name
         DefaultLookUp.put("dateofbirth", DATE_OF_BIRTH);
         DefaultLookUp.put("postalcode", POSTAL_CODE);
         DefaultLookUp.put("businesscategory", BUSINESS_CATEGORY);
+        DefaultLookUp.put("telephonenumber", TELEPHONE_NUMBER);
+        DefaultLookUp.put("name", NAME);
     }
 
     private X509NameEntryConverter  converter = null;
@@ -326,6 +354,9 @@ public class X509Name
 
     private ASN1Sequence            seq;
 
+    private boolean                 isHashCodeCalculated;
+    private int                     hashCodeValue;
+
     /**
      * Return a X509Name based on the passed in tagged object.
      * 
@@ -347,18 +378,27 @@ public class X509Name
         {
             return (X509Name)obj;
         }
-        else if (obj instanceof ASN1Sequence)
+        else if (obj instanceof X500Name)
         {
-            return new X509Name((ASN1Sequence)obj);
+            return new X509Name(ASN1Sequence.getInstance(((X500Name)obj).toASN1Primitive()));
+        }
+        else if (obj != null)
+        {
+            return new X509Name(ASN1Sequence.getInstance(obj));
         }
 
-        throw new IllegalArgumentException("unknown object in factory: " + obj.getClass().getName());
+        return null;
     }
 
+    protected X509Name()
+    {
+        // constructure use by new X500 Name class
+    }
     /**
      * Constructor from ASN1Sequence
      *
      * the principal will be a list of constructed sets, each containing an (OID, String) pair.
+     * @deprecated use X500Name.getInstance()
      */
     public X509Name(
         ASN1Sequence  seq)
@@ -369,24 +409,24 @@ public class X509Name
 
         while (e.hasMoreElements())
         {
-            ASN1Set         set = ASN1Set.getInstance(e.nextElement());
+            ASN1Set         set = ASN1Set.getInstance(((ASN1Encodable)e.nextElement()).toASN1Primitive());
 
             for (int i = 0; i < set.size(); i++) 
             {
-                   ASN1Sequence s = ASN1Sequence.getInstance(set.getObjectAt(i));
+                   ASN1Sequence s = ASN1Sequence.getInstance(set.getObjectAt(i).toASN1Primitive());
 
                    if (s.size() != 2)
                    {
                        throw new IllegalArgumentException("badly sized pair");
                    }
 
-                   ordering.addElement(DERObjectIdentifier.getInstance(s.getObjectAt(0)));
+                   ordering.addElement(ASN1ObjectIdentifier.getInstance(s.getObjectAt(0)));
                    
-                   DEREncodable value = s.getObjectAt(1);
-                   if (value instanceof DERString && !(value instanceof DERUniversalString))
+                   ASN1Encodable value = s.getObjectAt(1);
+                   if (value instanceof ASN1String && !(value instanceof DERUniversalString))
                    {
-                       String v = ((DERString)value).getString();
-                       if (v.charAt(0) == '#')
+                       String v = ((ASN1String)value).getString();
+                       if (v.length() > 0 && v.charAt(0) == '#')
                        {
                            values.addElement("\\" + v);
                        }
@@ -397,7 +437,14 @@ public class X509Name
                    }
                    else
                    {
-                       values.addElement("#" + bytesToString(Hex.encode(value.getDERObject().getDEREncoded())));
+                       try
+                       {
+                           values.addElement("#" + bytesToString(Hex.encode(value.toASN1Primitive().getEncoded(ASN1Encoding.DER))));
+                       }
+                       catch (IOException e1)
+                       {
+                           throw new IllegalArgumentException("cannot encode value");
+                       }
                    }
                    added.addElement((i != 0) ? TRUE : FALSE);  // to allow earlier JDK compatibility
             }
@@ -447,6 +494,7 @@ public class X509Name
      * <p>
      * The passed in converter will be used to convert the strings into their
      * ASN.1 counterparts.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         Vector                   ordering,
@@ -476,7 +524,7 @@ public class X509Name
 
         for (int i = 0; i != this.ordering.size(); i++)
         {
-            DERObjectIdentifier     oid = (DERObjectIdentifier)this.ordering.elementAt(i);
+            ASN1ObjectIdentifier     oid = (ASN1ObjectIdentifier)this.ordering.elementAt(i);
 
             if (attributes.get(oid) == null)
             {
@@ -489,6 +537,7 @@ public class X509Name
 
     /**
      * Takes two vectors one of the oids and the other of the values.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         Vector  oids,
@@ -502,6 +551,7 @@ public class X509Name
      * <p>
      * The passed in converter will be used to convert the strings into their
      * ASN.1 counterparts.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         Vector                  oids,
@@ -523,19 +573,20 @@ public class X509Name
         }
     }
 
-    private Boolean isEncoded(String s)
-    {
-        if (s.charAt(0) == '#')
-        {
-            return TRUE;
-        }
-
-        return FALSE;
-    }
+//    private Boolean isEncoded(String s)
+//    {
+//        if (s.charAt(0) == '#')
+//        {
+//            return TRUE;
+//        }
+//
+//        return FALSE;
+//    }
 
     /**
      * Takes an X509 dir name as a string of the format "C=AU, ST=Victoria", or
      * some such, converting it into an ordered set of name attributes.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         String  dirName)
@@ -548,6 +599,7 @@ public class X509Name
      * some such, converting it into an ordered set of name attributes with each
      * string value being converted to its associated ASN.1 type using the passed
      * in converter.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         String                  dirName,
@@ -561,6 +613,7 @@ public class X509Name
      * some such, converting it into an ordered set of name attributes. If reverse
      * is true, create the encoded version of the sequence starting from the
      * last element in the string.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         boolean reverse,
@@ -575,6 +628,7 @@ public class X509Name
      * string value being converted to its associated ASN.1 type using the passed
      * in converter. If reverse is true the ASN.1 sequence representing the DN will
      * be built by starting at the end of the string, rather than the start.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         boolean                 reverse,
@@ -588,7 +642,7 @@ public class X509Name
      * Takes an X509 dir name as a string of the format "C=AU, ST=Victoria", or
      * some such, converting it into an ordered set of name attributes. lookUp
      * should provide a table of lookups, indexed by lowercase only strings and
-     * yielding a DERObjectIdentifier, other than that OID. and numeric oids
+     * yielding a ASN1ObjectIdentifier, other than that OID. and numeric oids
      * will be processed automatically.
      * <br>
      * If reverse is true, create the encoded version of the sequence
@@ -596,6 +650,7 @@ public class X509Name
      * @param reverse true if we should start scanning from the end (RFC 2553).
      * @param lookUp table of names and their oids.
      * @param dirName the X.500 string to be parsed.
+     * @deprecated use X500Name, X500NameBuilder
      */
     public X509Name(
         boolean     reverse,
@@ -605,20 +660,21 @@ public class X509Name
         this(reverse, lookUp, dirName, new X509DefaultEntryConverter());
     }
 
-    private DERObjectIdentifier decodeOID(
+    private ASN1ObjectIdentifier decodeOID(
         String      name,
         Hashtable   lookUp)
     {
+        name = name.trim();
         if (Strings.toUpperCase(name).startsWith("OID."))
         {
-            return new DERObjectIdentifier(name.substring(4));
+            return new ASN1ObjectIdentifier(name.substring(4));
         }
         else if (name.charAt(0) >= '0' && name.charAt(0) <= '9')
         {
-            return new DERObjectIdentifier(name);
+            return new ASN1ObjectIdentifier(name);
         }
 
-        DERObjectIdentifier oid = (DERObjectIdentifier)lookUp.get(Strings.toLowerCase(name));
+        ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)lookUp.get(Strings.toLowerCase(name));
         if (oid == null)
         {
             throw new IllegalArgumentException("Unknown object id - " + name + " - passed to distinguished name");
@@ -627,11 +683,86 @@ public class X509Name
         return oid;
     }
 
+    private String unescape(String elt)
+    {
+        if (elt.length() == 0 || (elt.indexOf('\\') < 0 && elt.indexOf('"') < 0))
+        {
+            return elt.trim();
+        }
+
+        char[] elts = elt.toCharArray();
+        boolean escaped = false;
+        boolean quoted = false;
+        StringBuffer buf = new StringBuffer(elt.length());
+        int start = 0;
+
+        // if it's an escaped hash string and not an actual encoding in string form
+        // we need to leave it escaped.
+        if (elts[0] == '\\')
+        {
+            if (elts[1] == '#')
+            {
+                start = 2;
+                buf.append("\\#");
+            }
+        }
+
+        boolean nonWhiteSpaceEncountered = false;
+        int     lastEscaped = 0;
+
+        for (int i = start; i != elts.length; i++)
+        {
+            char c = elts[i];
+
+            if (c != ' ')
+            {
+                nonWhiteSpaceEncountered = true;
+            }
+
+            if (c == '"')
+            {
+                if (!escaped)
+                {
+                    quoted = !quoted;
+                }
+                else
+                {
+                    buf.append(c);
+                }
+                escaped = false;
+            }
+            else if (c == '\\' && !(escaped || quoted))
+            {
+                escaped = true;
+                lastEscaped = buf.length();
+            }
+            else
+            {
+                if (c == ' ' && !escaped && !nonWhiteSpaceEncountered)
+                {
+                    continue;
+                }
+                buf.append(c);
+                escaped = false;
+            }
+        }
+
+        if (buf.length() > 0)
+        {
+            while (buf.charAt(buf.length() - 1) == ' ' && lastEscaped != (buf.length() - 1))
+            {
+                buf.setLength(buf.length() - 1);
+            }
+        }
+
+        return buf.toString();
+    }
+
     /**
      * Takes an X509 dir name as a string of the format "C=AU, ST=Victoria", or
      * some such, converting it into an ordered set of name attributes. lookUp
      * should provide a table of lookups, indexed by lowercase only strings and
-     * yielding a DERObjectIdentifier, other than that OID. and numeric oids
+     * yielding a ASN1ObjectIdentifier, other than that OID. and numeric oids
      * will be processed automatically. The passed in converter is used to convert the
      * string values to the right of each equals sign to their ASN.1 counterparts.
      * <br>
@@ -652,43 +783,21 @@ public class X509Name
         while (nTok.hasMoreTokens())
         {
             String  token = nTok.nextToken();
-            int     index = token.indexOf('=');
 
-            if (index == -1)
+            if (token.indexOf('+') > 0)
             {
-                throw new IllegalArgumentException("badly formated directory string");
-            }
+                X509NameTokenizer   pTok = new X509NameTokenizer(token, '+');
 
-            String              name = token.substring(0, index);
-            String              value = token.substring(index + 1);
-            DERObjectIdentifier oid = decodeOID(name, lookUp);
+                addEntry(lookUp, pTok.nextToken(), FALSE);
 
-            if (value.indexOf('+') > 0)
-            {
-                X509NameTokenizer   vTok = new X509NameTokenizer(value, '+');
-                String  v = vTok.nextToken();
-
-                this.ordering.addElement(oid);
-                this.values.addElement(v);
-                this.added.addElement(FALSE);
-
-                while (vTok.hasMoreTokens())
+                while (pTok.hasMoreTokens())
                 {
-                    String  sv = vTok.nextToken();
-                    int     ndx = sv.indexOf('=');
-
-                    String  nm = sv.substring(0, ndx);
-                    String  vl = sv.substring(ndx + 1);
-                    this.ordering.addElement(decodeOID(nm, lookUp));
-                    this.values.addElement(vl);
-                    this.added.addElement(TRUE);
+                    addEntry(lookUp, pTok.nextToken(), TRUE);
                 }
             }
             else
             {
-                this.ordering.addElement(oid);
-                this.values.addElement(value);
-                this.added.addElement(FALSE);
+                addEntry(lookUp, token, FALSE);
             }
         }
 
@@ -724,6 +833,29 @@ public class X509Name
         }
     }
 
+    private void addEntry(Hashtable lookUp, String token, Boolean isAdded)
+    {
+        X509NameTokenizer vTok;
+        String name;
+        String value;ASN1ObjectIdentifier oid;
+        vTok = new X509NameTokenizer(token, '=');
+
+        name = vTok.nextToken();
+
+        if (!vTok.hasMoreTokens())
+        {
+           throw new IllegalArgumentException("badly formatted directory string");
+        }
+
+        value = vTok.nextToken();
+
+        oid = decodeOID(name, lookUp);
+
+        this.ordering.addElement(oid);
+        this.values.addElement(unescape(value));
+        this.added.addElement(isAdded);
+    }
+
     /**
      * return a vector of the oids in the name, in the order they were found.
      */
@@ -760,7 +892,7 @@ public class X509Name
      * were found, with the DN label corresponding to passed in oid.
      */
     public Vector getValues(
-        DERObjectIdentifier oid)
+        ASN1ObjectIdentifier oid)
     {
         Vector  v = new Vector();
 
@@ -784,18 +916,18 @@ public class X509Name
         return v;
     }
 
-    public DERObject toASN1Object()
+    public ASN1Primitive toASN1Primitive()
     {
         if (seq == null)
         {
             ASN1EncodableVector  vec = new ASN1EncodableVector();
             ASN1EncodableVector  sVec = new ASN1EncodableVector();
-            DERObjectIdentifier  lstOid = null;
+            ASN1ObjectIdentifier  lstOid = null;
             
             for (int i = 0; i != ordering.size(); i++)
             {
                 ASN1EncodableVector     v = new ASN1EncodableVector();
-                DERObjectIdentifier     oid = (DERObjectIdentifier)ordering.elementAt(i);
+                ASN1ObjectIdentifier     oid = (ASN1ObjectIdentifier)ordering.elementAt(i);
 
                 v.add(oid);
 
@@ -848,9 +980,9 @@ public class X509Name
             return false;
         }
 
-        DERObject derO = ((DEREncodable)obj).getDERObject();
+        ASN1Primitive derO = ((ASN1Encodable)obj).toASN1Primitive();
 
-        if (this.getDERObject().equals(derO))
+        if (this.toASN1Primitive().equals(derO))
         {
             return true;
         }
@@ -875,8 +1007,8 @@ public class X509Name
 
         for (int i = 0; i < orderingSize; i++)
         {
-            DERObjectIdentifier  oid = (DERObjectIdentifier)ordering.elementAt(i);
-            DERObjectIdentifier  oOid = (DERObjectIdentifier)other.ordering.elementAt(i);
+            ASN1ObjectIdentifier  oid = (ASN1ObjectIdentifier)ordering.elementAt(i);
+            ASN1ObjectIdentifier  oOid = (ASN1ObjectIdentifier)other.ordering.elementAt(i);
 
             if (oid.equals(oOid))
             {
@@ -897,6 +1029,30 @@ public class X509Name
         return true;
     }
 
+    public int hashCode()
+    {
+        if (isHashCodeCalculated)
+        {
+            return hashCodeValue;
+        }
+
+        isHashCodeCalculated = true;
+
+        // this needs to be order independent, like equals
+        for (int i = 0; i != ordering.size(); i += 1)
+        {
+            String value = (String)values.elementAt(i);
+
+            value = canonicalize(value);
+            value = stripInternalSpaces(value);
+
+            hashCodeValue ^= ordering.elementAt(i).hashCode();
+            hashCodeValue ^= value.hashCode();
+        }
+
+        return hashCodeValue;
+    }
+
     /**
      * test for equality - note: case is ignored.
      */
@@ -912,9 +1068,9 @@ public class X509Name
             return false;
         }
         
-        DERObject derO = ((DEREncodable)obj).getDERObject();
+        ASN1Primitive derO = ((ASN1Encodable)obj).toASN1Primitive();
         
-        if (this.getDERObject().equals(derO))
+        if (this.toASN1Primitive().equals(derO))
         {
             return true;
         }
@@ -956,17 +1112,17 @@ public class X509Name
         for (int i = start; i != end; i += delta)
         {
             boolean              found = false;
-            DERObjectIdentifier  oid = (DERObjectIdentifier)ordering.elementAt(i);
+            ASN1ObjectIdentifier  oid = (ASN1ObjectIdentifier)ordering.elementAt(i);
             String               value = (String)values.elementAt(i);
-            
+
             for (int j = 0; j < orderingSize; j++)
             {
                 if (indexes[j])
                 {
                     continue;
                 }
-                
-                DERObjectIdentifier oOid = (DERObjectIdentifier)other.ordering.elementAt(j);
+
+                ASN1ObjectIdentifier oOid = (ASN1ObjectIdentifier)other.ordering.elementAt(j);
 
                 if (oid.equals(oOid))
                 {
@@ -992,9 +1148,9 @@ public class X509Name
 
     private boolean equivalentStrings(String s1, String s2)
     {
-        String value = Strings.toLowerCase(s1.trim());
-        String oValue = Strings.toLowerCase(s2.trim());
-
+        String value = canonicalize(s1);
+        String oValue = canonicalize(s2);
+        
         if (!value.equals(oValue))
         {
             value = stripInternalSpaces(value);
@@ -1009,6 +1165,35 @@ public class X509Name
         return true;
     }
 
+    private String canonicalize(String s)
+    {
+        String value = Strings.toLowerCase(s.trim());
+        
+        if (value.length() > 0 && value.charAt(0) == '#')
+        {
+            ASN1Primitive obj = decodeObject(value);
+
+            if (obj instanceof ASN1String)
+            {
+                value = Strings.toLowerCase(((ASN1String)obj).getString().trim());
+            }
+        }
+
+        return value;
+    }
+
+    private ASN1Primitive decodeObject(String oValue)
+    {
+        try
+        {
+            return ASN1Primitive.fromByteArray(Hex.decode(oValue.substring(1)));
+        }
+        catch (IOException e)
+        {
+            throw new IllegalStateException("unknown encoding in name: " + e);
+        }
+    }
+
     private String stripInternalSpaces(
         String str)
     {
@@ -1034,24 +1219,10 @@ public class X509Name
         return res.toString();
     }
 
-    public int hashCode()
-    {
-        ASN1Sequence  seq = (ASN1Sequence)this.getDERObject();
-        Enumeration   e = seq.getObjects();
-        int           hashCode = 0;
-
-        while (e.hasMoreElements())
-        {
-            hashCode ^= e.nextElement().hashCode();
-        }
-
-        return hashCode;
-    }
-
     private void appendValue(
         StringBuffer        buf,
         Hashtable           oidSymbols,
-        DERObjectIdentifier oid,
+        ASN1ObjectIdentifier oid,
         String              value)
     {
         String  sym = (String)oidSymbols.get(oid);
@@ -1068,7 +1239,8 @@ public class X509Name
         buf.append('=');
 
         int     index = buf.length();
-        
+        int     start = index;
+
         buf.append(value);
 
         int     end = buf.length();
@@ -1084,6 +1256,7 @@ public class X509Name
                || (buf.charAt(index) == '"')
                || (buf.charAt(index) == '\\')
                || (buf.charAt(index) == '+')
+               || (buf.charAt(index) == '=')
                || (buf.charAt(index) == '<')
                || (buf.charAt(index) == '>')
                || (buf.charAt(index) == ';'))
@@ -1095,6 +1268,20 @@ public class X509Name
 
             index++;
         }
+
+        while (buf.charAt(start) == ' ')
+        {
+            buf.insert(start, "\\");
+            start += 2;
+        }
+
+        int endBuf = buf.length() - 1;
+
+        while (endBuf >= 0 && buf.charAt(endBuf) == ' ')
+        {
+            buf.insert(endBuf, '\\');
+            endBuf--;
+        }
     }
 
     /**
@@ -1125,14 +1312,14 @@ public class X509Name
             {
                 ava.append('+');
                 appendValue(ava, oidSymbols,
-                    (DERObjectIdentifier)ordering.elementAt(i),
+                    (ASN1ObjectIdentifier)ordering.elementAt(i),
                     (String)values.elementAt(i));
             }
             else
             {
                 ava = new StringBuffer();
                 appendValue(ava, oidSymbols,
-                    (DERObjectIdentifier)ordering.elementAt(i),
+                    (ASN1ObjectIdentifier)ordering.elementAt(i),
                     (String)values.elementAt(i));
                 components.addElement(ava);
             }
