@@ -1,14 +1,14 @@
 package org.bouncycastle.asn1.x9;
 
-import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.math.ec.ECCurve;
-import org.bouncycastle.util.Strings;
-import org.bouncycastle.util.encoders.Hex;
-
 import java.math.BigInteger;
 import java.util.Enumeration;
 import java.util.Hashtable;
 
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.math.ec.ECCurve;
+import org.bouncycastle.util.Strings;
+import org.bouncycastle.util.encoders.Hex;
+
 
 /**
  * table of the current named curves defined in X.962 EC-DSA.
@@ -170,7 +170,7 @@ public class X962NamedCurves
                 c2m163v1.decodePoint(
                     Hex.decode("0307AF69989546103D79329FCC3D74880F33BBE803CB")),
                 c2m163v1n, c2m163v1h,
-                Hex.decode("D2COFB15760860DEF1EEF4D696E6768756151754"));
+                Hex.decode("D2C0FB15760860DEF1EEF4D696E6768756151754"));
         }
     };
 
@@ -523,7 +523,7 @@ public class X962NamedCurves
     static final Hashtable curves = new Hashtable();
     static final Hashtable names = new Hashtable();
 
-    static void defineCurve(String name, DERObjectIdentifier oid, X9ECParametersHolder holder)
+    static void defineCurve(String name, ASN1ObjectIdentifier oid, X9ECParametersHolder holder)
     {
         objIds.put(name, oid);
         names.put(oid, name);
@@ -560,7 +560,7 @@ public class X962NamedCurves
     public static X9ECParameters getByName(
         String name)
     {
-        DERObjectIdentifier oid = (DERObjectIdentifier)objIds.get(Strings.toLowerCase(name));
+        ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)objIds.get(Strings.toLowerCase(name));
 
         if (oid != null)
         {
@@ -577,7 +577,7 @@ public class X962NamedCurves
      * @param oid an object identifier representing a named curve, if present.
      */
     public static X9ECParameters getByOID(
-        DERObjectIdentifier oid)
+        ASN1ObjectIdentifier oid)
     {
         X9ECParametersHolder holder = (X9ECParametersHolder)curves.get(oid);
 
@@ -595,17 +595,17 @@ public class X962NamedCurves
      *
      * @return the object identifier associated with name, if present.
      */
-    public static DERObjectIdentifier getOID(
+    public static ASN1ObjectIdentifier getOID(
         String name)
     {
-        return (DERObjectIdentifier)objIds.get(Strings.toLowerCase(name));
+        return (ASN1ObjectIdentifier)objIds.get(Strings.toLowerCase(name));
     }
 
     /**
      * return the named curve name represented by the given object identifier.
      */
     public static String getName(
-        DERObjectIdentifier oid)
+        ASN1ObjectIdentifier oid)
     {
         return (String)names.get(oid);
     }
