@@ -3,6 +3,7 @@ package org.bouncycastle.cert.crmf;
 import java.io.IOException;
 import java.io.OutputStream;
 
+import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.cmp.PBMParameter;
 import org.bouncycastle.asn1.crmf.PKMACValue;
 import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
@@ -28,7 +29,7 @@ class PKMACValueVerifier
 
         try
         {
-            macOut.write(keyInfo.getDEREncoded());
+            macOut.write(keyInfo.getEncoded(ASN1Encoding.DER));
 
             macOut.close();
         }
