@@ -1,31 +1,33 @@
 package org.bouncycastle.cms;
 
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.Provider;
+import java.security.SecureRandom;
+import java.util.HashMap;
+import java.util.Iterator;
+
+import javax.crypto.KeyGenerator;
+
 import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
-import org.bouncycastle.asn1.BERConstructedOctetString;
+import org.bouncycastle.asn1.ASN1Set;
+import org.bouncycastle.asn1.BEROctetString;
+import org.bouncycastle.asn1.BERSet;
 import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.cms.AttributeTable;
+import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
 import org.bouncycastle.asn1.cms.ContentInfo;
 import org.bouncycastle.asn1.cms.EncryptedContentInfo;
 import org.bouncycastle.asn1.cms.EnvelopedData;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
-
-import javax.crypto.Cipher;
-import javax.crypto.CipherOutputStream;
-import javax.crypto.KeyGenerator;
-import javax.crypto.NoSuchPaddingException;
-import javax.crypto.SecretKey;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.security.AlgorithmParameters;
-import java.security.GeneralSecurityException;
-import java.security.InvalidAlgorithmParameterException;
-import java.security.InvalidKeyException;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.SecureRandom;
-import java.security.Provider;
-import java.util.Iterator;
+import org.bouncycastle.cms.jcajce.JceCMSContentEncryptorBuilder;
+import org.bouncycastle.operator.GenericKey;
+import org.bouncycastle.operator.OutputEncryptor;
 
 /**
  * General class for generating a CMS enveloped-data message.
@@ -33,11 +35,17 @@ import java.util.Iterator;
  * A simple example of usage.
  *
  * <pre>
- *      CMSEnvelopedDataGenerator  fact = new CMSEnvelopedDataGenerator();
+ *       CMSTypedData msg     = new CMSProcessableByteArray("Hello World!".getBytes());
  *
- *      fact.addKeyTransRecipient(cert);
+ *       CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
+ *
+ *       edGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(recipientCert).setProvider("BC"));
+ *
+ *       CMSEnvelopedData ed = edGen.generate(
+ *                                       msg,
+ *                                       new JceCMSContentEncryptorBuilder(CMSAlgorithm.DES_EDE3_CBC)
+ *                                              .setProvider("BC").build());
  *
- *      CMSEnvelopedData         data = fact.generate(content, algorithm, "BC");
  * </pre>
  */
 public class CMSEnvelopedDataGenerator
@@ -53,6 +61,7 @@ public class CMSEnvelopedDataGenerator
     /**
      * constructor allowing specific source of randomness
      * @param rand instance of SecureRandom to use
+     * @deprecated use no args constructor.
      */
     public CMSEnvelopedDataGenerator(
         SecureRandom rand)
@@ -65,105 +74,117 @@ public class CMSEnvelopedDataGenerator
      * object using the given provider and the passed in key generator.
      */
     private CMSEnvelopedData generate(
-        CMSProcessable  content,
+        final CMSProcessable  content,
         String          encryptionOID,
-        KeyGenerator    keyGen,
+        int             keySize,
+        Provider        encProvider,
         Provider        provider)
         throws NoSuchAlgorithmException, CMSException
     {
-        Provider                encProvider = keyGen.getProvider();
+        convertOldRecipients(rand, provider);
+
+        JceCMSContentEncryptorBuilder builder;
+
+        if (keySize != -1)
+        {
+            builder =  new JceCMSContentEncryptorBuilder(new ASN1ObjectIdentifier(encryptionOID), keySize);
+        }
+        else
+        {
+            builder = new JceCMSContentEncryptorBuilder(new ASN1ObjectIdentifier(encryptionOID));
+        }
+
+        builder.setProvider(encProvider);
+        builder.setSecureRandom(rand);
+
+        return doGenerate(new CMSTypedData()
+        {
+            public ASN1ObjectIdentifier getContentType()
+            {
+                return CMSObjectIdentifiers.data;
+            }
+
+            public void write(OutputStream out)
+                throws IOException, CMSException
+            {
+                content.write(out);
+            }
+
+            public Object getContent()
+            {
+                return content;
+            }
+        }, builder.build());
+    }
+
+    private CMSEnvelopedData doGenerate(
+        CMSTypedData content,
+        OutputEncryptor contentEncryptor)
+        throws CMSException
+    {
+        if (!oldRecipientInfoGenerators.isEmpty())
+        {
+            throw new IllegalStateException("can only use addRecipientGenerator() with this method");
+        }
+
         ASN1EncodableVector     recipientInfos = new ASN1EncodableVector();
         AlgorithmIdentifier     encAlgId;
-        SecretKey               encKey;
         ASN1OctetString         encContent;
 
+        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
+
         try
         {
-            Cipher cipher = CMSEnvelopedHelper.INSTANCE.getSymmetricCipher(encryptionOID, encProvider);
-
-            AlgorithmParameters params;
-            
-            encKey = keyGen.generateKey();
-            params = generateParameters(encryptionOID, encKey, encProvider);
-
-            cipher.init(Cipher.ENCRYPT_MODE, encKey, params, rand);
-
-            //
-            // If params are null we try and second guess on them as some providers don't provide
-            // algorithm parameter generation explicity but instead generate them under the hood.
-            //
-            if (params == null)
-            {
-                params = cipher.getParameters();
-            }
-            
-            encAlgId = getAlgorithmIdentifier(encryptionOID, params);
-
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            CipherOutputStream      cOut = new CipherOutputStream(bOut, cipher);
+            OutputStream cOut = contentEncryptor.getOutputStream(bOut);
 
             content.write(cOut);
 
             cOut.close();
-
-            encContent = new BERConstructedOctetString(bOut.toByteArray());
-        }
-        catch (InvalidKeyException e)
-        {
-            throw new CMSException("key invalid in message.", e);
-        }
-        catch (NoSuchPaddingException e)
-        {
-            throw new CMSException("required padding not supported.", e);
-        }
-        catch (InvalidAlgorithmParameterException e)
-        {
-            throw new CMSException("algorithm parameters invalid.", e);
         }
         catch (IOException e)
         {
-            throw new CMSException("exception decoding algorithm parameters.", e);
+            throw new CMSException("");
         }
 
-        Iterator            it = recipientInfs.iterator();
+        byte[] encryptedContent = bOut.toByteArray();
 
-        while (it.hasNext())
+        encAlgId = contentEncryptor.getAlgorithmIdentifier();
+
+        encContent = new BEROctetString(encryptedContent);
+
+        GenericKey encKey = contentEncryptor.getKey();
+
+        for (Iterator it = recipientInfoGenerators.iterator(); it.hasNext();)
         {
-            RecipientInf            recipient = (RecipientInf)it.next();
+            RecipientInfoGenerator recipient = (RecipientInfoGenerator)it.next();
 
-            try
-            {
-                recipientInfos.add(recipient.toRecipientInfo(encKey, rand, provider));
-            }
-            catch (IOException e)
-            {
-                throw new CMSException("encoding error.", e);
-            }
-            catch (InvalidKeyException e)
-            {
-                throw new CMSException("key inappropriate for algorithm.", e);
-            }
-            catch (GeneralSecurityException e)
-            {
-                throw new CMSException("error making encrypted content.", e);
-            }
+            recipientInfos.add(recipient.generate(encKey));
         }
 
         EncryptedContentInfo  eci = new EncryptedContentInfo(
-                                 PKCSObjectIdentifiers.data,
-                                 encAlgId, 
-                                 encContent);
+                        content.getContentType(),
+                        encAlgId,
+                        encContent);
+
+        ASN1Set unprotectedAttrSet = null;
+        if (unprotectedAttributeGenerator != null)
+        {
+            AttributeTable attrTable = unprotectedAttributeGenerator.getAttributes(new HashMap());
+
+            unprotectedAttrSet = new BERSet(attrTable.toASN1EncodableVector());
+        }
 
         ContentInfo contentInfo = new ContentInfo(
-                PKCSObjectIdentifiers.envelopedData,
-                new EnvelopedData(null, new DERSet(recipientInfos), eci, null));
+                CMSObjectIdentifiers.envelopedData,
+                new EnvelopedData(originatorInfo, new DERSet(recipientInfos), eci, unprotectedAttrSet));
 
         return new CMSEnvelopedData(contentInfo);
     }
-    
+
     /**
      * generate an enveloped object that contains an CMS Enveloped Data
      * object using the given provider.
+     * @deprecated use OutputEncryptor method.
      */
     public CMSEnvelopedData generate(
         CMSProcessable  content,
@@ -177,6 +198,7 @@ public class CMSEnvelopedDataGenerator
     /**
      * generate an enveloped object that contains an CMS Enveloped Data
      * object using the given provider.
+     * @deprecated use OutputEncryptor method.
      */
     public CMSEnvelopedData generate(
         CMSProcessable  content,
@@ -186,14 +208,13 @@ public class CMSEnvelopedDataGenerator
     {
         KeyGenerator keyGen = CMSEnvelopedHelper.INSTANCE.createSymmetricKeyGenerator(encryptionOID, provider);
 
-        keyGen.init(rand);
-
-        return generate(content, encryptionOID, keyGen, provider);
+        return generate(content, encryptionOID, -1, keyGen.getProvider(), provider);
     }
 
     /**
      * generate an enveloped object that contains an CMS Enveloped Data
      * object using the given provider.
+     * @deprecated use OutputEncryptor method.
      */
     public CMSEnvelopedData generate(
         CMSProcessable  content,
@@ -208,6 +229,7 @@ public class CMSEnvelopedDataGenerator
     /**
      * generate an enveloped object that contains an CMS Enveloped Data
      * object using the given provider.
+     * @deprecated use OutputEncryptor method.
      */
     public CMSEnvelopedData generate(
         CMSProcessable  content,
@@ -218,8 +240,21 @@ public class CMSEnvelopedDataGenerator
     {
         KeyGenerator keyGen = CMSEnvelopedHelper.INSTANCE.createSymmetricKeyGenerator(encryptionOID, provider);
 
-        keyGen.init(keySize, rand);
+        return generate(content, encryptionOID, keySize, keyGen.getProvider(), provider);
+    }
 
-        return generate(content, encryptionOID, keyGen, provider);
+    /**
+     * generate an enveloped object that contains an CMS Enveloped Data
+     * object using the given provider.
+     *
+     * @param content the content to be encrypted
+     * @param contentEncryptor the symmetric key based encryptor to encrypt the content with.
+     */
+    public CMSEnvelopedData generate(
+        CMSTypedData content,
+        OutputEncryptor contentEncryptor)
+        throws CMSException
+    {
+        return doGenerate(content, contentEncryptor);
     }
 }
