@@ -1,31 +1,26 @@
 package org.bouncycastle.cms;
 
-import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.asn1.ASN1OctetStringParser;
-import org.bouncycastle.asn1.ASN1SequenceParser;
-import org.bouncycastle.asn1.ASN1SetParser;
-import org.bouncycastle.asn1.DEREncodable;
-import org.bouncycastle.asn1.DERSet;
-import org.bouncycastle.asn1.DERTags;
-import org.bouncycastle.asn1.cms.AttributeTable;
-import org.bouncycastle.asn1.cms.EncryptedContentInfoParser;
-import org.bouncycastle.asn1.cms.EnvelopedDataParser;
-import org.bouncycastle.asn1.cms.KEKRecipientInfo;
-import org.bouncycastle.asn1.cms.KeyAgreeRecipientInfo;
-import org.bouncycastle.asn1.cms.KeyTransRecipientInfo;
-import org.bouncycastle.asn1.cms.PasswordRecipientInfo;
-import org.bouncycastle.asn1.cms.RecipientInfo;
-import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
-
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.security.AlgorithmParameters;
 import java.security.NoSuchProviderException;
 import java.security.Provider;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
+
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1OctetStringParser;
+import org.bouncycastle.asn1.ASN1SequenceParser;
+import org.bouncycastle.asn1.ASN1Set;
+import org.bouncycastle.asn1.ASN1SetParser;
+import org.bouncycastle.asn1.BERTags;
+import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.cms.AttributeTable;
+import org.bouncycastle.asn1.cms.EncryptedContentInfoParser;
+import org.bouncycastle.asn1.cms.EnvelopedDataParser;
+import org.bouncycastle.asn1.cms.OriginatorInfo;
+import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.cms.jcajce.JceAlgorithmIdentifierConverter;
 
 /**
  * Parsing class for an CMS Enveloped Data object from an input stream.
@@ -47,7 +42,7 @@ import java.util.List;
  *      {
  *          RecipientInformation   recipient = (RecipientInformation)it.next();
  *
- *          CMSTypedStream recData = recipient.getContentStream(privateKey, "BC");
+ *          CMSTypedStream recData = recipient.getContentStream(new JceKeyTransEnvelopedRecipient(privateKey).setProvider("BC"));
  *          
  *          processDataStream(recData.getContentStream());
  *      }
@@ -62,12 +57,13 @@ import java.util.List;
 public class CMSEnvelopedDataParser
     extends CMSContentInfoParser
 {
-    RecipientInformationStore   _recipientInfoStore;
-    EnvelopedDataParser         _envelopedData;
+    RecipientInformationStore recipientInfoStore;
+    EnvelopedDataParser envelopedData;
     
-    private AlgorithmIdentifier _encAlg;
-    private AttributeTable      _unprotectedAttributes;
-    private boolean             _attrNotRead;
+    private AlgorithmIdentifier encAlg;
+    private AttributeTable unprotectedAttributes;
+    private boolean attrNotRead;
+    private OriginatorInformation  originatorInfo;
 
     public CMSEnvelopedDataParser(
         byte[]    envelopedData) 
@@ -82,74 +78,47 @@ public class CMSEnvelopedDataParser
     {
         super(envelopedData);
 
-        this._attrNotRead = true;
-        this._envelopedData = new EnvelopedDataParser((ASN1SequenceParser)_contentInfo.getContent(DERTags.SEQUENCE));
+        this.attrNotRead = true;
+        this.envelopedData = new EnvelopedDataParser((ASN1SequenceParser)_contentInfo.getContent(BERTags.SEQUENCE));
 
         // TODO Validate version?
-		//DERInteger version = this._envelopedData.getVersion();
+        //DERInteger version = this._envelopedData.getVersion();
 
-        //
-        // load the RecipientInfoStore
-        //
-        ASN1SetParser s = _envelopedData.getRecipientInfos();
-        List          baseInfos = new ArrayList();
+        OriginatorInfo info = this.envelopedData.getOriginatorInfo();
 
-        DEREncodable entry;
-        while ((entry = s.readObject()) != null)
+        if (info != null)
         {
-            baseInfos.add(RecipientInfo.getInstance(entry.getDERObject()));
+            this.originatorInfo = new OriginatorInformation(info);
         }
 
+        //
+        // read the recipients
+        //
+        ASN1Set recipientInfos = ASN1Set.getInstance(this.envelopedData.getRecipientInfos().toASN1Primitive());
+
         //
         // read the encrypted content info
         //
-        EncryptedContentInfoParser encInfo = _envelopedData.getEncryptedContentInfo();
-        
-        this._encAlg = encInfo.getContentEncryptionAlgorithm();
-        
-        //
-        // prime the recipients
-        //
-        List        infos = new ArrayList();
-        Iterator    it = baseInfos.iterator();
-        InputStream dataStream = ((ASN1OctetStringParser)encInfo.getEncryptedContent(DERTags.OCTET_STRING)).getOctetStream();
-        
-        while (it.hasNext())
-        {
-            RecipientInfo   info = (RecipientInfo)it.next();
-            DEREncodable    recipInfo = info.getInfo();    
+        EncryptedContentInfoParser encInfo = this.envelopedData.getEncryptedContentInfo();
+        this.encAlg = encInfo.getContentEncryptionAlgorithm();
+        CMSReadable readable = new CMSProcessableInputStream(
+            ((ASN1OctetStringParser)encInfo.getEncryptedContent(BERTags.OCTET_STRING)).getOctetStream());
+        CMSSecureReadable secureReadable = new CMSEnvelopedHelper.CMSEnvelopedSecureReadable(
+            this.encAlg, readable);
 
-            if (recipInfo instanceof KeyTransRecipientInfo)
-            {
-                infos.add(new KeyTransRecipientInformation(
-                            (KeyTransRecipientInfo)recipInfo, _encAlg, dataStream));
-            }
-            else if (recipInfo instanceof KEKRecipientInfo)
-            {
-                infos.add(new KEKRecipientInformation(
-                            (KEKRecipientInfo)recipInfo, _encAlg, dataStream));
-            }
-            else if (recipInfo instanceof KeyAgreeRecipientInfo)
-            {
-                infos.add(new KeyAgreeRecipientInformation(
-                            (KeyAgreeRecipientInfo)recipInfo, _encAlg, dataStream));
-            }
-            else if (recipInfo instanceof PasswordRecipientInfo)
-            {
-                infos.add(new PasswordRecipientInformation(
-                            (PasswordRecipientInfo)recipInfo, _encAlg, dataStream));
-            }
-        }
-        
-        _recipientInfoStore = new RecipientInformationStore(infos);
+        //
+        // build the RecipientInformationStore
+        //
+        this.recipientInfoStore = CMSEnvelopedHelper.buildRecipientInformationStore(
+            recipientInfos, this.encAlg, secureReadable);
     }
-    
+
     /**
      * return the object identifier for the content encryption algorithm.
      */
     public String getEncryptionAlgOID()
     {
-        return _encAlg.getObjectId().toString();
+        return encAlg.getAlgorithm().toString();
     }
 
     /**
@@ -160,28 +129,22 @@ public class CMSEnvelopedDataParser
     {
         try
         {
-            return encodeObj(_encAlg.getParameters());
+            return encodeObj(encAlg.getParameters());
         }
         catch (Exception e)
         {
             throw new RuntimeException("exception getting encryption parameters " + e);
         }
     }
-    
+
     /**
-     * Return an AlgorithmParameters object giving the encryption parameters
-     * used to encrypt the message content.
-     * 
-     * @param provider the name of the provider to generate the parameters for.
-     * @return the parameters object, null if there is not one.
-     * @throws CMSException if the algorithm cannot be found, or the parameters can't be parsed.
-     * @throws NoSuchProviderException if the provider cannot be found.
+     * Return the content encryption algorithm details for the data in this object.
+     *
+     * @return AlgorithmIdentifier representing the content encryption algorithm.
      */
-    public AlgorithmParameters getEncryptionAlgorithmParameters(
-        String  provider) 
-        throws CMSException, NoSuchProviderException
+    public AlgorithmIdentifier getContentEncryptionAlgorithm()
     {
-        return getEncryptionAlgorithmParameters(CMSUtils.getProvider(provider));
+        return encAlg;
     }
 
     /**
@@ -191,12 +154,40 @@ public class CMSEnvelopedDataParser
      * @param provider the provider to generate the parameters for.
      * @return the parameters object, null if there is not one.
      * @throws CMSException if the algorithm cannot be found, or the parameters can't be parsed.
+     * @throws NoSuchProviderException if the provider cannot be found.
+     * @deprecated use getContentEncryptionAlgorithm and JceAlgorithmIdentifierConverter().
+     */
+    public AlgorithmParameters getEncryptionAlgorithmParameters(
+        String  provider)
+    throws CMSException, NoSuchProviderException
+    {
+        return new JceAlgorithmIdentifierConverter().setProvider(provider).getAlgorithmParameters(encAlg);
+    }
+
+    /**
+     * Return an AlgorithmParameters object giving the encryption parameters
+     * used to encrypt the message content.
+     *
+     * @param provider the provider to generate the parameters for.
+     * @return the parameters object, null if there is not one.
+     * @throws CMSException if the algorithm cannot be found, or the parameters can't be parsed.
+     * @deprecated use getContentEncryptionAlgorithm and JceAlgorithmIdentifierConverter().
      */
     public AlgorithmParameters getEncryptionAlgorithmParameters(
         Provider provider)
-        throws CMSException
+    throws CMSException
     {
-        return CMSEnvelopedHelper.INSTANCE.getEncryptionAlgorithmParameters(getEncryptionAlgOID(), getEncryptionAlgParams(), provider);
+        return new JceAlgorithmIdentifierConverter().setProvider(provider).getAlgorithmParameters(encAlg);
+    }
+
+    /**
+     * Return the originator information associated with this message if present.
+     *
+     * @return OriginatorInformation, null if not present.
+     */
+    public OriginatorInformation getOriginatorInfo()
+    {
+        return originatorInfo;
     }
 
     /**
@@ -204,7 +195,7 @@ public class CMSEnvelopedDataParser
      */
     public RecipientInformationStore getRecipientInfos()
     {
-        return _recipientInfoStore;
+        return recipientInfoStore;
     }
 
     /**
@@ -215,38 +206,38 @@ public class CMSEnvelopedDataParser
     public AttributeTable getUnprotectedAttributes() 
         throws IOException
     {
-        if (_unprotectedAttributes == null && _attrNotRead)
+        if (unprotectedAttributes == null && attrNotRead)
         {
-            ASN1SetParser             set = _envelopedData.getUnprotectedAttrs();
+            ASN1SetParser             set = envelopedData.getUnprotectedAttrs();
             
-            _attrNotRead = false;
+            attrNotRead = false;
             
             if (set != null)
             {
                 ASN1EncodableVector v = new ASN1EncodableVector();
-                DEREncodable        o;
+                ASN1Encodable        o;
                 
                 while ((o = set.readObject()) != null)
                 {
                     ASN1SequenceParser    seq = (ASN1SequenceParser)o;
                     
-                    v.add(seq.getDERObject());
+                    v.add(seq.toASN1Primitive());
                 }
                 
-                _unprotectedAttributes = new AttributeTable(new DERSet(v));
+                unprotectedAttributes = new AttributeTable(new DERSet(v));
             }
         }
 
-        return _unprotectedAttributes;
+        return unprotectedAttributes;
     }
 
     private byte[] encodeObj(
-        DEREncodable    obj)
+        ASN1Encodable obj)
         throws IOException
     {
         if (obj != null)
         {
-            return obj.getDERObject().getEncoded();
+            return obj.toASN1Primitive().getEncoded();
         }
 
         return null;
