@@ -1,53 +1,42 @@
 package org.bouncycastle.cms;
 
-import org.bouncycastle.asn1.ASN1Object;
-import org.bouncycastle.asn1.DEREncodable;
-import org.bouncycastle.asn1.DERNull;
-import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.asn1.DEROctetString;
-import org.bouncycastle.asn1.cms.KEKIdentifier;
-import org.bouncycastle.asn1.cms.OriginatorIdentifierOrKey;
-import org.bouncycastle.asn1.cms.OriginatorPublicKey;
-import org.bouncycastle.asn1.kisa.KISAObjectIdentifiers;
-import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
-import org.bouncycastle.asn1.ntt.NTTObjectIdentifiers;
-import org.bouncycastle.asn1.pkcs.PBKDF2Params;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
-import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
-import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
-
-import javax.crypto.KeyAgreement;
-import javax.crypto.SecretKey;
-import javax.crypto.spec.RC2ParameterSpec;
-import javax.crypto.spec.SecretKeySpec;
 import java.io.IOException;
-import java.security.AlgorithmParameterGenerator;
 import java.security.AlgorithmParameters;
-import java.security.InvalidAlgorithmParameterException;
 import java.security.InvalidKeyException;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
 import java.security.PrivateKey;
+import java.security.Provider;
 import java.security.PublicKey;
 import java.security.SecureRandom;
-import java.security.Provider;
+import java.security.cert.CertificateEncodingException;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Iterator;
 import java.util.List;
 
+import javax.crypto.SecretKey;
+
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.DERNull;
+import org.bouncycastle.asn1.cms.KEKIdentifier;
+import org.bouncycastle.asn1.cms.OriginatorInfo;
+import org.bouncycastle.asn1.kisa.KISAObjectIdentifiers;
+import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
+import org.bouncycastle.asn1.ntt.NTTObjectIdentifiers;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
+import org.bouncycastle.cms.jcajce.JceKEKRecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JceKeyTransRecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JcePasswordRecipientInfoGenerator;
+
 /**
  * General class for generating a CMS enveloped-data message.
- *
- * A simple example of usage.
- *
- * <pre>
- *      CMSEnvelopedDataGenerator  fact = new CMSEnvelopedDataGenerator();
- *
- *      fact.addKeyTransRecipient(cert);
- *
- *      CMSEnvelopedData         data = fact.generate(content, algorithm, "BC");
- * </pre>
  */
 public class CMSEnvelopedGenerator
 {
@@ -73,10 +62,15 @@ public class CMSEnvelopedGenerator
     public static final String  SEED_WRAP       = KISAObjectIdentifiers.id_npki_app_cmsSeed_wrap.getId();
 
     public static final String  ECDH_SHA1KDF    = X9ObjectIdentifiers.dhSinglePass_stdDH_sha1kdf_scheme.getId();
-//    public static final String  ECMQV_SHA1KDF   = X9ObjectIdentifiers.mqvSinglePass_sha1kdf_scheme.getId();
+    public static final String  ECMQV_SHA1KDF   = X9ObjectIdentifiers.mqvSinglePass_sha1kdf_scheme.getId();
 
+    final List oldRecipientInfoGenerators = new ArrayList();
     final List recipientInfoGenerators = new ArrayList();
+
+    protected CMSAttributeTableGenerator unprotectedAttributeGenerator = null;
+
     final SecureRandom rand;
+    protected OriginatorInfo originatorInfo;
 
     /**
      * base constructor
@@ -96,9 +90,21 @@ public class CMSEnvelopedGenerator
         this.rand = rand;
     }
 
+    public void setUnprotectedAttributeGenerator(CMSAttributeTableGenerator unprotectedAttributeGenerator)
+    {
+        this.unprotectedAttributeGenerator = unprotectedAttributeGenerator;
+    }
+
+
+    public void setOriginatorInfo(OriginatorInformation originatorInfo)
+    {
+        this.originatorInfo = originatorInfo.toASN1Structure();
+    }
+
     /**
      * add a recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyTransRecipientInfoGenerator
      * @param cert recipient's public key certificate
      * @exception IllegalArgumentException if there is a problem with the certificate
      */
@@ -106,15 +112,20 @@ public class CMSEnvelopedGenerator
         X509Certificate cert)
         throws IllegalArgumentException
     {
-        KeyTransRecipientInfoGenerator ktrig = new KeyTransRecipientInfoGenerator();
-        ktrig.setRecipientCert(cert);
-
-        recipientInfoGenerators.add(ktrig);
+        try
+        {
+            oldRecipientInfoGenerators.add(new JceKeyTransRecipientInfoGenerator(cert));
+        }
+        catch (CertificateEncodingException e)
+        {
+            throw new IllegalArgumentException("unable to encode certificate: " + e.getMessage());
+        }
     }
 
     /**
      * add a recipient
      *
+     * @deprecated use the addRecipientGenerator and JceKeyTransRecipientInfoGenerator
      * @param key the public key used by the recipient
      * @param subKeyId the identifier for the recipient's public key
      * @exception IllegalArgumentException if there is a problem with the key
@@ -124,15 +135,13 @@ public class CMSEnvelopedGenerator
         byte[]      subKeyId)
         throws IllegalArgumentException
     {
-        KeyTransRecipientInfoGenerator ktrig = new KeyTransRecipientInfoGenerator();
-        ktrig.setRecipientPublicKey(key);
-        ktrig.setSubjectKeyIdentifier(new DEROctetString(subKeyId));
-
-        recipientInfoGenerators.add(ktrig);
+        oldRecipientInfoGenerators.add(new JceKeyTransRecipientInfoGenerator(subKeyId, key));
     }
 
     /**
      * add a KEK recipient.
+     *
+     * @deprecated use the addRecipientGenerator and JceKEKRecipientInfoGenerator
      * @param key the secret key to use for wrapping
      * @param keyIdentifier the byte string that identifies the key
      */
@@ -140,29 +149,41 @@ public class CMSEnvelopedGenerator
         SecretKey   key,
         byte[]      keyIdentifier)
     {
-        KEKRecipientInfoGenerator kekrig = new KEKRecipientInfoGenerator();
-        kekrig.setKEKIdentifier(new KEKIdentifier(keyIdentifier, null, null));
-        kekrig.setWrapKey(key);
-
-        recipientInfoGenerators.add(kekrig);
+        addKEKRecipient(key, new KEKIdentifier(keyIdentifier, null, null));
     }
 
+    /**
+     * add a KEK recipient.
+     *
+     * @deprecated use the addRecipientGenerator and JceKEKRecipientInfoGenerator
+     * @param key the secret key to use for wrapping
+     * @param kekIdentifier a KEKIdentifier structure (identifies the key)
+     */
+    public void addKEKRecipient(
+        SecretKey       key,
+        KEKIdentifier   kekIdentifier)
+    {
+        oldRecipientInfoGenerators.add(new JceKEKRecipientInfoGenerator(kekIdentifier, key));
+    }
+
+    /**
+     * @deprecated use addRecipientGenerator and JcePasswordRecipientInfoGenerator
+     * @param pbeKey PBE key
+     * @param kekAlgorithmOid key encryption algorithm to use.
+     */
     public void addPasswordRecipient(
         CMSPBEKey pbeKey,
         String    kekAlgorithmOid)
     {
-        PBKDF2Params params = new PBKDF2Params(pbeKey.getSalt(), pbeKey.getIterationCount());
-
-        PasswordRecipientInfoGenerator prig = new PasswordRecipientInfoGenerator();
-        prig.setDerivationAlg(new AlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, params));
-        prig.setWrapKey(new SecretKeySpec(pbeKey.getEncoded(kekAlgorithmOid), kekAlgorithmOid));
-
-        recipientInfoGenerators.add(prig);
+        oldRecipientInfoGenerators.add(new JcePasswordRecipientInfoGenerator(new ASN1ObjectIdentifier(kekAlgorithmOid), pbeKey.getPassword())
+            .setSaltAndIterationCount(pbeKey.getSalt(), pbeKey.getIterationCount())
+            .setPasswordConversionScheme((pbeKey instanceof PKCS5Scheme2UTF8PBEKey) ? PasswordRecipient.PKCS5_SCHEME2_UTF8 : PasswordRecipient.PKCS5_SCHEME2));
     }
 
     /**
      * Add a key agreement based recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
      * @param agreementAlgorithm key agreement algorithm to use.
      * @param senderPrivateKey private key to initialise sender side of agreement with.
      * @param senderPublicKey sender public key to include with message.
@@ -188,6 +209,7 @@ public class CMSEnvelopedGenerator
     /**
      * Add a key agreement based recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
      * @param agreementAlgorithm key agreement algorithm to use.
      * @param senderPrivateKey private key to initialise sender side of agreement with.
      * @param senderPublicKey sender public key to include with message.
@@ -206,43 +228,94 @@ public class CMSEnvelopedGenerator
         Provider         provider)
         throws NoSuchAlgorithmException, InvalidKeyException
     {
-        KeyAgreement agreement = KeyAgreement.getInstance(agreementAlgorithm, provider);
+        List recipients = new ArrayList();
 
-        agreement.init(senderPrivateKey, rand);
+        recipients.add(recipientCert);
 
-        agreement.doPhase(recipientCert.getPublicKey(), true);
+        addKeyAgreementRecipients(agreementAlgorithm, senderPrivateKey, senderPublicKey,
+            recipients, cekWrapAlgorithm, provider);
+    }
 
-        try
+    /**
+     * Add multiple key agreement based recipients (sharing a single KeyAgreeRecipientInfo structure).
+     *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
+     * @param agreementAlgorithm key agreement algorithm to use.
+     * @param senderPrivateKey private key to initialise sender side of agreement with.
+     * @param senderPublicKey sender public key to include with message.
+     * @param recipientCerts recipients' public key certificates.
+     * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
+     * @param provider provider to use for the agreement calculation.
+     * @exception NoSuchAlgorithmException if the algorithm requested cannot be found
+     * @exception InvalidKeyException if the keys are inappropriate for the algorithm specified
+     */
+    public void addKeyAgreementRecipients(
+        String           agreementAlgorithm,
+        PrivateKey       senderPrivateKey,
+        PublicKey        senderPublicKey,
+        Collection       recipientCerts,
+        String           cekWrapAlgorithm,
+        String           provider)
+        throws NoSuchProviderException, NoSuchAlgorithmException, InvalidKeyException
+    {
+        addKeyAgreementRecipients(agreementAlgorithm, senderPrivateKey, senderPublicKey, recipientCerts, cekWrapAlgorithm, CMSUtils.getProvider(provider));
+    }
+
+    /**
+     * Add multiple key agreement based recipients (sharing a single KeyAgreeRecipientInfo structure).
+     *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
+     * @param agreementAlgorithm key agreement algorithm to use.
+     * @param senderPrivateKey private key to initialise sender side of agreement with.
+     * @param senderPublicKey sender public key to include with message.
+     * @param recipientCerts recipients' public key certificates.
+     * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
+     * @param provider provider to use for the agreement calculation.
+     * @exception NoSuchAlgorithmException if the algorithm requested cannot be found
+     * @exception InvalidKeyException if the keys are inappropriate for the algorithm specified
+     */
+    public void addKeyAgreementRecipients(
+        String           agreementAlgorithm,
+        PrivateKey       senderPrivateKey,
+        PublicKey        senderPublicKey,
+        Collection       recipientCerts,
+        String           cekWrapAlgorithm,
+        Provider         provider)
+        throws NoSuchAlgorithmException, InvalidKeyException
+    {
+        JceKeyAgreeRecipientInfoGenerator recipientInfoGenerator = new JceKeyAgreeRecipientInfoGenerator(new ASN1ObjectIdentifier(agreementAlgorithm), senderPrivateKey, senderPublicKey, new ASN1ObjectIdentifier(cekWrapAlgorithm)).setProvider(provider);
+
+        for (Iterator it = recipientCerts.iterator(); it.hasNext();)
         {
-            SubjectPublicKeyInfo oPubKeyInfo = SubjectPublicKeyInfo.getInstance(ASN1Object.fromByteArray(senderPublicKey.getEncoded()));
-            OriginatorIdentifierOrKey originator = new OriginatorIdentifierOrKey(
-                                                       new OriginatorPublicKey(
-                                                            new AlgorithmIdentifier(oPubKeyInfo.getAlgorithmId().getObjectId(), new DERNull()),
-                                                            oPubKeyInfo.getPublicKeyData().getBytes()));
-
-            SecretKey wrapKey = agreement.generateSecret(cekWrapAlgorithm);
-
-            KeyAgreeRecipientInfoGenerator karig = new KeyAgreeRecipientInfoGenerator();
-            karig.setAlgorithmOID(new DERObjectIdentifier(agreementAlgorithm));
-            karig.setOriginator(originator);
-            karig.setRecipientCert(recipientCert);
-            karig.setWrapKey(wrapKey);
-            karig.setWrapAlgorithmOID(new DERObjectIdentifier(cekWrapAlgorithm));
-
-            recipientInfoGenerators.add(karig);
-        }
-        catch (IOException e)
-        {
-            throw new InvalidKeyException("cannot extract originator public key: " + e);
+            try
+            {
+                recipientInfoGenerator.addRecipient((X509Certificate)it.next());
+            }
+            catch (CertificateEncodingException e)
+            {
+                throw new IllegalArgumentException("unable to encode certificate: " + e.getMessage());
+            }
         }
+
+        oldRecipientInfoGenerators.add(recipientInfoGenerator);
+    }
+
+    /**
+     * Add a generator to produce the recipient info required.
+     * 
+     * @param recipientGenerator a generator of a recipient info object.
+     */
+    public void addRecipientInfoGenerator(RecipientInfoGenerator recipientGenerator)
+    {
+        recipientInfoGenerators.add(recipientGenerator);
     }
 
     protected AlgorithmIdentifier getAlgorithmIdentifier(String encryptionOID, AlgorithmParameters params) throws IOException
     {
-        DEREncodable asn1Params;
+        ASN1Encodable asn1Params;
         if (params != null)
         {
-            asn1Params = ASN1Object.fromByteArray(params.getEncoded("ASN.1"));
+            asn1Params = ASN1Primitive.fromByteArray(params.getEncoded("ASN.1"));
         }
         else
         {
@@ -250,38 +323,68 @@ public class CMSEnvelopedGenerator
         }
 
         return new AlgorithmIdentifier(
-            new DERObjectIdentifier(encryptionOID),
+            new ASN1ObjectIdentifier(encryptionOID),
             asn1Params);
     }
 
-    protected AlgorithmParameters generateParameters(String encryptionOID, SecretKey encKey, Provider encProvider)
-        throws CMSException
+    protected void convertOldRecipients(SecureRandom rand, Provider provider)
     {
-        try
+        for (Iterator it = oldRecipientInfoGenerators.iterator(); it.hasNext();)
         {
-            AlgorithmParameterGenerator pGen = AlgorithmParameterGenerator.getInstance(encryptionOID, encProvider);
+            Object recipient = it.next();
 
-            if (encryptionOID.equals(RC2_CBC))
+            if (recipient instanceof JceKeyTransRecipientInfoGenerator)
             {
-                byte[]  iv = new byte[8];
+                JceKeyTransRecipientInfoGenerator recip = (JceKeyTransRecipientInfoGenerator)recipient;
 
-                rand.nextBytes(iv);
+                if (provider != null)
+                {
+                    recip.setProvider(provider);
+                }
 
-                try
-                {
-                    pGen.init(new RC2ParameterSpec(encKey.getEncoded().length * 8, iv), rand);
-                }
-                catch (InvalidAlgorithmParameterException e)
-                {
-                    throw new CMSException("parameters generation error: " + e, e);
-                }
+                recipientInfoGenerators.add(recip);
             }
+            else if (recipient instanceof KEKRecipientInfoGenerator)
+            {
+                JceKEKRecipientInfoGenerator recip = (JceKEKRecipientInfoGenerator)recipient;
 
-            return pGen.generateParameters();
-        }
-        catch (NoSuchAlgorithmException e)
-        {
-            return null;
+                if (provider != null)
+                {
+                    recip.setProvider(provider);
+                }
+
+                recip.setSecureRandom(rand);
+
+                recipientInfoGenerators.add(recip);
+            }
+            else if (recipient instanceof JcePasswordRecipientInfoGenerator)
+            {
+                JcePasswordRecipientInfoGenerator recip = (JcePasswordRecipientInfoGenerator)recipient;
+
+                if (provider != null)
+                {
+                    recip.setProvider(provider);
+                }
+
+                recip.setSecureRandom(rand);
+
+                recipientInfoGenerators.add(recip);
+            }
+            else if (recipient instanceof JceKeyAgreeRecipientInfoGenerator)
+            {
+                JceKeyAgreeRecipientInfoGenerator recip = (JceKeyAgreeRecipientInfoGenerator)recipient;
+
+                if (provider != null)
+                {
+                    recip.setProvider(provider);
+                }
+
+                recip.setSecureRandom(rand);
+
+                recipientInfoGenerators.add(recip);
+            }
         }
+
+        oldRecipientInfoGenerators.clear();
     }
 }
