@@ -1,42 +1,12 @@
 package org.bouncycastle.cms;
 
-import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.asn1.ASN1Generator;
-import org.bouncycastle.asn1.ASN1InputStream;
-import org.bouncycastle.asn1.ASN1OctetStringParser;
-import org.bouncycastle.asn1.ASN1SequenceParser;
-import org.bouncycastle.asn1.ASN1Set;
-import org.bouncycastle.asn1.ASN1SetParser;
-import org.bouncycastle.asn1.ASN1StreamParser;
-import org.bouncycastle.asn1.ASN1TaggedObject;
-import org.bouncycastle.asn1.BEROctetStringGenerator;
-import org.bouncycastle.asn1.BERSequenceGenerator;
-import org.bouncycastle.asn1.BERSetParser;
-import org.bouncycastle.asn1.BERTaggedObject;
-import org.bouncycastle.asn1.DEREncodable;
-import org.bouncycastle.asn1.DERNull;
-import org.bouncycastle.asn1.DERObject;
-import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.asn1.DERSet;
-import org.bouncycastle.asn1.DERTaggedObject;
-import org.bouncycastle.asn1.DERTags;
-import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
-import org.bouncycastle.asn1.cms.ContentInfoParser;
-import org.bouncycastle.asn1.cms.SignedDataParser;
-import org.bouncycastle.asn1.cms.SignerInfo;
-import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
-import org.bouncycastle.util.io.Streams;
-import org.bouncycastle.x509.NoSuchStoreException;
-import org.bouncycastle.x509.X509Store;
-
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.security.DigestInputStream;
-import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
+import java.security.Provider;
 import java.security.cert.CertStore;
 import java.security.cert.CertStoreException;
 import java.util.ArrayList;
@@ -45,6 +15,36 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1Generator;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1OctetStringParser;
+import org.bouncycastle.asn1.ASN1SequenceParser;
+import org.bouncycastle.asn1.ASN1Set;
+import org.bouncycastle.asn1.ASN1SetParser;
+import org.bouncycastle.asn1.ASN1StreamParser;
+import org.bouncycastle.asn1.BERSequenceGenerator;
+import org.bouncycastle.asn1.BERSetParser;
+import org.bouncycastle.asn1.BERTaggedObject;
+import org.bouncycastle.asn1.BERTags;
+import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.DERTaggedObject;
+import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
+import org.bouncycastle.asn1.cms.ContentInfoParser;
+import org.bouncycastle.asn1.cms.SignedDataParser;
+import org.bouncycastle.asn1.cms.SignerInfo;
+import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.cert.jcajce.JcaCertStoreBuilder;
+import org.bouncycastle.operator.DigestCalculator;
+import org.bouncycastle.operator.DigestCalculatorProvider;
+import org.bouncycastle.operator.OperatorCreationException;
+import org.bouncycastle.operator.bc.BcDigestCalculatorProvider;
+import org.bouncycastle.util.Store;
+import org.bouncycastle.util.io.Streams;
+import org.bouncycastle.x509.NoSuchStoreException;
+import org.bouncycastle.x509.X509Store;
+
 /**
  * Parsing class for an CMS Signed Data object from an input stream.
  * <p>
@@ -61,11 +61,11 @@ import java.util.Map;
  * mode the order of the operations is important.
  * </p>
  * <pre>
- *      CMSSignedDataParser     sp = new CMSSignedDataParser(encapSigData);
+ *      CMSSignedDataParser     sp = new CMSSignedDataParser(new JcaDigestCalculatorProviderBuilder().setProvider("BC").build(), encapSigData);
  *
  *      sp.getSignedContent().drain();
  *
- *      CertStore               certs = sp.getCertificatesAndCRLs("Collection", "BC");
+ *      Store                   certStore = sp.getCertificates();
  *      SignerInformationStore  signers = sp.getSignerInfos();
  *      
  *      Collection              c = signers.getSigners();
@@ -74,12 +74,12 @@ import java.util.Map;
  *      while (it.hasNext())
  *      {
  *          SignerInformation   signer = (SignerInformation)it.next();
- *          Collection          certCollection = certs.getCertificates(signer.getSID());
+ *          Collection          certCollection = certStore.getMatches(signer.getSID());
  *
  *          Iterator        certIt = certCollection.iterator();
- *          X509Certificate cert = (X509Certificate)certIt.next();
+ *          X509CertificateHolder cert = (X509CertificateHolder)certIt.next();
  *
- *          System.out.println("verify returns: " + signer.verify(cert, "BC"));
+ *          System.out.println("verify returns: " + signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert)));
  *      }
  * </pre>
  *  Note also: this class does not introduce buffering - if you are processing large files you should create
@@ -92,13 +92,13 @@ import java.util.Map;
 public class CMSSignedDataParser
     extends CMSContentInfoParser
 {
-    private static CMSSignedHelper HELPER = CMSSignedHelper.INSTANCE;
+    private static final CMSSignedHelper HELPER = CMSSignedHelper.INSTANCE;
 
     private SignedDataParser        _signedData;
+    private ASN1ObjectIdentifier    _signedContentType;
     private CMSTypedStream          _signedContent;
-    private Map                     _digests;
-    
-    private CertStore               _certStore;
+    private Map                     digests;
+
     private SignerInformationStore  _signerInfoStore;
     private X509Store               _attributeStore;
     private ASN1Set                 _certSet, _crlSet;
@@ -106,29 +106,75 @@ public class CMSSignedDataParser
     private X509Store               _certificateStore;
     private X509Store               _crlStore;
 
+    /**
+     * @deprecated use method taking a DigestCalculatorProvider
+     */
     public CMSSignedDataParser(
         byte[]      sigBlock)
         throws CMSException
     {
-        this(new ByteArrayInputStream(sigBlock));
+        this(createDefaultDigestProvider(), new ByteArrayInputStream(sigBlock));
     }
 
+
+    public CMSSignedDataParser(
+        DigestCalculatorProvider digestCalculatorProvider,
+        byte[]      sigBlock)
+        throws CMSException
+    {
+        this(digestCalculatorProvider, new ByteArrayInputStream(sigBlock));
+    }
+
+    /**
+     * @deprecated use method taking digest calculator provider.
+     * @param signedContent
+     * @param sigBlock
+     * @throws CMSException
+     */
     public CMSSignedDataParser(
         CMSTypedStream  signedContent,
         byte[]          sigBlock)
         throws CMSException
     {
-        this(signedContent, new ByteArrayInputStream(sigBlock));
+        this(createDefaultDigestProvider(), signedContent, new ByteArrayInputStream(sigBlock));
+    }
+
+    public CMSSignedDataParser(
+        DigestCalculatorProvider digestCalculatorProvider,
+        CMSTypedStream  signedContent,
+        byte[]          sigBlock)
+        throws CMSException
+    {
+        this(digestCalculatorProvider, signedContent, new ByteArrayInputStream(sigBlock));
+    }
+
+    private static DigestCalculatorProvider createDefaultDigestProvider()
+        throws CMSException
+    {
+        return new BcDigestCalculatorProvider();
     }
 
     /**
      * base constructor - with encapsulated content
+     *
+     * @deprecated use method taking a DigestCalculatorProvider
      */
     public CMSSignedDataParser(
         InputStream sigData)
         throws CMSException
     {
-        this(null, sigData);
+        this(createDefaultDigestProvider(), null, sigData);
+    }
+
+     /**
+     * base constructor - with encapsulated content
+     */
+    public CMSSignedDataParser(
+        DigestCalculatorProvider digestCalculatorProvider,
+        InputStream sigData)
+        throws CMSException
+    {
+        this(digestCalculatorProvider, null, sigData);
     }
 
     /**
@@ -136,38 +182,54 @@ public class CMSSignedDataParser
      *
      * @param signedContent the content that was signed.
      * @param sigData the signature object stream.
+     *      *
+     * @deprecated use method taking a DigestCalculatorProvider
      */
     public CMSSignedDataParser(
         CMSTypedStream  signedContent,
         InputStream     sigData) 
         throws CMSException
+    {
+        this(createDefaultDigestProvider(), signedContent, sigData);
+    }
+
+    /**
+     * base constructor
+     *
+     * @param digestCalculatorProvider for generating accumulating digests
+     * @param signedContent the content that was signed.
+     * @param sigData the signature object stream.
+     */
+    public CMSSignedDataParser(
+        DigestCalculatorProvider digestCalculatorProvider,
+        CMSTypedStream  signedContent,
+        InputStream     sigData)
+        throws CMSException
     {
         super(sigData);
         
         try
         {
             _signedContent = signedContent;
-            _signedData = SignedDataParser.getInstance(_contentInfo.getContent(DERTags.SEQUENCE));
-            _digests = new HashMap();
+            _signedData = SignedDataParser.getInstance(_contentInfo.getContent(BERTags.SEQUENCE));
+            digests = new HashMap();
             
             ASN1SetParser digAlgs = _signedData.getDigestAlgorithms();
-            DEREncodable  o;
+            ASN1Encodable  o;
             
             while ((o = digAlgs.readObject()) != null)
             {
-                AlgorithmIdentifier id = AlgorithmIdentifier.getInstance(o.getDERObject());
+                AlgorithmIdentifier algId = AlgorithmIdentifier.getInstance(o);
                 try
                 {
-                    String        digestName = CMSSignedHelper.INSTANCE.getDigestAlgName(id.getObjectId().toString());
-                    MessageDigest dig = CMSSignedHelper.INSTANCE.getDigestInstance(digestName, null);
+                    DigestCalculator calculator = digestCalculatorProvider.get(algId);
 
-                    this._digests.put(digestName, dig);
+                    if (calculator != null)
+                    {
+                        this.digests.put(algId.getAlgorithm(), calculator);
+                    }
                 }
-                catch (NoSuchAlgorithmException e)
-                {
-                     //  ignore
-                }
-                catch (NoSuchProviderException e)
+                catch (OperatorCreationException e)
                 {
                      //  ignore
                 }
@@ -178,7 +240,7 @@ public class CMSSignedDataParser
             //
             ContentInfoParser     cont = _signedData.getEncapContentInfo();
             ASN1OctetStringParser octs = (ASN1OctetStringParser)
-                cont.getContent(DERTags.OCTET_STRING);
+                cont.getContent(BERTags.OCTET_STRING);
 
             if (octs != null)
             {
@@ -197,13 +259,22 @@ public class CMSSignedDataParser
                     ctStr.drain();
                 }
             }
+
+            if (signedContent == null)
+            {
+                _signedContentType = cont.getContentType();
+            }
+            else
+            {
+                _signedContentType = _signedContent.getContentType();
+            }
         }
         catch (IOException e)
         {
             throw new CMSException("io exception: " + e.getMessage(), e);
         }
         
-        if (_digests.isEmpty())
+        if (digests.isEmpty())
         {
             throw new CMSException("no digests could be created for message.");
         }
@@ -234,27 +305,26 @@ public class CMSSignedDataParser
             List      signerInfos = new ArrayList();
             Map       hashes = new HashMap();
             
-            Iterator  it = _digests.keySet().iterator();
+            Iterator  it = digests.keySet().iterator();
             while (it.hasNext())
             {
                 Object digestKey = it.next();
-                
-                hashes.put(digestKey, ((MessageDigest)_digests.get(digestKey)).digest());
+
+                hashes.put(digestKey, ((DigestCalculator)digests.get(digestKey)).getDigest());
             }
             
             try
             {
                 ASN1SetParser     s = _signedData.getSignerInfos();
-                DEREncodable      o;
-                
+                ASN1Encodable      o;
+
                 while ((o = s.readObject()) != null)
                 {
-                    SignerInfo info = SignerInfo.getInstance(o.getDERObject());
-                    String     digestName = CMSSignedHelper.INSTANCE.getDigestAlgName(info.getDigestAlgorithm().getObjectId().getId());
-                    
-                    byte[] hash = (byte[])hashes.get(digestName);
-                    
-                    signerInfos.add(new SignerInformation(info, new DERObjectIdentifier(_signedContent.getContentType()), null, new BaseDigestCalculator(hash)));
+                    SignerInfo info = SignerInfo.getInstance(o.toASN1Primitive());
+
+                    byte[] hash = (byte[])hashes.get(info.getDigestAlgorithm().getAlgorithm());
+
+                    signerInfos.add(new SignerInformation(info, _signedContentType, null, hash));
                 }
             }
             catch (IOException e)
@@ -273,28 +343,48 @@ public class CMSSignedDataParser
      * in this message.
      *
      * @param type type of store to create
-     * @param provider provider to use
+     * @param provider name of provider to use
      * @return a store of attribute certificates
      * @exception NoSuchProviderException if the provider requested isn't available.
      * @exception org.bouncycastle.x509.NoSuchStoreException if the store type isn't available.
      * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getAttributeCertificates()
      */
     public X509Store getAttributeCertificates(
         String type,
         String provider)
         throws NoSuchStoreException, NoSuchProviderException, CMSException
+    {
+        return getAttributeCertificates(type, CMSUtils.getProvider(provider));
+    }
+
+    /**
+     * return a X509Store containing the attribute certificates, if any, contained
+     * in this message.
+     *
+     * @param type type of store to create
+     * @param provider provider to use
+     * @return a store of attribute certificates
+     * @exception org.bouncycastle.x509.NoSuchStoreException if the store type isn't available.
+     * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getAttributeCertificates()
+     */
+    public X509Store getAttributeCertificates(
+        String type,
+        Provider provider)
+        throws NoSuchStoreException, CMSException
     {
         if (_attributeStore == null)
         {
             populateCertCrlSets();
 
-            _attributeStore = HELPER.createAttributeStore(type, provider, _certSet);
+            _attributeStore = HELPER.createAttributeStore(type, provider, this.getAttributeCertificates());
         }
 
         return _attributeStore;
     }
 
-        /**
+    /**
      * return a X509Store containing the public key certificates, if any, contained
      * in this message.
      *
@@ -304,17 +394,37 @@ public class CMSSignedDataParser
      * @exception NoSuchProviderException if the provider requested isn't available.
      * @exception NoSuchStoreException if the store type isn't available.
      * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getCertificates()
      */
     public X509Store getCertificates(
         String type,
         String provider)
         throws NoSuchStoreException, NoSuchProviderException, CMSException
+    {
+        return getCertificates(type, CMSUtils.getProvider(provider));
+    }
+
+    /**
+     * return a X509Store containing the public key certificates, if any, contained
+     * in this message.
+     *
+     * @param type type of store to create
+     * @param provider provider to use
+     * @return a store of public key certificates
+     * @exception NoSuchStoreException if the store type isn't available.
+     * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getCertificates()
+     */
+    public X509Store getCertificates(
+        String type,
+        Provider provider)
+        throws NoSuchStoreException, CMSException
     {
         if (_certificateStore == null)
         {
             populateCertCrlSets();
 
-            _certificateStore = HELPER.createCertificateStore(type, provider, _certSet);
+            _certificateStore = HELPER.createCertificateStore(type, provider, this.getCertificates());
         }
 
         return _certificateStore;
@@ -325,22 +435,42 @@ public class CMSSignedDataParser
      * in this message.
      *
      * @param type type of store to create
-     * @param provider provider to use
+     * @param provider name of provider to use
      * @return a store of CRLs
      * @exception NoSuchProviderException if the provider requested isn't available.
      * @exception NoSuchStoreException if the store type isn't available.
      * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getCRLs()
      */
     public X509Store getCRLs(
         String type,
         String provider)
         throws NoSuchStoreException, NoSuchProviderException, CMSException
+    {
+        return getCRLs(type, CMSUtils.getProvider(provider));
+    }
+
+    /**
+     * return a X509Store containing CRLs, if any, contained
+     * in this message.
+     *
+     * @param type type of store to create
+     * @param provider provider to use
+     * @return a store of CRLs
+     * @exception NoSuchStoreException if the store type isn't available.
+     * @exception CMSException if a general exception prevents creation of the X509Store
+     * @deprecated use getCRLs()
+     */
+    public X509Store getCRLs(
+        String type,
+        Provider provider)
+        throws NoSuchStoreException, CMSException
     {
         if (_crlStore == null)
         {
             populateCertCrlSets();
 
-            _crlStore = HELPER.createCRLsStore(type, provider, _crlSet);
+            _crlStore = HELPER.createCRLsStore(type, provider, getCRLs());
         }
 
         return _crlStore;
@@ -353,20 +483,109 @@ public class CMSSignedDataParser
      * @exception NoSuchProviderException if the provider requested isn't available.
      * @exception NoSuchAlgorithmException if the cert store isn't available.
      * @exception CMSException if a general exception prevents creation of the CertStore
+     * @deprecated use getCertificates() and org.bouncycastle.cert.jcajce.JcaCertStoreBuilder
      */
     public CertStore getCertificatesAndCRLs(
         String  type,
         String  provider)
         throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
     {
-        if (_certStore == null)
-        {
-            populateCertCrlSets();
-            
-            _certStore = HELPER.createCertStore(type, provider, _certSet, _crlSet);
-        }
+        return getCertificatesAndCRLs(type, CMSUtils.getProvider(provider));
+    }
 
-        return _certStore;
+    /**
+     * return a CertStore containing the certificates and CRLs associated with
+     * this message.
+     *
+     * @exception NoSuchProviderException if the provider requested isn't available.
+     * @exception NoSuchAlgorithmException if the cert store isn't available.
+     * @exception CMSException if a general exception prevents creation of the CertStore
+     * @deprecated use getCertificates() and org.bouncycastle.cert.jcajce.JcaCertStoreBuilder
+     */
+    public CertStore getCertificatesAndCRLs(
+        String  type,
+        Provider  provider)
+        throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
+    {
+        populateCertCrlSets();
+
+        try
+        {
+            JcaCertStoreBuilder certStoreBuilder = new JcaCertStoreBuilder().setType(type);
+
+            if (provider != null)
+            {
+                certStoreBuilder.setProvider(provider);
+            }
+
+            certStoreBuilder.addCertificates(this.getCertificates());
+            certStoreBuilder.addCRLs(this.getCRLs());
+
+            return certStoreBuilder.build();
+        }
+        catch (NoSuchAlgorithmException e)
+        {
+            throw e;
+        }
+        catch (Exception e)
+        {
+            throw new CMSException("exception creating CertStore: " + e.getMessage(), e);
+        }
+    }
+
+    /**
+     * Return any X.509 certificate objects in this SignedData structure as a Store of X509CertificateHolder objects.
+     *
+     * @return a Store of X509CertificateHolder objects.
+     */
+    public Store getCertificates()
+        throws CMSException
+    {
+        populateCertCrlSets();
+
+        return HELPER.getCertificates(_certSet);
+    }
+
+    /**
+     * Return any X.509 CRL objects in this SignedData structure as a Store of X509CRLHolder objects.
+     *
+     * @return a Store of X509CRLHolder objects.
+     */
+    public Store getCRLs()
+        throws CMSException
+    {
+        populateCertCrlSets();
+
+        return HELPER.getCRLs(_crlSet);
+    }
+
+    /**
+     * Return any X.509 attribute certificate objects in this SignedData structure as a Store of X509AttributeCertificateHolder objects.
+     *
+     * @return a Store of X509AttributeCertificateHolder objects.
+     */
+    public Store getAttributeCertificates()
+        throws CMSException
+    {
+        populateCertCrlSets();
+
+        return HELPER.getAttributeCertificates(_certSet);
+    }
+
+    /**
+     * Return any OtherRevocationInfo OtherRevInfo objects of the type indicated by otherRevocationInfoFormat in
+     * this SignedData structure.
+     *
+     * @param otherRevocationInfoFormat OID of the format type been looked for.
+     *
+     * @return a Store of ASN1Encodable objects representing any objects of otherRevocationInfoFormat found.
+     */
+    public Store getOtherRevocationInfo(ASN1ObjectIdentifier otherRevocationInfoFormat)
+        throws CMSException
+    {
+        populateCertCrlSets();
+
+        return HELPER.getOtherRevocationInfo(otherRevocationInfoFormat, _crlSet);
     }
 
     private void populateCertCrlSets()
@@ -391,27 +610,29 @@ public class CMSSignedDataParser
         }
     }
 
+    /**
+     * Return the a string representation of the OID associated with the
+     * encapsulated content info structure carried in the signed data.
+     * 
+     * @return the OID for the content type.
+     */
+    public String getSignedContentTypeOID()
+    {
+        return _signedContentType.getId();
+    }
+
     public CMSTypedStream getSignedContent()
     {
-        if (_signedContent != null)
-        {
-            InputStream digStream = _signedContent.getContentStream();
-            
-            Iterator it = _digests.values().iterator();
-            
-            while (it.hasNext())
-            {
-                digStream = new DigestInputStream(digStream, (MessageDigest)it.next());
-            }
-            
-            return new CMSTypedStream(_signedContent.getContentType(), digStream);
-        }
-        else
+        if (_signedContent == null)
         {
             return null;
         }
-    }
 
+        InputStream digStream = CMSUtils.attachDigestsToInputStream(
+            digests.values(), _signedContent.getContentStream());
+
+        return new CMSTypedStream(_signedContent.getContentType(), digStream);
+    }
 
     /**
      * Replace the signerinformation store associated with the passed
@@ -432,9 +653,9 @@ public class CMSSignedDataParser
         OutputStream            out)
         throws CMSException, IOException
     {
-        ASN1StreamParser in = new ASN1StreamParser(original, CMSUtils.getMaximumMemory());
+        ASN1StreamParser in = new ASN1StreamParser(original);
         ContentInfoParser contentInfo = new ContentInfoParser((ASN1SequenceParser)in.readObject());
-        SignedDataParser signedData = SignedDataParser.getInstance(contentInfo.getContent(DERTags.SEQUENCE));
+        SignedDataParser signedData = SignedDataParser.getInstance(contentInfo.getContent(BERTags.SEQUENCE));
 
         BERSequenceGenerator sGen = new BERSequenceGenerator(out);
 
@@ -446,18 +667,14 @@ public class CMSSignedDataParser
         sigGen.addObject(signedData.getVersion());
 
         // digests
-        signedData.getDigestAlgorithms().getDERObject();  // skip old ones
+        signedData.getDigestAlgorithms().toASN1Primitive();  // skip old ones
 
         ASN1EncodableVector digestAlgs = new ASN1EncodableVector();
 
         for (Iterator it = signerInformationStore.getSigners().iterator(); it.hasNext();)
         {
-            SignerInformation        signer = (SignerInformation)it.next();
-            AlgorithmIdentifier     digAlgId;
-
-            digAlgId = makeAlgId(signer.getDigestAlgOID(), signer.getDigestAlgParams());
-
-            digestAlgs.add(digAlgId);
+            SignerInformation signer = (SignerInformation)it.next();
+            digestAlgs.add(CMSSignedHelper.INSTANCE.fixAlgID(signer.getDigestAlgorithmID()));
         }
 
         sigGen.getRawOutputStream().write(new DERSet(digestAlgs).getEncoded());
@@ -469,13 +686,7 @@ public class CMSSignedDataParser
 
         eiGen.addObject(encapContentInfo.getContentType());
 
-        ASN1OctetStringParser octs = (ASN1OctetStringParser)
-            encapContentInfo.getContent(DERTags.OCTET_STRING);
-
-        if (octs != null)
-        {
-            pipeOctetString(octs, eiGen.getRawOutputStream());
-        }
+        pipeEncapsulatedOctetString(encapContentInfo, eiGen.getRawOutputStream());
 
         eiGen.close();
 
@@ -489,7 +700,7 @@ public class CMSSignedDataParser
         {
             SignerInformation        signer = (SignerInformation)it.next();
 
-            signerInfos.add(signer.toSignerInfo());
+            signerInfos.add(signer.toASN1Structure());
         }
 
         sigGen.getRawOutputStream().write(new DERSet(signerInfos).getEncoded());
@@ -512,6 +723,7 @@ public class CMSSignedDataParser
      * @param out the stream to write the new signed data object to.
      * @return out.
      * @exception CMSException if there is an error processing the CertStore
+     * @deprecated use method that takes Store objects.
      */
     public static OutputStream replaceCertificatesAndCRLs(
         InputStream   original,
@@ -519,9 +731,9 @@ public class CMSSignedDataParser
         OutputStream  out)
         throws CMSException, IOException
     {
-        ASN1StreamParser in = new ASN1StreamParser(original, CMSUtils.getMaximumMemory());
+        ASN1StreamParser in = new ASN1StreamParser(original);
         ContentInfoParser contentInfo = new ContentInfoParser((ASN1SequenceParser)in.readObject());
-        SignedDataParser signedData = SignedDataParser.getInstance(contentInfo.getContent(DERTags.SEQUENCE));
+        SignedDataParser signedData = SignedDataParser.getInstance(contentInfo.getContent(BERTags.SEQUENCE));
 
         BERSequenceGenerator sGen = new BERSequenceGenerator(out);
 
@@ -533,7 +745,7 @@ public class CMSSignedDataParser
         sigGen.addObject(signedData.getVersion());
 
         // digests
-        sigGen.getRawOutputStream().write(signedData.getDigestAlgorithms().getDERObject().getEncoded());
+        sigGen.getRawOutputStream().write(signedData.getDigestAlgorithms().toASN1Primitive().getEncoded());
 
         // encap content info
         ContentInfoParser encapContentInfo = signedData.getEncapContentInfo();
@@ -542,13 +754,7 @@ public class CMSSignedDataParser
 
         eiGen.addObject(encapContentInfo.getContentType());
 
-        ASN1OctetStringParser octs = (ASN1OctetStringParser)
-            encapContentInfo.getContent(DERTags.OCTET_STRING);
-
-        if (octs != null)
-        {
-            pipeOctetString(octs, eiGen.getRawOutputStream());
-        }
+        pipeEncapsulatedOctetString(encapContentInfo, eiGen.getRawOutputStream());
 
         eiGen.close();
 
@@ -593,7 +799,7 @@ public class CMSSignedDataParser
             sigGen.getRawOutputStream().write(new DERTaggedObject(false, 1, crls).getEncoded());
         }
 
-        sigGen.getRawOutputStream().write(signedData.getSignerInfos().getDERObject().getEncoded());
+        sigGen.getRawOutputStream().write(signedData.getSignerInfos().toASN1Primitive().getEncoded());
 
         sigGen.close();
 
@@ -602,35 +808,102 @@ public class CMSSignedDataParser
         return out;
     }
 
-    private static DERObject makeObj(
-        byte[]  encoding)
-        throws IOException
+    /**
+     * Replace the certificate and CRL information associated with this
+     * CMSSignedData object with the new one passed in.
+     * <p>
+     * The output stream is returned unclosed.
+     * </p>
+     * @param original the signed data stream to be used as a base.
+     * @param certs new certificates to be used, if any.
+     * @param crls new CRLs to be used, if any.
+     * @param attrCerts new attribute certificates to be used, if any.
+     * @param out the stream to write the new signed data object to.
+     * @return out.
+     * @exception CMSException if there is an error processing the CertStore
+     */
+    public static OutputStream replaceCertificatesAndCRLs(
+        InputStream   original,
+        Store         certs,
+        Store         crls,
+        Store         attrCerts,
+        OutputStream  out)
+        throws CMSException, IOException
     {
-        if (encoding == null)
+        ASN1StreamParser in = new ASN1StreamParser(original);
+        ContentInfoParser contentInfo = new ContentInfoParser((ASN1SequenceParser)in.readObject());
+        SignedDataParser signedData = SignedDataParser.getInstance(contentInfo.getContent(BERTags.SEQUENCE));
+
+        BERSequenceGenerator sGen = new BERSequenceGenerator(out);
+
+        sGen.addObject(CMSObjectIdentifiers.signedData);
+
+        BERSequenceGenerator sigGen = new BERSequenceGenerator(sGen.getRawOutputStream(), 0, true);
+
+        // version number
+        sigGen.addObject(signedData.getVersion());
+
+        // digests
+        sigGen.getRawOutputStream().write(signedData.getDigestAlgorithms().toASN1Primitive().getEncoded());
+
+        // encap content info
+        ContentInfoParser encapContentInfo = signedData.getEncapContentInfo();
+
+        BERSequenceGenerator eiGen = new BERSequenceGenerator(sigGen.getRawOutputStream());
+
+        eiGen.addObject(encapContentInfo.getContentType());
+
+        pipeEncapsulatedOctetString(encapContentInfo, eiGen.getRawOutputStream());
+
+        eiGen.close();
+
+        //
+        // skip existing certs and CRLs
+        //
+        getASN1Set(signedData.getCertificates());
+        getASN1Set(signedData.getCrls());
+
+        //
+        // replace the certs and crls in the SignedData object
+        //
+        if (certs != null || attrCerts != null)
         {
-            return null;
+            List certificates = new ArrayList();
+
+            if (certs != null)
+            {
+                certificates.addAll(CMSUtils.getCertificatesFromStore(certs));
+            }
+            if (attrCerts != null)
+            {
+                certificates.addAll(CMSUtils.getAttributeCertificatesFromStore(attrCerts));
+            }
+
+            ASN1Set asn1Certs = CMSUtils.createBerSetFromList(certificates);
+
+            if (asn1Certs.size() > 0)
+            {
+                sigGen.getRawOutputStream().write(new DERTaggedObject(false, 0, asn1Certs).getEncoded());
+            }
         }
 
-        ASN1InputStream         aIn = new ASN1InputStream(encoding);
-
-        return aIn.readObject();
-    }
-
-    private static AlgorithmIdentifier makeAlgId(
-        String  oid,
-        byte[]  params)
-        throws IOException
-    {
-        if (params != null)
+        if (crls != null)
         {
-            return new AlgorithmIdentifier(
-                            new DERObjectIdentifier(oid), makeObj(params));
-        }
-        else
-        {
-            return new AlgorithmIdentifier(
-                            new DERObjectIdentifier(oid), new DERNull());
+            ASN1Set asn1Crls = CMSUtils.createBerSetFromList(CMSUtils.getCRLsFromStore(crls));
+
+            if (asn1Crls.size() > 0)
+            {
+                sigGen.getRawOutputStream().write(new DERTaggedObject(false, 1, asn1Crls).getEncoded());
+            }
         }
+
+        sigGen.getRawOutputStream().write(signedData.getSignerInfos().toASN1Primitive().getEncoded());
+
+        sigGen.close();
+
+        sGen.close();
+
+        return out;
     }
 
     private static void writeSetToGeneratorTagged(
@@ -643,11 +916,14 @@ public class CMSSignedDataParser
 
         if (asn1Set != null)
         {
-            ASN1TaggedObject taggedObj = (asn1SetParser instanceof BERSetParser)
-                ?   new BERTaggedObject(false, tagNo, asn1Set)
-                :   new DERTaggedObject(false, tagNo, asn1Set);
-
-            asn1Gen.getRawOutputStream().write(taggedObj.getEncoded());                
+            if (asn1SetParser instanceof BERSetParser)
+            {
+                asn1Gen.getRawOutputStream().write(new BERTaggedObject(false, tagNo, asn1Set).getEncoded());
+            }
+            else
+            {
+                asn1Gen.getRawOutputStream().write(new DERTaggedObject(false, tagNo, asn1Set).getEncoded());
+            }
         }
     }
 
@@ -656,7 +932,38 @@ public class CMSSignedDataParser
     {
         return asn1SetParser == null
             ?   null
-            :   ASN1Set.getInstance(asn1SetParser.getDERObject());
+            :   ASN1Set.getInstance(asn1SetParser.toASN1Primitive());
+    }
+
+    private static void pipeEncapsulatedOctetString(ContentInfoParser encapContentInfo,
+        OutputStream rawOutputStream) throws IOException
+    {
+        ASN1OctetStringParser octs = (ASN1OctetStringParser)
+            encapContentInfo.getContent(BERTags.OCTET_STRING);
+
+        if (octs != null)
+        {
+            pipeOctetString(octs, rawOutputStream);
+        }
+
+//        BERTaggedObjectParser contentObject = (BERTaggedObjectParser)encapContentInfo.getContentObject();
+//        if (contentObject != null)
+//        {
+//            // Handle IndefiniteLengthInputStream safely
+//            InputStream input = ASN1StreamParser.getSafeRawInputStream(contentObject.getContentStream(true));
+//
+//            // TODO BerTaggedObjectGenerator?
+//            BEROutputStream berOut = new BEROutputStream(rawOutputStream);
+//            berOut.write(DERTags.CONSTRUCTED | DERTags.TAGGED | 0);
+//            berOut.write(0x80);
+//
+//            pipeRawOctetString(input, rawOutputStream);
+//
+//            berOut.write(0x00);
+//            berOut.write(0x00);
+//
+//            input.close();
+//        }
     }
 
     private static void pipeOctetString(
@@ -664,10 +971,21 @@ public class CMSSignedDataParser
         OutputStream          output)
         throws IOException
     {
-        BEROctetStringGenerator octGen = new BEROctetStringGenerator(output, 0, true);
         // TODO Allow specification of a specific fragment size?
-        OutputStream outOctets = octGen.getOctetOutputStream();
+        OutputStream outOctets = CMSUtils.createBEROctetOutputStream(
+            output, 0, true, 0);
         Streams.pipeAll(octs.getOctetStream(), outOctets);
         outOctets.close();
     }
+
+//    private static void pipeRawOctetString(
+//        InputStream     rawInput,
+//        OutputStream    rawOutput)
+//        throws IOException
+//    {
+//        InputStream tee = new TeeInputStream(rawInput, rawOutput);
+//        ASN1StreamParser sp = new ASN1StreamParser(tee);
+//        ASN1OctetStringParser octs = (ASN1OctetStringParser)sp.readObject();
+//        Streams.drain(octs.getOctetStream());
+//    }
 }
