@@ -1,5 +1,6 @@
 package org.bouncycastle.cms.bc;
 
+import org.bouncycastle.asn1.cms.IssuerAndSerialNumber;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cms.KeyTransRecipientInfoGenerator;
 import org.bouncycastle.operator.bc.BcAsymmetricKeyWrapper;
@@ -9,7 +10,7 @@ public abstract class BcKeyTransRecipientInfoGenerator
 {
     public BcKeyTransRecipientInfoGenerator(X509CertificateHolder recipientCert, BcAsymmetricKeyWrapper wrapper)
     {
-        super(recipientCert.getIssuerAndSerialNumber(), wrapper);
+        super(new IssuerAndSerialNumber(recipientCert.toASN1Structure()), wrapper);
     }
 
     public BcKeyTransRecipientInfoGenerator(byte[] subjectKeyIdentifier, BcAsymmetricKeyWrapper wrapper)
