@@ -1,6 +1,7 @@
 package org.bouncycastle.crypto.tls;
 
-public class TlsRuntimeException extends RuntimeException
+public class TlsRuntimeException
+    extends RuntimeException
 {
     private static final long serialVersionUID = 1928023487348344086L;
 
