@@ -1,9 +1,18 @@
 package org.bouncycastle.mail.smime;
 
-import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
-import org.bouncycastle.cms.CMSEnvelopedDataStreamGenerator;
-import org.bouncycastle.cms.CMSException;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.security.InvalidKeyException;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.PrivateKey;
+import java.security.Provider;
+import java.security.PublicKey;
+import java.security.cert.CertificateEncodingException;
+import java.security.cert.X509Certificate;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
 
 import javax.activation.CommandMap;
 import javax.activation.MailcapCommandMap;
@@ -11,16 +20,18 @@ import javax.crypto.SecretKey;
 import javax.mail.MessagingException;
 import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMessage;
-import java.io.IOException;
-import java.io.OutputStream;
-import java.security.AlgorithmParameters;
-import java.security.InvalidKeyException;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.PrivateKey;
-import java.security.PublicKey;
-import java.security.Provider;
-import java.security.cert.X509Certificate;
+
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
+import org.bouncycastle.cms.CMSEnvelopedDataStreamGenerator;
+import org.bouncycastle.cms.CMSException;
+import org.bouncycastle.cms.RecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JceCMSContentEncryptorBuilder;
+import org.bouncycastle.cms.jcajce.JceKEKRecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientInfoGenerator;
+import org.bouncycastle.cms.jcajce.JceKeyTransRecipientInfoGenerator;
+import org.bouncycastle.operator.OutputEncryptor;
 
 /**
  * General class for generating a pkcs7-mime message.
@@ -30,9 +41,9 @@ import java.security.cert.X509Certificate;
  * <pre>
  *      SMIMEEnvelopedGenerator  fact = new SMIMEEnvelopedGenerator();
  *
- *      fact.addKeyTransRecipient(cert);
+ *      fact.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(recipientCert).setProvider("BC"));
  *
- *      MimeBodyPart           smime = fact.generate(content, algorithm, "BC");
+ *      MimeBodyPart mp = fact.generate(content, new JceCMSContentEncryptorBuilder(CMSAlgorithm.RC2_CBC, 40).setProvider("BC").build());
  * </pre>
  *
  * <b>Note:<b> Most clients expect the MimeBodyPart to be in a MimeMultipart
@@ -69,6 +80,7 @@ public class SMIMEEnvelopedGenerator
     private static final String ENCRYPTED_CONTENT_TYPE = "application/pkcs7-mime; name=\"smime.p7m\"; smime-type=enveloped-data";
     
     private EnvelopedGenerator fact;
+    private List               recipients = new ArrayList();
 
     static
     {
@@ -96,14 +108,34 @@ public class SMIMEEnvelopedGenerator
         fact = new EnvelopedGenerator();
     }
 
+    /**
+     * add a recipientInfoGenerator.
+     */
+    public void addRecipientInfoGenerator(
+        RecipientInfoGenerator recipientInfoGen)
+        throws IllegalArgumentException
+    {
+        fact.addRecipientInfoGenerator(recipientInfoGen);
+    }
+
     /**
      * add a recipient.
+     * @deprecated use addRecipientInfoGenerator()
      */
     public void addKeyTransRecipient(
         X509Certificate cert)
         throws IllegalArgumentException
     {
-        fact.addKeyTransRecipient(cert);
+        try
+        {
+            JceKeyTransRecipientInfoGenerator infoGenerator = new JceKeyTransRecipientInfoGenerator(cert);
+            recipients.add(infoGenerator);
+            fact.addRecipientInfoGenerator(infoGenerator);
+        }
+        catch (CertificateEncodingException e)
+        {
+            throw new IllegalArgumentException(e.toString());
+        }
     }
 
     /**
@@ -111,24 +143,30 @@ public class SMIMEEnvelopedGenerator
      *
      * @param key the recipient's public key
      * @param subKeyId the subject key id for the recipient's public key
+     * @deprecated use addRecipientInfoGenerator()
      */
     public void addKeyTransRecipient(
         PublicKey   key,
         byte[]      subKeyId)
         throws IllegalArgumentException
     {
-        fact.addKeyTransRecipient(key, subKeyId);
+        JceKeyTransRecipientInfoGenerator infoGenerator = new JceKeyTransRecipientInfoGenerator(subKeyId, key);
+        recipients.add(infoGenerator);
+        fact.addRecipientInfoGenerator(infoGenerator);
     }
 
     /**
      * add a KEK recipient.
+     * @deprecated use addRecipientInfoGenerator()
      */
     public void addKEKRecipient(
         SecretKey   key,
         byte[]      keyIdentifier)
         throws IllegalArgumentException
     {
-        fact.addKEKRecipient(key, keyIdentifier);
+        JceKEKRecipientInfoGenerator infoGenerator = new JceKEKRecipientInfoGenerator(keyIdentifier, key);
+        recipients.add(infoGenerator);
+        fact.addRecipientInfoGenerator(infoGenerator);
     }
 
     /**
@@ -139,6 +177,7 @@ public class SMIMEEnvelopedGenerator
      * @param recipientCert recipient's public key certificate.
      * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
      * @param provider provider to use for the agreement calculation.
+     * @deprecated use addRecipientInfoGenerator()
      */
     public void addKeyAgreementRecipient(
         String           agreementAlgorithm,
@@ -149,7 +188,7 @@ public class SMIMEEnvelopedGenerator
         String           provider)
         throws NoSuchProviderException, NoSuchAlgorithmException, InvalidKeyException
     {
-        fact.addKeyAgreementRecipient(agreementAlgorithm, senderPrivateKey, senderPublicKey, recipientCert, cekWrapAlgorithm, provider);
+        this.addKeyAgreementRecipient(agreementAlgorithm, senderPrivateKey, senderPublicKey, recipientCert, cekWrapAlgorithm, provider);
     }
 
     /**
@@ -160,6 +199,7 @@ public class SMIMEEnvelopedGenerator
      * @param recipientCert recipient's public key certificate.
      * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
      * @param provider provider to use for the agreement calculation.
+     * @deprecated use addRecipientInfoGenerator()
      */
     public void addKeyAgreementRecipient(
         String           agreementAlgorithm,
@@ -170,7 +210,23 @@ public class SMIMEEnvelopedGenerator
         Provider         provider)
         throws NoSuchProviderException, NoSuchAlgorithmException, InvalidKeyException
     {
-        fact.addKeyAgreementRecipient(agreementAlgorithm, senderPrivateKey, senderPublicKey, recipientCert, cekWrapAlgorithm, provider);
+        try
+        {
+            JceKeyAgreeRecipientInfoGenerator infoGenerator = new JceKeyAgreeRecipientInfoGenerator(new ASN1ObjectIdentifier(agreementAlgorithm), senderPrivateKey, senderPublicKey, new ASN1ObjectIdentifier(cekWrapAlgorithm));
+
+            infoGenerator.addRecipient(recipientCert);
+
+            if (provider != null)
+            {
+                infoGenerator.setProvider(provider);
+            }
+
+            fact.addRecipientInfoGenerator(infoGenerator);
+        }
+        catch (CertificateEncodingException e)
+        {
+            throw new NoSuchAlgorithmException("cannot set up generator: " + e);
+        }
     }
 
     /**
@@ -187,7 +243,7 @@ public class SMIMEEnvelopedGenerator
      */
     private MimeBodyPart make(
         MimeBodyPart    content,
-        String          encryptionOID,
+        ASN1ObjectIdentifier encryptionOID,
         int             keySize,
         Provider        provider)
         throws NoSuchAlgorithmException, SMIMEException
@@ -195,7 +251,7 @@ public class SMIMEEnvelopedGenerator
         //
         // check the base algorithm and provider is available
         //
-        createSymmetricKeyGenerator(encryptionOID, provider);
+        createSymmetricKeyGenerator(encryptionOID.getId(), provider);
                 
         try
         {  
@@ -210,14 +266,79 @@ public class SMIMEEnvelopedGenerator
             return data;
         }
         catch (MessagingException e)
+        {
+            throw new SMIMEException("exception putting S/MIME message together.", e);
+        }
+        catch (CMSException e)
+        {
+            throw new SMIMEException("exception putting envelope together.", e);
+        }
+    }
+
+     /**
+     * if we get here we expect the Mime body part to be well defined.
+     */
+    private MimeBodyPart make(
+        MimeBodyPart    content,
+        OutputEncryptor encryptor)
+        throws SMIMEException
+    {
+        try
+        {
+            MimeBodyPart data = new MimeBodyPart();
+
+            data.setContent(new ContentEncryptor(content, encryptor), ENCRYPTED_CONTENT_TYPE);
+            data.addHeader("Content-Type", ENCRYPTED_CONTENT_TYPE);
+            data.addHeader("Content-Disposition", "attachment; filename=\"smime.p7m\"");
+            data.addHeader("Content-Description", "S/MIME Encrypted Message");
+            data.addHeader("Content-Transfer-Encoding", encoding);
+
+            return data;
+        }
+        catch (MessagingException e)
         {
             throw new SMIMEException("exception putting multi-part together.", e);
         }
     }
 
+    /**
+     * generate an enveloped object that contains an SMIME Enveloped
+     * object using the given content encryptor
+     */
+    public MimeBodyPart generate(
+        MimeBodyPart     content,
+        OutputEncryptor  encryptor)
+        throws SMIMEException
+    {
+        return make(makeContentBodyPart(content), encryptor);
+    }
+
+    /**
+     * generate an enveloped object that contains an SMIME Enveloped
+     * object using the given provider from the contents of the passed in
+     * message
+     */
+    public MimeBodyPart generate(
+        MimeMessage     message,
+        OutputEncryptor  encryptor)
+        throws SMIMEException
+    {
+        try
+        {
+            message.saveChanges();      // make sure we're up to date.
+        }
+        catch (MessagingException e)
+        {
+            throw new SMIMEException("unable to save message", e);
+        }
+
+        return make(makeContentBodyPart(message), encryptor);
+    }
+
     /**
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeBodyPart    content,
@@ -225,12 +346,13 @@ public class SMIMEEnvelopedGenerator
         String          provider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
     {
-        return make(makeContentBodyPart(content), encryptionOID, 0, SMIMEUtil.getProvider(provider));
+        return make(makeContentBodyPart(content), new ASN1ObjectIdentifier(encryptionOID), 0, SMIMEUtil.getProvider(provider));
     }
 
     /**
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeBodyPart    content,
@@ -238,13 +360,14 @@ public class SMIMEEnvelopedGenerator
         Provider        provider)
         throws NoSuchAlgorithmException, SMIMEException
     {
-        return make(makeContentBodyPart(content), encryptionOID, 0, provider);
+        return make(makeContentBodyPart(content), new ASN1ObjectIdentifier(encryptionOID), 0, provider);
     }
 
     /**
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider from the contents of the passed in
      * message
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeMessage     message,
@@ -259,6 +382,7 @@ public class SMIMEEnvelopedGenerator
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider from the contents of the passed in
      * message
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeMessage     message,
@@ -275,13 +399,14 @@ public class SMIMEEnvelopedGenerator
             throw new SMIMEException("unable to save message", e);
         }
                         
-        return make(makeContentBodyPart(message), encryptionOID, 0, provider);
+        return make(makeContentBodyPart(message),new ASN1ObjectIdentifier(encryptionOID), 0, provider);
     }
 
     /**
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider. The size of the encryption key
      * is determined by keysize.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeBodyPart    content,
@@ -297,6 +422,7 @@ public class SMIMEEnvelopedGenerator
      * generate an enveloped object that contains an SMIME Enveloped
      * object using the given provider. The size of the encryption key
      * is determined by keysize.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeBodyPart    content,
@@ -305,7 +431,7 @@ public class SMIMEEnvelopedGenerator
         Provider        provider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
     {
-        return make(makeContentBodyPart(content), encryptionOID, keySize, provider);
+        return make(makeContentBodyPart(content), new ASN1ObjectIdentifier(encryptionOID), keySize, provider);
     }
 
     /**
@@ -313,6 +439,7 @@ public class SMIMEEnvelopedGenerator
      * object using the given provider from the contents of the passed in
      * message. The size of the encryption key used to protect the message
      * is determined by keysize.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeMessage     message,
@@ -329,6 +456,7 @@ public class SMIMEEnvelopedGenerator
      * object using the given provider from the contents of the passed in
      * message. The size of the encryption key used to protect the message
      * is determined by keysize.
+     * @deprecated
      */
     public MimeBodyPart generate(
         MimeMessage     message,
@@ -346,31 +474,61 @@ public class SMIMEEnvelopedGenerator
             throw new SMIMEException("unable to save message", e);
         }
                         
-        return make(makeContentBodyPart(message), encryptionOID, keySize, provider);
+        return make(makeContentBodyPart(message), new ASN1ObjectIdentifier(encryptionOID), keySize, provider);
     }
     
     private class ContentEncryptor
         implements SMIMEStreamingProcessor
     {
         private final MimeBodyPart _content;
-        private final String _encryptionOid;
-        private final int    _keySize;
-        private final Provider _provider;
-        
+        private OutputEncryptor _encryptor;
+
         private boolean _firstTime = true;
         
         ContentEncryptor(
             MimeBodyPart content,
-            String       encryptionOid,
+            ASN1ObjectIdentifier       encryptionOid,
             int          keySize,
             Provider     provider)
+            throws CMSException
         {
             _content = content;
-            _encryptionOid = encryptionOid;
-            _keySize = keySize;
-            _provider = provider;
+
+            if (keySize == 0)  // use the default
+            {
+                _encryptor = new JceCMSContentEncryptorBuilder(encryptionOid).setProvider(provider).build();
+            }
+            else
+            {
+                _encryptor = new JceCMSContentEncryptorBuilder(encryptionOid, keySize).setProvider(provider).build();
+            }
+
+            if (provider != null)
+            {
+                for (Iterator it = recipients.iterator(); it.hasNext();)
+                {
+                    RecipientInfoGenerator rd = (RecipientInfoGenerator)it.next();
+
+                    if (rd instanceof JceKeyTransRecipientInfoGenerator)
+                    {
+                        ((JceKeyTransRecipientInfoGenerator)rd).setProvider(provider);
+                    }
+                    else if (rd instanceof JceKEKRecipientInfoGenerator)
+                    {
+                        ((JceKEKRecipientInfoGenerator)rd).setProvider(provider);
+                    }
+                }
+            }
         }
-    
+
+        ContentEncryptor(
+            MimeBodyPart content,
+            OutputEncryptor encryptor)
+        {
+            _content = content;
+            _encryptor = encryptor;
+        }
+
         public void write(OutputStream out)
             throws IOException
         {
@@ -380,20 +538,13 @@ public class SMIMEEnvelopedGenerator
             {
                 if (_firstTime)
                 {
-                    if (_keySize == 0)  // use the default
-                    {
-                        encrypted = fact.open(out, _encryptionOid, _provider);
-                    }
-                    else
-                    {
-                        encrypted = fact.open(out, _encryptionOid, _keySize, _provider);
-                    }
+                    encrypted = fact.open(out, _encryptor);
                     
                     _firstTime = false;
                 }
                 else
                 {
-                    encrypted = fact.regenerate(out, _provider);
+                    encrypted = fact.regenerate(out, _encryptor);
                 }
 
                 _content.getDataHandler().setCommandMap(addCommands(CommandMap.getDefaultCommandMap()));
@@ -406,14 +557,6 @@ public class SMIMEEnvelopedGenerator
             {
                 throw new WrappingIOException(e.toString(), e);
             }
-            catch (NoSuchAlgorithmException e)
-            {
-                throw new WrappingIOException(e.toString(), e);
-            }
-            catch (NoSuchProviderException e)
-            {
-                throw new WrappingIOException(e.toString(), e);
-            }
             catch (CMSException e)
             {
                 throw new WrappingIOException(e.toString(), e);
@@ -424,34 +567,28 @@ public class SMIMEEnvelopedGenerator
     private class EnvelopedGenerator
         extends CMSEnvelopedDataStreamGenerator
     {
-        private String _encryptionOID;
-        private SecretKey _encKey;
-        private AlgorithmParameters _params;
-        private ASN1EncodableVector _recipientInfos;
+        private ASN1ObjectIdentifier dataType;
+        private ASN1EncodableVector  recipientInfos;
 
         protected OutputStream open(
-            OutputStream        out,
-            String              encryptionOID,
-            SecretKey           encKey,
-            AlgorithmParameters params,
-            ASN1EncodableVector recepientInfos,
-            Provider            provider)
-            throws NoSuchAlgorithmException, CMSException
+            ASN1ObjectIdentifier dataType,
+            OutputStream         out,
+            ASN1EncodableVector  recipientInfos,
+            OutputEncryptor      encryptor)
+            throws IOException
         {
-            _encryptionOID = encryptionOID;
-            _encKey = encKey;
-            _params = params;
-            _recipientInfos = recepientInfos;
+            this.dataType = dataType;
+            this.recipientInfos = recipientInfos;
 
-            return super.open(out, encryptionOID, encKey, params, recepientInfos, provider);
+            return super.open(dataType, out, recipientInfos, encryptor);
         }
 
         OutputStream regenerate(
             OutputStream out,
-            Provider     provider)
-            throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
+            OutputEncryptor     encryptor)
+            throws IOException
         {
-            return super.open(out, _encryptionOID, _encKey, _params, _recipientInfos, provider);
+            return super.open(dataType, out, recipientInfos, encryptor);
         }
     }
 
