@@ -1,17 +1,29 @@
 package org.bouncycastle.mail.smime;
 
+import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+
+import javax.activation.CommandMap;
+import javax.activation.MailcapCommandMap;
+import javax.mail.BodyPart;
+import javax.mail.MessagingException;
+import javax.mail.Part;
+import javax.mail.Session;
+import javax.mail.internet.MimeBodyPart;
+import javax.mail.internet.MimeMessage;
+import javax.mail.internet.MimeMultipart;
+
 import org.bouncycastle.cms.CMSException;
 import org.bouncycastle.cms.CMSSignedDataParser;
 import org.bouncycastle.cms.CMSTypedStream;
-
-import javax.activation.CommandMap;
-import javax.activation.MailcapCommandMap;
-import javax.mail.*;
-import javax.mail.internet.MimeBodyPart;
-import javax.mail.internet.MimeMessage;
-import javax.mail.internet.MimeMultipart;
-import java.io.*;
-import java.util.Enumeration;
+import org.bouncycastle.operator.DigestCalculatorProvider;
 
 /**
  * general class for handling a pkcs7-signature message.
@@ -130,6 +142,7 @@ public class SMIMESignedParser
      * @exception MessagingException on an error extracting the signature or
      * otherwise processing the message.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         MimeMultipart message) 
@@ -146,6 +159,7 @@ public class SMIMESignedParser
      * @exception MessagingException on an error extracting the signature or
      * otherwise processing the message.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         MimeMultipart message,
@@ -164,6 +178,7 @@ public class SMIMESignedParser
      * @exception MessagingException on an error extracting the signature or
      * otherwise processing the message.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         MimeMultipart message,
@@ -173,6 +188,63 @@ public class SMIMESignedParser
         this(message, defaultContentTransferEncoding, getTmpFile());
     }
 
+    /**
+     * base constructor using a defaultContentTransferEncoding of 7bit. A temporary backing file
+     * will be created for the signed data.
+     *
+     * @param digCalcProvider provider for digest calculators.
+     * @param message signed message with signature.
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception CMSException if some other problem occurs.
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        MimeMultipart message)
+        throws MessagingException, CMSException
+    {
+        this(message, getTmpFile());
+    }
+
+    /**
+     * base constructor using a defaultContentTransferEncoding of 7bit and a specified backing file.
+     *
+     * @param digCalcProvider provider for digest calculators.
+     * @param message signed message with signature.
+     * @param backingFile the temporary file to use to back the signed data.
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception CMSException if some other problem occurs.
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        MimeMultipart message,
+        File          backingFile)
+        throws MessagingException, CMSException
+    {
+        this(message, "7bit", backingFile);
+    }
+
+    /**
+     * base constructor with settable contentTransferEncoding. A temporary backing file will be created
+     * to contain the signed data.
+     *
+     * @param digCalcProvider provider for digest calculators.
+     * @param message the signed message with signature.
+     * @param defaultContentTransferEncoding new default to use.
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception CMSException if some other problem occurs.r
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        MimeMultipart message,
+        String        defaultContentTransferEncoding)
+        throws MessagingException, CMSException
+    {
+        this(digCalcProvider, message, defaultContentTransferEncoding, getTmpFile());
+    }
+
     /**
      * base constructor with settable contentTransferEncoding and a specified backing file.
      *
@@ -182,6 +254,7 @@ public class SMIMESignedParser
      * @exception MessagingException on an error extracting the signature or
      * otherwise processing the message.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         MimeMultipart message,
@@ -197,6 +270,32 @@ public class SMIMESignedParser
         drainContent();
     }
 
+    /**
+     * base constructor with settable contentTransferEncoding and a specified backing file.
+     *
+     * @param digCalcProvider provider for digest calculators.
+     * @param message the signed message with signature.
+     * @param defaultContentTransferEncoding new default to use.
+     * @param backingFile the temporary file to use to back the signed data.
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception CMSException if some other problem occurs.
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        MimeMultipart message,
+        String        defaultContentTransferEncoding,
+        File          backingFile)
+        throws MessagingException, CMSException
+    {
+        super(digCalcProvider, getSignedInputStream(message.getBodyPart(0), defaultContentTransferEncoding, backingFile), getInputStream(message.getBodyPart(1)));
+
+        this.message = message;
+        this.content = (MimeBodyPart)message.getBodyPart(0);
+
+        drainContent();
+    }
+
     /**
      * base constructor for a signed message with encapsulated content.
      * <p>
@@ -209,6 +308,7 @@ public class SMIMESignedParser
      * otherwise processing the message.
      * @exception SMIMEException if the body part encapsulated in the message cannot be extracted.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         Part message) 
@@ -225,7 +325,38 @@ public class SMIMESignedParser
             this.content = SMIMEUtil.toWriteOnceBodyPart(cont);
         }
     }
-    
+
+    /**
+     * base constructor for a signed message with encapsulated content.
+     * <p>
+     * Note: in this case the encapsulated MimeBody part will only be suitable for a single
+     * writeTo - once writeTo has been called the file containing the body part will be deleted. If writeTo is not
+     * called the file will be left in the temp directory.
+     * </p>
+     * @param digCalcProvider provider for digest calculators.
+     * @param message the message containing the encapsulated signed data.
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception SMIMEException if the body part encapsulated in the message cannot be extracted.
+     * @exception CMSException if some other problem occurs.
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        Part message)
+        throws MessagingException, CMSException, SMIMEException
+    {
+        super(digCalcProvider, getInputStream(message));
+
+        this.message = message;
+
+        CMSTypedStream  cont = this.getSignedContent();
+
+        if (cont != null)
+        {
+            this.content = SMIMEUtil.toWriteOnceBodyPart(cont);
+        }
+    }
+
     /**
      * Constructor for a signed message with encapsulated content. The encapsulated
      * content, if it exists, is written to the file represented by the File object 
@@ -238,6 +369,7 @@ public class SMIMESignedParser
      * otherwise processing the message.
      * @exception SMIMEException if the body part encapsulated in the message cannot be extracted.
      * @exception CMSException if some other problem occurs.
+     * @deprecated use method that takes a DigestCalculatorProvider
      */
     public SMIMESignedParser(
         Part message,
@@ -256,6 +388,38 @@ public class SMIMESignedParser
         }
     }
 
+    /**
+     * Constructor for a signed message with encapsulated content. The encapsulated
+     * content, if it exists, is written to the file represented by the File object
+     * passed in.
+     *
+     * @param digCalcProvider provider for digest calculators.
+     * @param message the Part containing the signed content.
+     * @param file the file the encapsulated part is to be written to after it has been decoded.
+     *
+     * @exception MessagingException on an error extracting the signature or
+     * otherwise processing the message.
+     * @exception SMIMEException if the body part encapsulated in the message cannot be extracted.
+     * @exception CMSException if some other problem occurs.
+     */
+    public SMIMESignedParser(
+        DigestCalculatorProvider digCalcProvider,
+        Part message,
+        File file)
+        throws MessagingException, CMSException, SMIMEException
+    {
+        super(digCalcProvider, getInputStream(message));
+
+        this.message = message;
+
+        CMSTypedStream  cont = this.getSignedContent();
+
+        if (cont != null)
+        {
+            this.content = SMIMEUtil.toMimeBodyPart(cont, file);
+        }
+    }
+
     /**
      * return the content that was signed.
      * @return the signed body part in this message.
