@@ -1,38 +1,5 @@
 package org.bouncycastle.mail.smime.validator;
 
-import org.bouncycastle.asn1.ASN1InputStream;
-import org.bouncycastle.asn1.ASN1OctetString;
-import org.bouncycastle.asn1.ASN1TaggedObject;
-import org.bouncycastle.asn1.DERIA5String;
-import org.bouncycastle.asn1.DERObject;
-import org.bouncycastle.asn1.DEROctetString;
-import org.bouncycastle.asn1.DERSequence;
-import org.bouncycastle.asn1.cms.Attribute;
-import org.bouncycastle.asn1.cms.AttributeTable;
-import org.bouncycastle.asn1.cms.CMSAttributes;
-import org.bouncycastle.asn1.cms.Time;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.asn1.x509.AuthorityKeyIdentifier;
-import org.bouncycastle.asn1.x509.ExtendedKeyUsage;
-import org.bouncycastle.asn1.x509.KeyPurposeId;
-import org.bouncycastle.asn1.x509.X509Extensions;
-import org.bouncycastle.cms.SignerInformation;
-import org.bouncycastle.cms.SignerInformationStore;
-import org.bouncycastle.i18n.ErrorBundle;
-import org.bouncycastle.i18n.filter.TrustedInput;
-import org.bouncycastle.i18n.filter.UntrustedInput;
-import org.bouncycastle.jce.PrincipalUtil;
-import org.bouncycastle.jce.X509Principal;
-import org.bouncycastle.mail.smime.SMIMESigned;
-import org.bouncycastle.x509.CertPathReviewerException;
-import org.bouncycastle.x509.PKIXCertPathReviewer;
-
-import javax.mail.Address;
-import javax.mail.internet.InternetAddress;
-import javax.mail.internet.MimeMessage;
-import javax.mail.internet.MimeMultipart;
-import javax.mail.MessagingException;
-
 import java.io.IOException;
 import java.security.GeneralSecurityException;
 import java.security.PublicKey;
@@ -61,6 +28,42 @@ import java.util.Map;
 import java.util.Set;
 import java.util.Vector;
 
+import javax.mail.Address;
+import javax.mail.MessagingException;
+import javax.mail.internet.InternetAddress;
+import javax.mail.internet.MimeMessage;
+import javax.mail.internet.MimeMultipart;
+
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1OctetString;
+import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.ASN1Sequence;
+import org.bouncycastle.asn1.ASN1TaggedObject;
+import org.bouncycastle.asn1.DERIA5String;
+import org.bouncycastle.asn1.DEROctetString;
+import org.bouncycastle.asn1.cms.Attribute;
+import org.bouncycastle.asn1.cms.AttributeTable;
+import org.bouncycastle.asn1.cms.CMSAttributes;
+import org.bouncycastle.asn1.cms.Time;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.x509.AuthorityKeyIdentifier;
+import org.bouncycastle.asn1.x509.ExtendedKeyUsage;
+import org.bouncycastle.asn1.x509.KeyPurposeId;
+import org.bouncycastle.asn1.x509.X509Extensions;
+import org.bouncycastle.cms.SignerInformation;
+import org.bouncycastle.cms.SignerInformationStore;
+import org.bouncycastle.cms.jcajce.JcaX509CertSelectorConverter;
+import org.bouncycastle.i18n.ErrorBundle;
+import org.bouncycastle.i18n.filter.TrustedInput;
+import org.bouncycastle.i18n.filter.UntrustedInput;
+import org.bouncycastle.jce.PrincipalUtil;
+import org.bouncycastle.jce.X509Principal;
+import org.bouncycastle.mail.smime.SMIMESigned;
+import org.bouncycastle.util.Integers;
+import org.bouncycastle.x509.CertPathReviewerException;
+import org.bouncycastle.x509.PKIXCertPathReviewer;
+
 public class SignedMailValidator
 {
     private static final String RESOURCE_NAME = "org.bouncycastle.mail.smime.validator.SignedMailValidatorMessages";
@@ -78,6 +81,8 @@ public class SignedMailValidator
     // (365.25*30)*24*3600*1000
     private static final long THIRTY_YEARS_IN_MILLI_SEC = 21915l*12l*3600l*1000l;
 
+    private static final JcaX509CertSelectorConverter selectorConverter = new JcaX509CertSelectorConverter();
+
     private CertStore certs;
 
     private SignerInformationStore signers;
@@ -179,24 +184,28 @@ public class SignedMailValidator
 
             // save "from" addresses from message
             Address[] froms = message.getFrom();
-	    InternetAddress sender = null;
-	    try {
-		    if(message.getHeader("Sender") != null) {
-		     sender = new InternetAddress(message.getHeader("Sender")[0]);
-		    }
-	    }
-	    catch (MessagingException ex) {
-		    //ignore garbage in Sender: header
-	    }
-            fromAddresses = new String[froms.length + (sender!=null?1:0)];
-            for (int i = 0; i < froms.length; i++)
+        InternetAddress sender = null;
+        try
+        {
+            if(message.getHeader("Sender") != null)
             {
-                InternetAddress inetAddr = (InternetAddress) froms[i];
-                fromAddresses[i] = inetAddr.getAddress();
+                sender = new InternetAddress(message.getHeader("Sender")[0]);
             }
-	    if(sender!=null) {
-		    fromAddresses[froms.length] = sender.getAddress();
-	    }
+        }
+        catch (MessagingException ex)
+        {
+            //ignore garbage in Sender: header
+        }
+        fromAddresses = new String[froms.length + (sender!=null?1:0)];
+        for (int i = 0; i < froms.length; i++)
+        {
+            InternetAddress inetAddr = (InternetAddress) froms[i];
+            fromAddresses[i] = inetAddr.getAddress();
+        }
+        if(sender!=null)
+        {
+            fromAddresses[froms.length] = sender.getAddress();
+        }
 
             // initialize results
             results = new HashMap();
@@ -241,7 +250,7 @@ public class SignedMailValidator
             try
             {
                 Collection certCollection = findCerts(usedParameters
-                        .getCertStores(), signer.getSID());
+                        .getCertStores(), selectorConverter.getCertSelector(signer.getSID()));
 
                 Iterator certIt = certCollection.iterator();
                 if (certIt.hasNext())
@@ -419,7 +428,7 @@ public class SignedMailValidator
         byte[] ext = cert.getExtensionValue(SUBJECT_ALTERNATIVE_NAME);
         if (ext != null)
         {
-            DERSequence altNames = (DERSequence) getObject(ext);
+            ASN1Sequence altNames = ASN1Sequence.getInstance(getObject(ext));
             for (int j = 0; j < altNames.size(); j++)
             {
                 ASN1TaggedObject o = (ASN1TaggedObject) altNames
@@ -427,7 +436,7 @@ public class SignedMailValidator
 
                 if (o.getTagNo() == 1)
                 {
-                    String email = DERIA5String.getInstance(o, true)
+                    String email = DERIA5String.getInstance(o, false)
                             .getString().toLowerCase();
                     addresses.add(email);
                 }
@@ -437,7 +446,7 @@ public class SignedMailValidator
         return addresses;
     }
 
-    private static DERObject getObject(byte[] ext) throws IOException
+    private static ASN1Primitive getObject(byte[] ext) throws IOException
     {
         ASN1InputStream aIn = new ASN1InputStream(ext);
         ASN1OctetString octs = (ASN1OctetString) aIn.readObject();
@@ -464,7 +473,7 @@ public class SignedMailValidator
         {
             ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
                     "SignedMailValidator.shortSigningKey",
-                    new Object[] { new Integer(keyLenght) });
+                    new Object[]{Integers.valueOf(keyLenght)});
             notifications.add(msg);
         }
         
@@ -591,7 +600,7 @@ public class SignedMailValidator
             if (attr != null)
             {
                 Time t = Time.getInstance(attr.getAttrValues().getObjectAt(0)
-                        .getDERObject());
+                        .toASN1Primitive());
                 result = t.getDate();
             }
         }
@@ -739,7 +748,7 @@ public class SignedMailValidator
                         AuthorityKeyIdentifier kid = AuthorityKeyIdentifier.getInstance(getObject(authKeyIdentBytes));
                         if (kid.getKeyIdentifier() != null)
                         {
-                            select.setSubjectKeyIdentifier(new DEROctetString(kid.getKeyIdentifier()).getDEREncoded());
+                            select.setSubjectKeyIdentifier(new DEROctetString(kid.getKeyIdentifier()).getEncoded(ASN1Encoding.DER));
                         }
                     }
                     catch (IOException ioe)
