@@ -2,29 +2,27 @@ package org.bouncycastle.openpgp;
 
 import java.io.IOException;
 import java.io.OutputStream;
-import java.math.BigInteger;
-import java.security.DigestOutputStream;
-import java.security.Key;
-import java.security.MessageDigest;
 import java.security.NoSuchProviderException;
 import java.security.Provider;
 import java.security.SecureRandom;
-import java.security.Security;
 import java.util.ArrayList;
 import java.util.List;
 
-import javax.crypto.Cipher;
-import javax.crypto.CipherOutputStream;
-import javax.crypto.spec.IvParameterSpec;
-
 import org.bouncycastle.bcpg.BCPGOutputStream;
-import org.bouncycastle.bcpg.ContainedPacket;
 import org.bouncycastle.bcpg.HashAlgorithmTags;
 import org.bouncycastle.bcpg.PacketTags;
-import org.bouncycastle.bcpg.PublicKeyEncSessionPacket;
-import org.bouncycastle.bcpg.S2K;
 import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
-import org.bouncycastle.bcpg.SymmetricKeyEncSessionPacket;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.bouncycastle.openpgp.operator.PBEKeyEncryptionMethodGenerator;
+import org.bouncycastle.openpgp.operator.PGPDataEncryptor;
+import org.bouncycastle.openpgp.operator.PGPDataEncryptorBuilder;
+import org.bouncycastle.openpgp.operator.PGPDigestCalculator;
+import org.bouncycastle.openpgp.operator.PGPKeyEncryptionMethodGenerator;
+import org.bouncycastle.openpgp.operator.jcajce.JcaPGPDigestCalculatorProviderBuilder;
+import org.bouncycastle.openpgp.operator.jcajce.JcePBEKeyEncryptionMethodGenerator;
+import org.bouncycastle.openpgp.operator.jcajce.JcePGPDataEncryptorBuilder;
+import org.bouncycastle.openpgp.operator.jcajce.JcePublicKeyKeyEncryptionMethodGenerator;
+import org.bouncycastle.util.io.TeeOutputStream;
 
 /**
  *  Generator for encrypted objects.
@@ -58,231 +56,147 @@ public class PGPEncryptedDataGenerator
     public static final int S2K_SHA512 = HashAlgorithmTags.SHA512;
 
     private BCPGOutputStream     pOut;
-    private CipherOutputStream   cOut;
-    private Cipher               c;
-    private boolean              withIntegrityPacket = false;
+    private OutputStream         cOut;
     private boolean              oldFormat = false;
-    private DigestOutputStream   digestOut;
-        
-    private abstract class EncMethod
-        extends ContainedPacket
-    {
-        protected byte[]     sessionInfo;
-        protected int        encAlgorithm;
-        protected Key        key;
-        
-        public abstract void addSessionInfo(
-            byte[]    sessionInfo) 
-            throws Exception;
-    }
-    
-    private class PBEMethod
-        extends EncMethod
-    {
-        S2K             s2k;
+    private PGPDigestCalculator digestCalc;
+    private OutputStream            genOut;
+    private PGPDataEncryptorBuilder dataEncryptorBuilder;
 
-        PBEMethod(
-            int        encAlgorithm,
-            S2K        s2k,
-            Key        key)
-        {
-            this.encAlgorithm = encAlgorithm;
-            this.s2k = s2k;
-            this.key = key;
-        }
-
-        public Key getKey()
-        {
-            return key;
-        }
-
-        public void addSessionInfo(
-            byte[]    sessionInfo) 
-            throws Exception
-        {
-            String        cName = PGPUtil.getSymmetricCipherName(encAlgorithm);
-            Cipher        c = Cipher.getInstance(cName + "/CFB/NoPadding", defProvider);
-
-            c.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(new byte[c.getBlockSize()]), rand);
-        
-            this.sessionInfo = c.doFinal(sessionInfo, 0, sessionInfo.length - 2);
-        }
-
-        public void encode(BCPGOutputStream pOut) 
-            throws IOException
-        {
-            SymmetricKeyEncSessionPacket pk = new SymmetricKeyEncSessionPacket(encAlgorithm, s2k, sessionInfo);
-
-            pOut.writePacket(pk);
-        }
-    }
-
-    private class PubMethod
-        extends EncMethod
-    {
-        PGPPublicKey    pubKey;
-        BigInteger[]    data;
-        
-        PubMethod(
-            PGPPublicKey        pubKey)
-        {
-            this.pubKey = pubKey;
-        }
-    
-        public void addSessionInfo(
-            byte[]    sessionInfo) 
-            throws Exception
-        {
-            Cipher            c;
-
-            switch (pubKey.getAlgorithm())
-            {
-            case PGPPublicKey.RSA_ENCRYPT:
-            case PGPPublicKey.RSA_GENERAL:
-                c = Cipher.getInstance("RSA/ECB/PKCS1Padding", defProvider);
-                break;
-            case PGPPublicKey.ELGAMAL_ENCRYPT:
-            case PGPPublicKey.ELGAMAL_GENERAL:
-                c = Cipher.getInstance("ElGamal/ECB/PKCS1Padding", defProvider);
-                break;
-            case PGPPublicKey.DSA:
-                throw new PGPException("Can't use DSA for encryption.");
-            case PGPPublicKey.ECDSA:
-                throw new PGPException("Can't use ECDSA for encryption.");
-            default:
-                throw new PGPException("unknown asymmetric algorithm: " + pubKey.getAlgorithm());
-            }
-
-            Key key = pubKey.getKey(defProvider);
-            
-            c.init(Cipher.ENCRYPT_MODE, key, rand);
-        
-            byte[]    encKey = c.doFinal(sessionInfo);
-            
-            switch (pubKey.getAlgorithm())
-            {
-            case PGPPublicKey.RSA_ENCRYPT:
-            case PGPPublicKey.RSA_GENERAL:
-                data = new BigInteger[1];
-                
-                data[0] = new BigInteger(1, encKey);
-                break;
-            case PGPPublicKey.ELGAMAL_ENCRYPT:
-            case PGPPublicKey.ELGAMAL_GENERAL:
-                byte[]        b1 = new byte[encKey.length / 2];
-                byte[]        b2 = new byte[encKey.length / 2];
-                
-                System.arraycopy(encKey, 0, b1, 0, b1.length);
-                System.arraycopy(encKey, b1.length, b2, 0, b2.length);
-                
-                data = new BigInteger[2];
-                data[0] = new BigInteger(1, b1);
-                data[1] = new BigInteger(1, b2);
-                break;
-            default:
-                throw new PGPException("unknown asymmetric algorithm: " + encAlgorithm);
-            }
-        }
-        
-        public void encode(BCPGOutputStream pOut) 
-            throws IOException
-        {
-            PublicKeyEncSessionPacket    pk = new PublicKeyEncSessionPacket(pubKey.getKeyID(), pubKey.getAlgorithm(), data);
-            
-            pOut.writePacket(pk);
-        }
-    }
-    
     private List            methods = new ArrayList();
     private int             defAlgorithm;
     private SecureRandom    rand;
-    private Provider        defProvider;
+
+    private static Provider        defProvider;
     
-    /**
-     * Base constructor.
-     *
-     * @param encAlgorithm the symmetric algorithm to use.
-     * @param rand source of randomness
-     * @param provider the provider to use for encryption algorithms.
-     */
+   /**
+       * Base constructor.
+       *
+       * @param encAlgorithm the symmetric algorithm to use.
+       * @param rand source of randomness
+       * @param provider the provider name to use for encryption algorithms.
+       * @deprecated  use constructor that takes a PGPDataEncryptor
+       */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         SecureRandom        rand,
         String              provider)
     {
-        this(encAlgorithm, rand, Security.getProvider(provider));
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setSecureRandom(rand).setProvider(provider));
     }
 
+   /**
+       * Base constructor.
+       *
+       * @param encAlgorithm the symmetric algorithm to use.
+       * @param rand source of randomness
+       * @param provider the provider to use for encryption algorithms.
+       * @deprecated  use constructor that takes a PGPDataEncryptorBuilder
+       */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         SecureRandom        rand,
         Provider            provider)
     {
-        this.defAlgorithm = encAlgorithm;
-        this.rand = rand;
-        this.defProvider = provider;
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setSecureRandom(rand).setProvider(provider));
     }
 
     /**
-     * Creates a cipher stream which will have an integrity packet
-     * associated with it.
-     * 
-     * @param encAlgorithm
-     * @param withIntegrityPacket
-     * @param rand
-     * @param provider
-     */
+        * Creates a cipher stream which will have an integrity packet
+        * associated with it.
+        *
+        * @param encAlgorithm
+        * @param withIntegrityPacket
+        * @param rand
+        * @param provider
+        * @deprecated  use constructor that takes a PGPDataEncryptorBuilder
+        */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         boolean             withIntegrityPacket,
         SecureRandom        rand,
         String              provider)
     {
-        this(encAlgorithm, withIntegrityPacket, rand, Security.getProvider(provider));
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setWithIntegrityPacket(withIntegrityPacket).setSecureRandom(rand).setProvider(provider));
     }
 
+    /**
+        * Creates a cipher stream which will have an integrity packet
+        * associated with it.
+        *
+        * @param encAlgorithm
+        * @param withIntegrityPacket
+        * @param rand
+        * @param provider
+        * @deprecated  use constructor that takes a PGPDataEncryptorBuilder
+        */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         boolean             withIntegrityPacket,
         SecureRandom        rand,
         Provider            provider)
     {
-        this.defAlgorithm = encAlgorithm;
-        this.rand = rand;
-        this.defProvider = provider;
-        this.withIntegrityPacket = withIntegrityPacket;
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setWithIntegrityPacket(withIntegrityPacket).setSecureRandom(rand).setProvider(provider));
     }
 
-    /**
-     * Base constructor.
-     *
-     * @param encAlgorithm the symmetric algorithm to use.
-     * @param rand source of randomness
-     * @param oldFormat PGP 2.6.x compatability required.
-     * @param provider the provider to use for encryption algorithms.
-     */
+   /**
+       * Base constructor.
+       *
+       * @param encAlgorithm the symmetric algorithm to use.
+       * @param rand source of randomness
+       * @param oldFormat PGP 2.6.x compatibility required.
+       * @param provider the provider to use for encryption algorithms.
+       * @deprecated  use constructor that takes a PGPDataEncryptorBuilder
+       */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         SecureRandom        rand,
         boolean             oldFormat,
         String              provider)
     {
-        this.defAlgorithm = encAlgorithm;
-        this.rand = rand;
-        this.defProvider = Security.getProvider(provider);
-        this.oldFormat = oldFormat;
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setSecureRandom(rand).setProvider(provider), oldFormat);
     }
 
+   /**
+       * Base constructor.
+       *
+       * @param encAlgorithm the symmetric algorithm to use.
+       * @param rand source of randomness
+       * @param oldFormat PGP 2.6.x compatibility required.
+       * @param provider the provider to use for encryption algorithms.
+       * @deprecated  use constructor that takes a PGPDataEncryptorBuilder
+       */
     public PGPEncryptedDataGenerator(
         int                 encAlgorithm,
         SecureRandom        rand,
         boolean             oldFormat,
         Provider            provider)
     {
-        this.defAlgorithm = encAlgorithm;
-        this.rand = rand;
-        this.defProvider = provider;
+        this(new JcePGPDataEncryptorBuilder(encAlgorithm).setSecureRandom(rand).setProvider(provider), oldFormat);
+    }
+
+   /**
+       * Base constructor.
+       *
+       * @param encryptorBuilder builder to create actual data encryptor.
+       */
+    public PGPEncryptedDataGenerator(PGPDataEncryptorBuilder encryptorBuilder)
+    {
+        this(encryptorBuilder, false);
+    }
+
+   /**
+       * Base constructor with the option to turn on formatting for PGP 2.6.x compatibility.
+       *
+       * @param encryptorBuilder builder to create actual data encryptor.
+       * @param oldFormat PGP 2.6.x compatibility required.
+       */
+    public PGPEncryptedDataGenerator(PGPDataEncryptorBuilder encryptorBuilder, boolean oldFormat)
+    {
+        this.dataEncryptorBuilder = encryptorBuilder;
         this.oldFormat = oldFormat;
+
+        this.defAlgorithm = dataEncryptorBuilder.getAlgorithm();
+        this.rand = dataEncryptorBuilder.getSecureRandom();
     }
 
     /**
@@ -291,6 +205,7 @@ public class PGPEncryptedDataGenerator
      * @param passPhrase
      * @throws NoSuchProviderException
      * @throws PGPException
+     * @deprecated  use addMethod that takes  PGPKeyEncryptionMethodGenerator
      */
     public void addMethod(
         char[]    passPhrase) 
@@ -306,6 +221,7 @@ public class PGPEncryptedDataGenerator
      * @param s2kDigest digest algorithm to use for S2K calculation
      * @throws NoSuchProviderException
      * @throws PGPException
+     * @deprecated  use addMethod that takes  PGPKeyEncryptionMethodGenerator
      */
     public void addMethod(
         char[]    passPhrase,
@@ -314,16 +230,10 @@ public class PGPEncryptedDataGenerator
     {
         if (defProvider == null)
         {
-            throw new NoSuchProviderException("unable to find provider.");
+            defProvider = new BouncyCastleProvider();
         }
 
-        byte[]        iv = new byte[8];
-
-        rand.nextBytes(iv);
-
-        S2K            s2k = new S2K(s2kDigest, iv, 0x60);
-
-        methods.add(new PBEMethod(defAlgorithm, s2k, PGPUtil.makeKeyFromPassPhrase(defAlgorithm, s2k, passPhrase, defProvider)));
+        addMethod(new JcePBEKeyEncryptionMethodGenerator(passPhrase, new JcaPGPDigestCalculatorProviderBuilder().setProvider(defProvider).build().get(s2kDigest)).setProvider(defProvider).setSecureRandom(rand));
     }
 
     /**
@@ -332,6 +242,7 @@ public class PGPEncryptedDataGenerator
      * @param key
      * @throws NoSuchProviderException
      * @throws PGPException
+     * @deprecated  use addMethod that takes  PGPKeyEncryptionMethodGenerator
      */
     public void addMethod(
         PGPPublicKey    key) 
@@ -344,12 +255,23 @@ public class PGPEncryptedDataGenerator
 
         if (defProvider == null)
         {
-            throw new NoSuchProviderException("unable to find provider.");
+            defProvider = new BouncyCastleProvider();
         }
 
-        methods.add(new PubMethod(key));
+        addMethod(new JcePublicKeyKeyEncryptionMethodGenerator(key).setProvider(defProvider).setSecureRandom(rand));
     }
-    
+
+    /**
+        *  Added a key encryption method to be used to encrypt the session data associated
+        *  with this encrypted data.
+        *
+        * @param method  key encryption method to use.
+        */
+    public void addMethod(PGPKeyEncryptionMethodGenerator method)
+    {
+        methods.add(method);
+    }
+
     private void addCheckSum(
         byte[]    sessionInfo)
     {
@@ -365,10 +287,9 @@ public class PGPEncryptedDataGenerator
     }
 
     private byte[] createSessionInfo(
-        int algorithm,
-        Key key)
+        int     algorithm,
+        byte[]  keyBytes)
     {
-        byte[] keyBytes = key.getEncoded();
         byte[] sessionInfo = new byte[keyBytes.length + 3];
         sessionInfo[0] = (byte) algorithm;
         System.arraycopy(keyBytes, 0, sessionInfo, 1, keyBytes.length);
@@ -388,7 +309,7 @@ public class PGPEncryptedDataGenerator
      * @param length
      * @param buffer
      * @return
-     * @throws IOException
+     * @throws java.io.IOException
      * @throws PGPException
      * @throws IllegalStateException
      */
@@ -408,41 +329,32 @@ public class PGPEncryptedDataGenerator
             throw new IllegalStateException("no encryption methods specified");
         }
 
-        if (defProvider == null)
-        {
-            throw new IllegalStateException("provider resolves to null");
-        }
-
-        Key key = null;
+        byte[] key = null;
 
         pOut = new BCPGOutputStream(out);
 
+        defAlgorithm = dataEncryptorBuilder.getAlgorithm();
+        rand = dataEncryptorBuilder.getSecureRandom();
+
         if (methods.size() == 1)
         {    
-            if (methods.get(0) instanceof PBEMethod)
+
+            if (methods.get(0) instanceof PBEKeyEncryptionMethodGenerator)
             {
-                PBEMethod m = (PBEMethod)methods.get(0);
-                
-                key = m.getKey();
+                PBEKeyEncryptionMethodGenerator m = (PBEKeyEncryptionMethodGenerator)methods.get(0);
+
+                key = m.getKey(dataEncryptorBuilder.getAlgorithm());
+
+                pOut.writePacket(((PGPKeyEncryptionMethodGenerator)methods.get(0)).generate(defAlgorithm, null));
             }
             else
             {
                 key = PGPUtil.makeRandomKey(defAlgorithm, rand);
                 byte[] sessionInfo = createSessionInfo(defAlgorithm, key);
+                PGPKeyEncryptionMethodGenerator m = (PGPKeyEncryptionMethodGenerator)methods.get(0);
 
-                PubMethod m = (PubMethod)methods.get(0);
-
-                try
-                {
-                    m.addSessionInfo(sessionInfo);
-                }
-                catch (Exception e)
-                {
-                    throw new PGPException("exception encrypting session key", e);
-                }
+                pOut.writePacket(m.generate(defAlgorithm, sessionInfo));
             }
-            
-            pOut.writePacket((ContainedPacket)methods.get(0));
         }
         else // multiple methods
         {
@@ -451,81 +363,55 @@ public class PGPEncryptedDataGenerator
 
             for (int i = 0; i != methods.size(); i++)
             {
-                EncMethod m = (EncMethod)methods.get(i);
+                PGPKeyEncryptionMethodGenerator m = (PGPKeyEncryptionMethodGenerator)methods.get(i);
 
-                try
-                {
-                    m.addSessionInfo(sessionInfo);
-                }
-                catch (Exception e)
-                {
-                    throw new PGPException("exception encrypting session key", e);
-                }
-
-                pOut.writePacket(m);
+                pOut.writePacket(m.generate(defAlgorithm, sessionInfo));
             }
         }
 
-        String cName = PGPUtil.getSymmetricCipherName(defAlgorithm);
-
-        if (cName == null)
-        {
-            throw new PGPException("null cipher specified");
-        }
-
         try
         {
-            if (withIntegrityPacket)
-            {
-                c = Cipher.getInstance(cName + "/CFB/NoPadding", defProvider);
-            }
-            else
-            {
-                c = Cipher.getInstance(cName + "/OpenPGPCFB/NoPadding", defProvider);
-            }
+            PGPDataEncryptor dataEncryptor = dataEncryptorBuilder.build(key);
 
-            byte[] iv = new byte[c.getBlockSize()];
-            c.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv), rand);
+            digestCalc = dataEncryptor.getIntegrityCalculator();
             
             if (buffer == null)
             {
                 //
                 // we have to add block size + 2 for the generated IV and + 1 + 22 if integrity protected
                 //
-                if (withIntegrityPacket)
+                if (digestCalc != null)
                 {
-                    pOut = new BCPGOutputStream(out, PacketTags.SYM_ENC_INTEGRITY_PRO, length + c.getBlockSize() + 2 + 1 + 22);
+                    pOut = new ClosableBCPGOutputStream(out, PacketTags.SYM_ENC_INTEGRITY_PRO, length + dataEncryptor.getBlockSize() + 2 + 1 + 22);
+
                     pOut.write(1);        // version number
                 }
                 else
                 {
-                    pOut = new BCPGOutputStream(out, PacketTags.SYMMETRIC_KEY_ENC, length + c.getBlockSize() + 2, oldFormat);
+                    pOut = new ClosableBCPGOutputStream(out, PacketTags.SYMMETRIC_KEY_ENC, length + dataEncryptor.getBlockSize() + 2, oldFormat);
                 }
             }
             else
             {
-                if (withIntegrityPacket)
+                if (digestCalc != null)
                 {
-                    pOut = new BCPGOutputStream(out, PacketTags.SYM_ENC_INTEGRITY_PRO, buffer);
+                    pOut = new ClosableBCPGOutputStream(out, PacketTags.SYM_ENC_INTEGRITY_PRO, buffer);
                     pOut.write(1);        // version number
                 }
                 else
                 {
-                    pOut = new BCPGOutputStream(out, PacketTags.SYMMETRIC_KEY_ENC, buffer);
+                    pOut = new ClosableBCPGOutputStream(out, PacketTags.SYMMETRIC_KEY_ENC, buffer);
                 }
             }
 
+            genOut = cOut = dataEncryptor.getOutputStream(pOut);
 
-            OutputStream genOut = cOut = new CipherOutputStream(pOut, c);
-
-            if (withIntegrityPacket)
+            if (digestCalc != null)
             {
-                String digestName = PGPUtil.getDigestName(HashAlgorithmTags.SHA1);
-                MessageDigest digest = MessageDigest.getInstance(digestName, defProvider);
-                genOut = digestOut = new DigestOutputStream(cOut, digest);
+                genOut = new TeeOutputStream(digestCalc.getOutputStream(), cOut);
             }
 
-            byte[] inLineIv = new byte[c.getBlockSize() + 2];
+            byte[] inLineIv = new byte[dataEncryptor.getBlockSize() + 2];
             rand.nextBytes(inLineIv);
             inLineIv[inLineIv.length - 1] = inLineIv[inLineIv.length - 3];
             inLineIv[inLineIv.length - 2] = inLineIv[inLineIv.length - 4];
@@ -593,42 +479,59 @@ public class PGPEncryptedDataGenerator
      * returned by the open() method.
      * <p>
      * <b>Note</b>: This does not close the underlying output stream, only the stream on top of it created by the open() method.
-     * @throws IOException
+     * @throws java.io.IOException
      */
     public void close()
         throws IOException
     {
         if (cOut != null)
         {    
-            if (digestOut != null)
+            if (digestCalc != null)
             {
                 //
                 // hand code a mod detection packet
                 //
-                BCPGOutputStream bOut = new BCPGOutputStream(digestOut, PacketTags.MOD_DETECTION_CODE, 20);
+                BCPGOutputStream bOut = new BCPGOutputStream(genOut, PacketTags.MOD_DETECTION_CODE, 20);
 
                 bOut.flush();
-                digestOut.flush();
 
-                byte[] dig = digestOut.getMessageDigest().digest();
+                byte[] dig = digestCalc.getDigest();
 
                 cOut.write(dig);
             }
 
-            cOut.flush();
-
-            try
-            {
-                pOut.write(c.doFinal());
-                pOut.finish();
-            }
-            catch (Exception e)
-            {
-                throw new IOException(e.toString());
-            }
+            cOut.close();
 
             cOut = null;
             pOut = null;
         }
     }
+
+    private class ClosableBCPGOutputStream
+        extends BCPGOutputStream
+    {
+        public ClosableBCPGOutputStream(OutputStream out, int symmetricKeyEnc, byte[] buffer)
+            throws IOException
+        {
+            super(out, symmetricKeyEnc, buffer);
+        }
+
+        public ClosableBCPGOutputStream(OutputStream out, int symmetricKeyEnc, long length, boolean oldFormat)
+            throws IOException
+        {
+            super(out, symmetricKeyEnc, length, oldFormat);
+        }
+
+        public ClosableBCPGOutputStream(OutputStream out, int symEncIntegrityPro, long length)
+            throws IOException
+        {
+            super(out, symEncIntegrityPro, length);
+        }
+
+        public void close()
+            throws IOException
+        {
+             this.finish();
+        }
+    }
 }
