@@ -1,20 +1,21 @@
 package org.bouncycastle.openpgp;
 
+import java.util.ArrayList;
+import java.util.Date;
+import java.util.List;
+
 import org.bouncycastle.bcpg.SignatureSubpacket;
 import org.bouncycastle.bcpg.SignatureSubpacketTags;
+import org.bouncycastle.bcpg.sig.Features;
 import org.bouncycastle.bcpg.sig.IssuerKeyID;
 import org.bouncycastle.bcpg.sig.KeyExpirationTime;
 import org.bouncycastle.bcpg.sig.KeyFlags;
 import org.bouncycastle.bcpg.sig.NotationData;
 import org.bouncycastle.bcpg.sig.PreferredAlgorithms;
+import org.bouncycastle.bcpg.sig.PrimaryUserID;
 import org.bouncycastle.bcpg.sig.SignatureCreationTime;
 import org.bouncycastle.bcpg.sig.SignatureExpirationTime;
 import org.bouncycastle.bcpg.sig.SignerUserID;
-import org.bouncycastle.bcpg.sig.PrimaryUserID;
-
-import java.util.ArrayList;
-import java.util.Date;
-import java.util.List;
 
 /**
  * Container for a list of signature subpackets.
@@ -246,7 +247,19 @@ public class PGPSignatureSubpacketVector
         
         return list;
     }
-    
+
+    public Features getFeatures()
+    {
+        SignatureSubpacket    p = this.getSubpacket(SignatureSubpacketTags.FEATURES);
+
+        if (p == null)
+        {
+            return null;
+        }
+
+        return new Features(p.isCritical(), p.getData());
+    }
+
     /**
      * Return the number of packets this vector contains.
      * 
