@@ -16,8 +16,10 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1EncodableVector;
 import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Set;
 import org.bouncycastle.asn1.cms.Attribute;
@@ -29,8 +31,10 @@ import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
 import org.bouncycastle.asn1.x509.ExtendedKeyUsage;
+import org.bouncycastle.asn1.x509.Extension;
+import org.bouncycastle.asn1.x509.Extensions;
+import org.bouncycastle.asn1.x509.ExtensionsGenerator;
 import org.bouncycastle.asn1.x509.KeyPurposeId;
-import org.bouncycastle.asn1.x509.X509Extension;
 import org.bouncycastle.asn1.x509.X509Extensions;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cms.SignerInformation;
@@ -38,6 +42,7 @@ import org.bouncycastle.operator.DigestCalculator;
 import org.bouncycastle.operator.DigestCalculatorProvider;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.Integers;
 
 public class TSPUtil
 {
@@ -49,16 +54,16 @@ public class TSPUtil
 
     static
     {
-        digestLengths.put(PKCSObjectIdentifiers.md5.getId(), new Integer(16));
-        digestLengths.put(OIWObjectIdentifiers.idSHA1.getId(), new Integer(20));
-        digestLengths.put(NISTObjectIdentifiers.id_sha224.getId(), new Integer(28));
-        digestLengths.put(NISTObjectIdentifiers.id_sha256.getId(), new Integer(32));
-        digestLengths.put(NISTObjectIdentifiers.id_sha384.getId(), new Integer(48));
-        digestLengths.put(NISTObjectIdentifiers.id_sha512.getId(), new Integer(64));
-        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd128.getId(), new Integer(16));
-        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd160.getId(), new Integer(20));
-        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd256.getId(), new Integer(32));
-        digestLengths.put(CryptoProObjectIdentifiers.gostR3411.getId(), new Integer(32));
+        digestLengths.put(PKCSObjectIdentifiers.md5.getId(), Integers.valueOf(16));
+        digestLengths.put(OIWObjectIdentifiers.idSHA1.getId(), Integers.valueOf(20));
+        digestLengths.put(NISTObjectIdentifiers.id_sha224.getId(), Integers.valueOf(28));
+        digestLengths.put(NISTObjectIdentifiers.id_sha256.getId(), Integers.valueOf(32));
+        digestLengths.put(NISTObjectIdentifiers.id_sha384.getId(), Integers.valueOf(48));
+        digestLengths.put(NISTObjectIdentifiers.id_sha512.getId(), Integers.valueOf(64));
+        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd128.getId(), Integers.valueOf(16));
+        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd160.getId(), Integers.valueOf(20));
+        digestLengths.put(TeleTrusTObjectIdentifiers.ripemd256.getId(), Integers.valueOf(32));
+        digestLengths.put(CryptoProObjectIdentifiers.gostR3411.getId(), Integers.valueOf(32));
 
         digestNames.put(PKCSObjectIdentifiers.md5.getId(), "MD5");
         digestNames.put(OIWObjectIdentifiers.idSHA1.getId(), "SHA1");
@@ -106,11 +111,11 @@ public class TSPUtil
                 {
                     try
                     {
-                        ContentInfo contentInfo = ContentInfo.getInstance(tsAttrValues.getObjectAt(j).getDERObject());
+                        ContentInfo contentInfo = ContentInfo.getInstance(tsAttrValues.getObjectAt(j));
                         TimeStampToken timeStampToken = new TimeStampToken(contentInfo);
                         TimeStampTokenInfo tstInfo = timeStampToken.getTimeStampInfo();
 
-                        MessageDigest digest = createDigestInstance(tstInfo.getMessageImprintAlgOID(), provider);
+                        MessageDigest digest = createDigestInstance(tstInfo.getMessageImprintAlgOID().getId(), provider);
                         byte[] expectedDigest = digest.digest(signerInfo.getSignature());
 
                         if (!Arrays.constantTimeAreEqual(expectedDigest, tstInfo.getMessageImprintDigest()))
@@ -163,7 +168,7 @@ public class TSPUtil
                 {
                     try
                     {
-                        ContentInfo contentInfo = ContentInfo.getInstance(tsAttrValues.getObjectAt(j).getDERObject());
+                        ContentInfo contentInfo = ContentInfo.getInstance(tsAttrValues.getObjectAt(j));
                         TimeStampToken timeStampToken = new TimeStampToken(contentInfo);
                         TimeStampTokenInfo tstInfo = timeStampToken.getTimeStampInfo();
 
@@ -257,12 +262,12 @@ public class TSPUtil
         X509CertificateHolder cert)
         throws TSPValidationException
     {
-        if (cert.toASN1Structure().getVersion() != 3)
+        if (cert.toASN1Structure().getVersionNumber() != 3)
         {
             throw new IllegalArgumentException("Certificate must have an ExtendedKeyUsage extension.");
         }
 
-        X509Extension  ext = cert.getExtension(X509Extension.extendedKeyUsage);
+        Extension ext = cert.getExtension(Extension.extendedKeyUsage);
         if (ext == null)
         {
             throw new TSPValidationException("Certificate must have an ExtendedKeyUsage extension.");
@@ -273,7 +278,7 @@ public class TSPUtil
             throw new TSPValidationException("Certificate must have an ExtendedKeyUsage extension marked as critical.");
         }
 
-        ExtendedKeyUsage    extKey = ExtendedKeyUsage.getInstance(X509Extension.convertValueToObject(ext));
+        ExtendedKeyUsage    extKey = ExtendedKeyUsage.getInstance(ext.getParsedValue());
 
         if (!extKey.hasKeyPurposeId(KeyPurposeId.id_kp_timeStamping) || extKey.size() != 1)
         {
@@ -353,7 +358,7 @@ public class TSPUtil
         return Collections.unmodifiableSet(new HashSet(java.util.Arrays.asList(extensions.getNonCriticalExtensionOIDs())));
     }
 
-    static List getExtensionOIDs(X509Extensions extensions)
+    static List getExtensionOIDs(Extensions extensions)
     {
         if (extensions == null)
         {
@@ -362,4 +367,17 @@ public class TSPUtil
 
         return Collections.unmodifiableList(java.util.Arrays.asList(extensions.getExtensionOIDs()));
     }
+
+    static void addExtension(ExtensionsGenerator extGenerator, ASN1ObjectIdentifier oid, boolean isCritical, ASN1Encodable value)
+        throws TSPIOException
+    {
+        try
+        {
+            extGenerator.addExtension(oid, isCritical, value);
+        }
+        catch (IOException e)
+        {
+            throw new TSPIOException("cannot encode extension: " + e.getMessage(), e);
+        }
+    }
 }
