@@ -1,6 +1,8 @@
 package org.bouncycastle.tsp;
 
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.math.BigInteger;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -22,17 +24,20 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1Boolean;
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.ASN1GeneralizedTime;
+import org.bouncycastle.asn1.ASN1Integer;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
-import org.bouncycastle.asn1.DERBoolean;
-import org.bouncycastle.asn1.DERGeneralizedTime;
-import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.cms.Attribute;
 import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.ess.ESSCertID;
+import org.bouncycastle.asn1.ess.ESSCertIDv2;
 import org.bouncycastle.asn1.ess.SigningCertificate;
+import org.bouncycastle.asn1.ess.SigningCertificateV2;
+import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.tsp.Accuracy;
 import org.bouncycastle.asn1.tsp.MessageImprint;
@@ -53,12 +58,39 @@ import org.bouncycastle.cms.SignerInfoGenerator;
 import org.bouncycastle.cms.SimpleAttributeTableGenerator;
 import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
 import org.bouncycastle.jce.interfaces.GOST3410PrivateKey;
+import org.bouncycastle.operator.DigestCalculator;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
 import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
 import org.bouncycastle.util.CollectionStore;
 import org.bouncycastle.util.Store;
 
+/**
+ * Currently the class supports ESSCertID by if a digest calculator based on SHA1 is passed in, otherwise it uses
+ * ESSCertIDv2. In the event you need to pass both types, you will need to override the SignedAttributeGenerator
+ * for the SignerInfoGeneratorBuilder you are using. For the default for ESSCertIDv2 the code will look something
+ * like the following:
+ * <pre>
+ * final ESSCertID essCertid = new ESSCertID(certHashSha1, issuerSerial);
+ * final ESSCertIDv2 essCertidV2 = new ESSCertIDv2(certHashSha256, issuerSerial);
+ *
+ * signerInfoGenBuilder.setSignedAttributeGenerator(new CMSAttributeTableGenerator()
+ * {
+ *     public AttributeTable getAttributes(Map parameters)
+ *         throws CMSAttributeTableGenerationException
+ *     {
+ *         CMSAttributeTableGenerator attrGen = new DefaultSignedAttributeTableGenerator();
+ *
+ *         AttributeTable table = attrGen.getAttributes(parameters);
+ *
+ *         table = table.add(PKCSObjectIdentifiers.id_aa_signingCertificate, new SigningCertificate(essCertid));
+ *         table = table.add(PKCSObjectIdentifiers.id_aa_signingCertificateV2, new SigningCertificateV2(essCertidV2));
+ *
+ *         return table;
+ *     }
+ * });
+ * </pre>
+ */
 public class TimeStampTokenGenerator
 {
     int accuracySeconds = -1;
@@ -71,14 +103,13 @@ public class TimeStampTokenGenerator
 
     GeneralName tsa = null;
     
-    private String  tsaPolicyOID;
+    private ASN1ObjectIdentifier  tsaPolicyOID;
 
     PrivateKey      key;
     X509Certificate cert;
     String          digestOID;
     AttributeTable  signedAttr;
     AttributeTable  unsignedAttr;
-    CertStore       certsAndCrls;
 
     private List certs = new ArrayList();
     private List crls = new ArrayList();
@@ -86,42 +117,82 @@ public class TimeStampTokenGenerator
     private SignerInfoGenerator signerInfoGen;
 
     /**
+     * Basic Constructor - set up a calculator based on signerInfoGen with a ESSCertID calculated from
+     * the signer's associated certificate using the sha1DigestCalculator. If alternate values are required
+     * for id-aa-signingCertificate they should be added to the signerInfoGen object before it is passed in,
+     * otherwise a standard digest based value will be added.
      *
+     * @param signerInfoGen the generator for the signer we are using.
+     * @param digestCalculator calculator for to use for digest of certificate.
+     * @param tsaPolicy tasPolicy to send.
+     * @throws IllegalArgumentException if calculator is not SHA-1 or there is no associated certificate for the signer,
+     * @throws TSPException if the signer certificate cannot be processed.
      */
     public TimeStampTokenGenerator(
-        final SignerInfoGenerator     signerInfoGen,
-        ASN1ObjectIdentifier          tsaPolicy)
+        final SignerInfoGenerator       signerInfoGen,
+        DigestCalculator                digestCalculator,
+        ASN1ObjectIdentifier            tsaPolicy)
         throws IllegalArgumentException, TSPException
     {
         this.signerInfoGen = signerInfoGen;
-        this.tsaPolicyOID = tsaPolicy.getId();
+        this.tsaPolicyOID = tsaPolicy;
 
         if (!signerInfoGen.hasAssociatedCertificate())
         {
             throw new IllegalArgumentException("SignerInfoGenerator must have an associated certificate");
         }
-        
+
         TSPUtil.validateCertificate(signerInfoGen.getAssociatedCertificate());
 
         try
         {
-            final ESSCertID essCertid = new ESSCertID(MessageDigest.getInstance("SHA-1").digest(signerInfoGen.getAssociatedCertificate().getEncoded()));
+            OutputStream dOut = digestCalculator.getOutputStream();
 
-            this.signerInfoGen = new SignerInfoGenerator(signerInfoGen, new CMSAttributeTableGenerator()
+            dOut.write(signerInfoGen.getAssociatedCertificate().getEncoded());
+
+            dOut.close();
+
+            if (digestCalculator.getAlgorithmIdentifier().getAlgorithm().equals(OIWObjectIdentifiers.idSHA1))
             {
-                public AttributeTable getAttributes(Map parameters)
-                    throws CMSAttributeTableGenerationException
+                final ESSCertID essCertid = new ESSCertID(digestCalculator.getDigest());
+
+                this.signerInfoGen = new SignerInfoGenerator(signerInfoGen, new CMSAttributeTableGenerator()
                 {
-                    AttributeTable table = signerInfoGen.getSignedAttributeTableGenerator().getAttributes(parameters);
+                    public AttributeTable getAttributes(Map parameters)
+                        throws CMSAttributeTableGenerationException
+                    {
+                        AttributeTable table = signerInfoGen.getSignedAttributeTableGenerator().getAttributes(parameters);
 
-                    return table.add(PKCSObjectIdentifiers.id_aa_signingCertificate, new SigningCertificate(essCertid));
-                }
-            }, signerInfoGen.getUnsignedAttributeTableGenerator());
+                        if (table.get(PKCSObjectIdentifiers.id_aa_signingCertificate) == null)
+                        {
+                            return table.add(PKCSObjectIdentifiers.id_aa_signingCertificate, new SigningCertificate(essCertid));
+                        }
 
-        }
-        catch (NoSuchAlgorithmException e)
-        {
-            throw new TSPException("Can't find a SHA-1 implementation.", e);
+                        return table;
+                    }
+                }, signerInfoGen.getUnsignedAttributeTableGenerator());
+            }
+            else
+            {
+                AlgorithmIdentifier digAlgID = new AlgorithmIdentifier(digestCalculator.getAlgorithmIdentifier().getAlgorithm());
+                final ESSCertIDv2   essCertid = new ESSCertIDv2(digAlgID, digestCalculator.getDigest());
+
+                this.signerInfoGen = new SignerInfoGenerator(signerInfoGen, new CMSAttributeTableGenerator()
+                {
+                    public AttributeTable getAttributes(Map parameters)
+                        throws CMSAttributeTableGenerationException
+                    {
+                        AttributeTable table = signerInfoGen.getSignedAttributeTableGenerator().getAttributes(parameters);
+
+                        if (table.get(PKCSObjectIdentifiers.id_aa_signingCertificateV2) == null)
+                        {
+                            return table.add(PKCSObjectIdentifiers.id_aa_signingCertificateV2, new SigningCertificateV2(essCertid));
+                        }
+
+                        return table;
+                    }
+                }, signerInfoGen.getUnsignedAttributeTableGenerator());
+            }
         }
         catch (IOException e)
         {
@@ -129,9 +200,66 @@ public class TimeStampTokenGenerator
         }
     }
 
+    /**
+     * Basic Constructor - set up a calculator based on signerInfoGen with a ESSCertID calculated from
+     * the signer's associated certificate using the sha1DigestCalculator.
+     *
+     * @param sha1DigestCalculator calculator for SHA-1 of certificate.
+     * @param signerInfoGen the generator for the signer we are using.
+     * @param tsaPolicy tasPolicy to send.
+     * @throws IllegalArgumentException if calculator is not SHA-1 or there is no associated certificate for the signer,
+     * @throws TSPException if the signer certificate cannot be processed.
+     * @deprecated use constructor taking signerInfoGen first.
+     */
+    public TimeStampTokenGenerator(
+        DigestCalculator sha1DigestCalculator,
+        final SignerInfoGenerator         signerInfoGen,
+        ASN1ObjectIdentifier              tsaPolicy)
+        throws IllegalArgumentException, TSPException
+    {
+        this(signerInfoGen, sha1DigestCalculator, tsaPolicy);
+    }
+
     /**
      * basic creation - only the default attributes will be included here.
-     * @deprecated use SignerInfoGenerator constructor
+     * @deprecated use SignerInfoGenerator constructor that takes a digest calculator
+     */
+    public TimeStampTokenGenerator(
+        final SignerInfoGenerator     signerInfoGen,
+        ASN1ObjectIdentifier          tsaPolicy)
+        throws IllegalArgumentException, TSPException
+    {
+        this(new DigestCalculator()
+        {
+            private ByteArrayOutputStream bOut = new ByteArrayOutputStream();
+
+            public AlgorithmIdentifier getAlgorithmIdentifier()
+            {
+                return new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1, DERNull.INSTANCE);
+            }
+
+            public OutputStream getOutputStream()
+            {
+                return bOut;
+            }
+
+            public byte[] getDigest()
+            {
+                try
+                {
+                    return MessageDigest.getInstance("SHA-1").digest(bOut.toByteArray());
+                }
+                catch (NoSuchAlgorithmException e)
+                {
+                    throw new IllegalStateException("cannot find sha-1: "+ e.getMessage());
+                }
+            }
+        }, signerInfoGen, tsaPolicy);
+    }
+
+    /**
+     * basic creation - only the default attributes will be included here.
+     * @deprecated use SignerInfoGenerator constructor that takes a digest calculator.
      */
     public TimeStampTokenGenerator(
         PrivateKey      key,
@@ -143,9 +271,23 @@ public class TimeStampTokenGenerator
         this(key, cert, digestOID, tsaPolicyOID, null, null);
     }
 
+    /**
+     * basic creation - only the default attributes will be included here.
+     * @deprecated use SignerInfoGenerator constructor that takes a digest calculator.
+     */
+    public TimeStampTokenGenerator(
+        PrivateKey      key,
+        X509Certificate cert,
+        ASN1ObjectIdentifier          digestOID,
+        String          tsaPolicyOID)
+        throws IllegalArgumentException, TSPException
+    {
+        this(key, cert, digestOID.getId(), tsaPolicyOID, null, null);
+    }
+
     /**
      * create with a signer with extra signed/unsigned attributes.
-     * @deprecated use SignerInfoGenerator constructor
+     * @deprecated use SignerInfoGenerator constructor that takes a digest calculator.
      */
     public TimeStampTokenGenerator(
         PrivateKey      key,
@@ -159,7 +301,7 @@ public class TimeStampTokenGenerator
         this.key = key;
         this.cert = cert;
         this.digestOID = digestOID;
-        this.tsaPolicyOID = tsaPolicyOID;
+        this.tsaPolicyOID = new ASN1ObjectIdentifier(tsaPolicyOID);
         this.unsignedAttr = unsignedAttr;
 
         //
@@ -341,56 +483,56 @@ public class TimeStampTokenGenerator
             throw new IllegalStateException("can only use this method with SignerInfoGenerator constructor");
         }
 
-        ASN1ObjectIdentifier digestAlgOID = new ASN1ObjectIdentifier(request.getMessageImprintAlgOID());
+        ASN1ObjectIdentifier digestAlgOID = request.getMessageImprintAlgOID();
 
-        AlgorithmIdentifier algID = new AlgorithmIdentifier(digestAlgOID, new DERNull());
+        AlgorithmIdentifier algID = new AlgorithmIdentifier(digestAlgOID, DERNull.INSTANCE);
         MessageImprint      messageImprint = new MessageImprint(algID, request.getMessageImprintDigest());
 
         Accuracy accuracy = null;
         if (accuracySeconds > 0 || accuracyMillis > 0 || accuracyMicros > 0)
         {
-            DERInteger seconds = null;
+            ASN1Integer seconds = null;
             if (accuracySeconds > 0)
             {
-                seconds = new DERInteger(accuracySeconds);
+                seconds = new ASN1Integer(accuracySeconds);
             }
 
-            DERInteger millis = null;
+            ASN1Integer millis = null;
             if (accuracyMillis > 0)
             {
-                millis = new DERInteger(accuracyMillis);
+                millis = new ASN1Integer(accuracyMillis);
             }
 
-            DERInteger micros = null;
+            ASN1Integer micros = null;
             if (accuracyMicros > 0)
             {
-                micros = new DERInteger(accuracyMicros);
+                micros = new ASN1Integer(accuracyMicros);
             }
 
             accuracy = new Accuracy(seconds, millis, micros);
         }
 
-        DERBoolean derOrdering = null;
+        ASN1Boolean derOrdering = null;
         if (ordering)
         {
-            derOrdering = new DERBoolean(ordering);
+            derOrdering = new ASN1Boolean(ordering);
         }
 
-        DERInteger  nonce = null;
+        ASN1Integer  nonce = null;
         if (request.getNonce() != null)
         {
-            nonce = new DERInteger(request.getNonce());
+            nonce = new ASN1Integer(request.getNonce());
         }
 
-        ASN1ObjectIdentifier tsaPolicy = new ASN1ObjectIdentifier(tsaPolicyOID);
+        ASN1ObjectIdentifier tsaPolicy = tsaPolicyOID;
         if (request.getReqPolicy() != null)
         {
-            tsaPolicy = new ASN1ObjectIdentifier(request.getReqPolicy());
+            tsaPolicy = request.getReqPolicy();
         }
 
         TSTInfo tstInfo = new TSTInfo(tsaPolicy,
-                messageImprint, new DERInteger(serialNumber),
-                new DERGeneralizedTime(genTime), accuracy, derOrdering,
+                messageImprint, new ASN1Integer(serialNumber),
+                new ASN1GeneralizedTime(genTime), accuracy, derOrdering,
                 nonce, tsa, request.getExtensions());
 
         try
@@ -411,7 +553,7 @@ public class TimeStampTokenGenerator
 
             signedDataGenerator.addSignerInfoGenerator(signerInfoGen);
 
-            byte[] derEncodedTSTInfo = tstInfo.getEncoded(ASN1Encodable.DER);
+            byte[] derEncodedTSTInfo = tstInfo.getEncoded(ASN1Encoding.DER);
 
             CMSSignedData signedData = signedDataGenerator.generate(new CMSProcessableByteArray(PKCSObjectIdentifiers.id_ct_TSTInfo, derEncodedTSTInfo), true);
 
