@@ -1,11 +1,13 @@
 package org.bouncycastle.asn1.test;
 
+import java.io.ByteArrayInputStream;
+import java.util.Enumeration;
+
 import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1Integer;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
-import org.bouncycastle.asn1.DERBitString;
-import org.bouncycastle.asn1.DERInteger;
-import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.util.ASN1Dump;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.AttCertIssuer;
@@ -16,8 +18,11 @@ import org.bouncycastle.asn1.x509.AttributeCertificateInfo;
 import org.bouncycastle.asn1.x509.AuthorityKeyIdentifier;
 import org.bouncycastle.asn1.x509.BasicConstraints;
 import org.bouncycastle.asn1.x509.CRLDistPoint;
+import org.bouncycastle.asn1.x509.Certificate;
 import org.bouncycastle.asn1.x509.DistributionPoint;
 import org.bouncycastle.asn1.x509.ExtendedKeyUsage;
+import org.bouncycastle.asn1.x509.Extension;
+import org.bouncycastle.asn1.x509.Extensions;
 import org.bouncycastle.asn1.x509.GeneralName;
 import org.bouncycastle.asn1.x509.GeneralNames;
 import org.bouncycastle.asn1.x509.Holder;
@@ -25,16 +30,10 @@ import org.bouncycastle.asn1.x509.KeyPurposeId;
 import org.bouncycastle.asn1.x509.KeyUsage;
 import org.bouncycastle.asn1.x509.PolicyInformation;
 import org.bouncycastle.asn1.x509.SubjectKeyIdentifier;
-import org.bouncycastle.asn1.x509.TBSCertificateStructure;
-import org.bouncycastle.asn1.x509.X509CertificateStructure;
-import org.bouncycastle.asn1.x509.X509Extension;
-import org.bouncycastle.asn1.x509.X509Extensions;
+import org.bouncycastle.asn1.x509.TBSCertificate;
 import org.bouncycastle.util.encoders.Base64;
 import org.bouncycastle.util.test.SimpleTest;
 
-import java.io.ByteArrayInputStream;
-import java.util.Enumeration;
-
 public class CertificateTest
     extends SimpleTest
 {
@@ -224,77 +223,88 @@ public class CertificateTest
         byte[]  cert)
         throws Exception
     {
-        ByteArrayInputStream    bIn;
-        ASN1InputStream         aIn;
-        String                  dump = "";
-
-        bIn = new ByteArrayInputStream(cert);
-        aIn = new ASN1InputStream(bIn);
+        ByteArrayInputStream bIn = new ByteArrayInputStream(cert);
+        ASN1InputStream aIn = new ASN1InputStream(bIn);
 
         ASN1Sequence      seq = (ASN1Sequence)aIn.readObject();
-        dump = ASN1Dump.dumpAsString(seq);
+//        String dump = ASN1Dump.dumpAsString(seq);
 
-        X509CertificateStructure    obj = new X509CertificateStructure(seq);
-        TBSCertificateStructure     tbsCert = obj.getTBSCertificate();
+        Certificate obj = Certificate.getInstance(seq);
+        TBSCertificate     tbsCert = obj.getTBSCertificate();
         
         if (!tbsCert.getSubject().toString().equals(subjects[id - 1]))
         {
             fail("failed subject test for certificate id " + id + " got " + tbsCert.getSubject().toString());
         }
         
-        if (tbsCert.getVersion() == 3)
+        if (tbsCert.getVersionNumber() == 3)
         {
-            X509Extensions                ext = tbsCert.getExtensions();
+            Extensions                ext = tbsCert.getExtensions();
             if (ext != null)
             {
                 Enumeration    en = ext.oids();
                 while (en.hasMoreElements())
                 {
-                    DERObjectIdentifier    oid = (DERObjectIdentifier)en.nextElement();
-                    X509Extension            extVal = ext.getExtension(oid);
+                    ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)en.nextElement();
+                    Extension            extVal = ext.getExtension(oid);
                     
-                    ASN1OctetString        oct = extVal.getValue();
+                    ASN1OctetString        oct = extVal.getExtnValue();
                     ASN1InputStream        extIn = new ASN1InputStream(new ByteArrayInputStream(oct.getOctets()));
                     
-                    if (oid.equals(X509Extensions.SubjectKeyIdentifier))
+                    if (oid.equals(Extension.subjectKeyIdentifier))
                     {
                         SubjectKeyIdentifier si = SubjectKeyIdentifier.getInstance(extIn.readObject());
+
+                        if (!si.equals(SubjectKeyIdentifier.fromExtensions(ext)))
+                        {
+                            fail("SubjectKeyIdentifier not matched");
+                        }
                     }
-                    else if (oid.equals(X509Extensions.KeyUsage))
+                    else if (oid.equals(Extension.keyUsage))
                     {
-                        DERBitString ku = KeyUsage.getInstance(extIn.readObject());
+                        KeyUsage ku = KeyUsage.getInstance(extIn.readObject());
+
+                        if (!ku.equals(KeyUsage.fromExtensions(ext)))
+                        {
+                            fail("KeyUsage not matched");
+                        }
                     }
-                    else if (oid.equals(X509Extensions.ExtendedKeyUsage))
+                    else if (oid.equals(Extension.extendedKeyUsage))
                     {
                         ExtendedKeyUsage ku = ExtendedKeyUsage.getInstance(extIn.readObject());
                         
-                        ASN1Sequence    sq = (ASN1Sequence)ku.getDERObject();
+                        ASN1Sequence    sq = (ASN1Sequence)ku.toASN1Primitive();
                         for (int i = 0; i != sq.size(); i++)
                         {
-                            DERObjectIdentifier    p = KeyPurposeId.getInstance(sq.getObjectAt(i));
+                            ASN1ObjectIdentifier    p = ASN1ObjectIdentifier.getInstance(KeyPurposeId.getInstance(sq.getObjectAt(i)));
+                        }
+
+                        if (!ku.equals(ExtendedKeyUsage.fromExtensions(ext)))
+                        {
+                            fail("ExtendedKeyUsage not matched");
                         }
                     }
-                    else if (oid.equals(X509Extensions.SubjectAlternativeName))
+                    else if (oid.equals(Extension.subjectAlternativeName))
                     {
                         GeneralNames    gn = GeneralNames.getInstance(extIn.readObject());
                         
-                        ASN1Sequence    sq = (ASN1Sequence)gn.getDERObject();
+                        ASN1Sequence    sq = (ASN1Sequence)gn.toASN1Primitive();
                         for (int i = 0; i != sq.size(); i++)
                         {
                             GeneralName    n = GeneralName.getInstance(sq.getObjectAt(i));
                         }
                     }
-                    else if (oid.equals(X509Extensions.IssuerAlternativeName))
+                    else if (oid.equals(Extension.issuerAlternativeName))
                     {
                         GeneralNames    gn = GeneralNames.getInstance(extIn.readObject());
                         
-                        ASN1Sequence    sq = (ASN1Sequence)gn.getDERObject();
+                        ASN1Sequence    sq = (ASN1Sequence)gn.toASN1Primitive();
                         for (int i = 0; i != sq.size(); i++)
                         {
                             GeneralName    n = GeneralName.getInstance(sq.getObjectAt(i));
                         }
                     }
-                    else if (oid.equals(X509Extensions.CRLDistributionPoints))
+                    else if (oid.equals(Extension.cRLDistributionPoints))
                     {
                         CRLDistPoint    p = CRLDistPoint.getInstance(extIn.readObject());
                         
@@ -304,7 +314,7 @@ public class CertificateTest
                             // do nothing
                         }
                     }
-                    else if (oid.equals(X509Extensions.CertificatePolicies))
+                    else if (oid.equals(Extension.certificatePolicies))
                     {
                         ASN1Sequence    cp = (ASN1Sequence)extIn.readObject();
                         
@@ -313,13 +323,23 @@ public class CertificateTest
                             PolicyInformation.getInstance(cp.getObjectAt(i));
                         }
                     }
-                    else if (oid.equals(X509Extensions.AuthorityKeyIdentifier))
+                    else if (oid.equals(Extension.authorityKeyIdentifier))
                     {
                         AuthorityKeyIdentifier    auth = AuthorityKeyIdentifier.getInstance(extIn.readObject());
+
+                        if (!auth.equals(AuthorityKeyIdentifier.fromExtensions(ext)))
+                        {
+                            fail("AuthorityKeyIdentifier not matched");
+                        }
                     }
-                    else if (oid.equals(X509Extensions.BasicConstraints))
+                    else if (oid.equals(Extension.basicConstraints))
                     {
                         BasicConstraints    bc = BasicConstraints.getInstance(extIn.readObject());
+
+                        if (!bc.equals(BasicConstraints.fromExtensions(ext)))
+                        {
+                            fail("BasicConstraints not matched");
+                        }
                     }
                     else
                     {
@@ -350,8 +370,8 @@ public class CertificateTest
         AttributeCertificateInfo acInfo = obj.getAcinfo();
 
         // Version
-        if (!(acInfo.getVersion().equals(new DERInteger(1)))
-                && (!(acInfo.getVersion().equals(new DERInteger(2)))))
+        if (!(acInfo.getVersion().equals(new ASN1Integer(1)))
+                && (!(acInfo.getVersion().equals(new ASN1Integer(2)))))
         {
             fail(
                     "failed AC Version test for id " + id);
@@ -382,7 +402,7 @@ public class CertificateTest
         }
 
         // Serial
-        DERInteger serial = acInfo.getSerialNumber();
+        ASN1Integer serial = acInfo.getSerialNumber();
 
         // Validity
         AttCertValidityPeriod validity = acInfo.getAttrCertValidityPeriod();
@@ -403,15 +423,15 @@ public class CertificateTest
         // TODO, how to best test?
 
         // X509 Extensions
-        X509Extensions ext = acInfo.getExtensions();
+        Extensions ext = acInfo.getExtensions();
         if (ext != null)
         {
             Enumeration en = ext.oids();
             while (en.hasMoreElements())
             {
-                DERObjectIdentifier oid = (DERObjectIdentifier) en
+                ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier) en
                         .nextElement();
-                X509Extension extVal = ext.getExtension(oid);
+                Extension extVal = ext.getExtension(oid);
             }
         }
     }
