@@ -1,23 +1,5 @@
 package org.bouncycastle.cms.test;
 
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
-import org.bouncycastle.asn1.ASN1InputStream;
-import org.bouncycastle.asn1.DEROutputStream;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
-import org.bouncycastle.cms.CMSEnvelopedDataParser;
-import org.bouncycastle.cms.CMSEnvelopedDataStreamGenerator;
-import org.bouncycastle.cms.CMSTypedStream;
-import org.bouncycastle.cms.RecipientId;
-import org.bouncycastle.cms.RecipientInformation;
-import org.bouncycastle.cms.RecipientInformationStore;
-import org.bouncycastle.jce.PrincipalUtil;
-import org.bouncycastle.util.encoders.Base64;
-import org.bouncycastle.util.encoders.Hex;
-
-import javax.crypto.SecretKey;
 import java.io.BufferedOutputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.InputStream;
@@ -32,9 +14,32 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Iterator;
 
+import javax.crypto.SecretKey;
+
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
+import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.DEROutputStream;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
+import org.bouncycastle.cms.CMSEnvelopedDataParser;
+import org.bouncycastle.cms.CMSEnvelopedDataStreamGenerator;
+import org.bouncycastle.cms.CMSTypedStream;
+import org.bouncycastle.cms.KEKRecipientId;
+import org.bouncycastle.cms.RecipientId;
+import org.bouncycastle.cms.RecipientInformation;
+import org.bouncycastle.cms.RecipientInformationStore;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientId;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.bouncycastle.util.encoders.Base64;
+import org.bouncycastle.util.encoders.Hex;
+
 public class EnvelopedDataStreamTest
     extends TestCase
 {
+    private static final String BC = BouncyCastleProvider.PROVIDER_NAME;
+
     private static final int BUFFER_SIZE = 4000;
     private static String          _signDN;
     private static KeyPair         _signKP;
@@ -129,7 +134,7 @@ public class EnvelopedDataStreamTest
         Iterator    it = c.iterator();
 
         PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyData);
-        KeyFactory          keyFact = KeyFactory.getInstance("RSA", "BC");
+        KeyFactory          keyFact = KeyFactory.getInstance("RSA", BC);
         Key                 priKey = keyFact.generatePrivate(keySpec);
         byte[]              data = Hex.decode("57616c6c6157616c6c6157617368696e67746f6e");
 
@@ -139,7 +144,7 @@ public class EnvelopedDataStreamTest
 
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
             
-            CMSTypedStream recData = recipient.getContentStream(priKey, "BC");
+            CMSTypedStream recData = recipient.getContentStream(priKey, BC);
             
             assertEquals(true, Arrays.equals(data, CMSTestUtil.streamToByteArray(recData.getContentStream())));
         }
@@ -165,7 +170,7 @@ public class EnvelopedDataStreamTest
     
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
             
-            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), "BC");
+            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), BC);
             
             assertEquals(true, Arrays.equals(expectedData, CMSTestUtil.streamToByteArray(recData.getContentStream())));
         }
@@ -191,7 +196,7 @@ public class EnvelopedDataStreamTest
         ByteArrayOutputStream  bOut = new ByteArrayOutputStream();
         
         OutputStream out = edGen.open(
-                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         for (int i = 0; i != 2000; i++)
         {
@@ -213,7 +218,7 @@ public class EnvelopedDataStreamTest
     
         bOut = new ByteArrayOutputStream();
         
-        out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+        out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         BufferedOutputStream bfOut = new BufferedOutputStream(out, 300);
         
@@ -249,7 +254,7 @@ public class EnvelopedDataStreamTest
         ByteArrayOutputStream  bOut = new ByteArrayOutputStream();
         
         OutputStream out = edGen.open(
-                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         for (int i = 0; i != 2000; i++)
         {
@@ -273,7 +278,7 @@ public class EnvelopedDataStreamTest
     
         bOut = new ByteArrayOutputStream();
         
-        out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+        out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         for (int i = 0; i != 2000; i++)
         {
@@ -304,7 +309,7 @@ public class EnvelopedDataStreamTest
         ByteArrayOutputStream  bOut = new ByteArrayOutputStream();
         
         OutputStream out = edGen.open(
-                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         for (int i = 0; i != 2000; i++)
         {
@@ -346,7 +351,7 @@ public class EnvelopedDataStreamTest
     
         ByteArrayOutputStream bOut = new ByteArrayOutputStream();
         
-        OutputStream out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+        OutputStream out = edGen.open(bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         for (int i = 0; i != data.length; i++)
         {
@@ -366,7 +371,7 @@ public class EnvelopedDataStreamTest
     
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
             
-            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), "BC");
+            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), BC);
             
             InputStream           dataStream = recData.getContentStream();
             ByteArrayOutputStream dataOut = new ByteArrayOutputStream();
@@ -405,7 +410,7 @@ public class EnvelopedDataStreamTest
         ByteArrayOutputStream  bOut = new ByteArrayOutputStream();
         
         OutputStream out = edGen.open(
-                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                                bOut, CMSEnvelopedDataGenerator.AES128_CBC, BC);
     
         out.write(data);
         
@@ -426,7 +431,7 @@ public class EnvelopedDataStreamTest
     
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
             
-            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), "BC");
+            CMSTypedStream recData = recipient.getContentStream(_reciKP.getPrivate(), BC);
             
             assertEquals(true, Arrays.equals(data, CMSTestUtil.streamToByteArray(recData.getContentStream())));
         }
@@ -502,7 +507,7 @@ public class EnvelopedDataStreamTest
         
         OutputStream out = edGen.open(
                                 bOut,
-                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, "BC");
+                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, BC);
         out.write(data);
         
         out.close();
@@ -522,7 +527,7 @@ public class EnvelopedDataStreamTest
 
             assertEquals(recipient.getKeyEncryptionAlgOID(), "2.16.840.1.101.3.4.1.25");
             
-            CMSTypedStream recData = recipient.getContentStream(kek, "BC");
+            CMSTypedStream recData = recipient.getContentStream(kek, BC);
             
             assertEquals(true, Arrays.equals(data, CMSTestUtil.streamToByteArray(recData.getContentStream())));
         }
@@ -549,7 +554,7 @@ public class EnvelopedDataStreamTest
         
         OutputStream out = edGen.open(
                                 bOut,
-                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, "BC");
+                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, BC);
         out.write(data);
         
         out.close();
@@ -560,15 +565,13 @@ public class EnvelopedDataStreamTest
     
         assertEquals(ep.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.DES_EDE3_CBC);
         
-        RecipientId                recSel = new RecipientId();
-        
-        recSel.setKeyIdentifier(kekId2);
+        RecipientId                recSel = new KEKRecipientId(kekId2);
         
         RecipientInformation       recipient = recipients.get(recSel);
         
         assertEquals(recipient.getKeyEncryptionAlgOID(), "2.16.840.1.101.3.4.1.25");
         
-        CMSTypedStream recData = recipient.getContentStream(kek2, "BC");
+        CMSTypedStream recData = recipient.getContentStream(kek2, BC);
         
         assertEquals(true, Arrays.equals(data, CMSTestUtil.streamToByteArray(recData.getContentStream())));
 
@@ -582,13 +585,13 @@ public class EnvelopedDataStreamTest
 
         CMSEnvelopedDataStreamGenerator edGen = new CMSEnvelopedDataStreamGenerator();
 
-        edGen.addKeyAgreementRecipient(CMSEnvelopedDataGenerator.ECDH_SHA1KDF, _origEcKP.getPrivate(), _origEcKP.getPublic(), _reciEcCert, CMSEnvelopedDataGenerator.AES128_WRAP, "BC");
+        edGen.addKeyAgreementRecipient(CMSEnvelopedDataGenerator.ECDH_SHA1KDF, _origEcKP.getPrivate(), _origEcKP.getPublic(), _reciEcCert, CMSEnvelopedDataGenerator.AES128_WRAP, BC);
 
         ByteArrayOutputStream  bOut = new ByteArrayOutputStream();
 
         OutputStream out = edGen.open(
                                 bOut,
-                                CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                                CMSEnvelopedDataGenerator.AES128_CBC, BC);
         out.write(data);
 
         out.close();
@@ -599,14 +602,11 @@ public class EnvelopedDataStreamTest
 
         assertEquals(ep.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.AES128_CBC);
 
-        RecipientId                recSel = new RecipientId();
-
-        recSel.setIssuer(PrincipalUtil.getIssuerX509Principal(_reciEcCert).getEncoded());
-        recSel.setSerialNumber(_reciEcCert.getSerialNumber());
+        RecipientId                recSel = new JceKeyAgreeRecipientId(_reciEcCert);
 
         RecipientInformation       recipient = recipients.get(recSel);
 
-        CMSTypedStream recData = recipient.getContentStream(_reciEcKP.getPrivate(), "BC");
+        CMSTypedStream recData = recipient.getContentStream(_reciEcKP.getPrivate(), BC);
 
         assertEquals(true, Arrays.equals(data, CMSTestUtil.streamToByteArray(recData.getContentStream())));
 
