@@ -22,9 +22,9 @@ import java.util.Set;
 
 import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1String;
 import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.DERSequence;
-import org.bouncycastle.asn1.DERString;
 import org.bouncycastle.asn1.x509.GeneralName;
 import org.bouncycastle.asn1.x509.GeneralNames;
 import org.bouncycastle.jce.X509Principal;
@@ -344,7 +344,7 @@ public class AttrCertTest
             fail("wrong general name type found in role");
         }
         
-        if (!((DERString)role.getName()).getString().equals("DAU123456789"))
+        if (!((ASN1String)role.getName()).getString().equals("DAU123456789"))
         {
             fail("wrong general name value found in role");
         }
