@@ -67,8 +67,10 @@ public class RegressionTest
         new SigNameTest(),
         new MQVTest(),
         new CMacTest(),
+        new GMacTest(),
         new DSTU4145Test(),
         new CRL5Test(),
+        new SipHashTest(),
         new SHA3Test()
     };
 
