@@ -1,41 +1,42 @@
 package org.bouncycastle.jce.provider.test.nist;
 
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
-import org.bouncycastle.asn1.ASN1Encodable;
-import org.bouncycastle.asn1.x509.X509Extensions;
-import org.bouncycastle.x509.extension.X509ExtensionUtil;
-
 import java.io.FileInputStream;
 import java.io.InputStream;
 import java.security.Security;
+import java.security.cert.CertPath;
+import java.security.cert.CertPathBuilder;
+import java.security.cert.CertPathBuilderException;
 import java.security.cert.CertPathValidator;
 import java.security.cert.CertPathValidatorException;
 import java.security.cert.CertStore;
 import java.security.cert.CertificateFactory;
 import java.security.cert.CollectionCertStoreParameters;
+import java.security.cert.PKIXBuilderParameters;
+import java.security.cert.PKIXCertPathBuilderResult;
 import java.security.cert.PKIXCertPathValidatorResult;
 import java.security.cert.PKIXParameters;
 import java.security.cert.TrustAnchor;
 import java.security.cert.X509CRL;
-import java.security.cert.X509Certificate;
-import java.security.cert.CertPathBuilder;
 import java.security.cert.X509CertSelector;
-import java.security.cert.PKIXBuilderParameters;
-import java.security.cert.CertPathBuilderResult;
-import java.security.cert.CertPathBuilderException;
-import java.security.cert.CertPath;
-import java.security.cert.PKIXCertPathBuilderResult;
+import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.x509.X509Extension;
+import org.bouncycastle.x509.extension.X509ExtensionUtil;
+
 /**
  * NIST CertPath test data for RFC 3280
  */
@@ -205,7 +206,7 @@ public class NistCertPathTest
                 new String[] { "NegativeSerialNumberCACert", "InvalidNegativeSerialNumberTest15EE" }, 
                 new String[] { TRUST_ANCHOR_ROOT_CRL, "NegativeSerialNumberCACRL" },
                 0,
-                "Certificate revocation after Fri Apr 20 00:57:20 EST 2001, reason: keyCompromise");
+                "Certificate revocation after Fri Apr 20 00:57:20", "reason: keyCompromise");
     }
     
     //
@@ -630,6 +631,29 @@ public class NistCertPathTest
             assertEquals(message, e.getMessage());
         }
     }
+
+    private void doExceptionTest(
+        String      trustAnchor,
+        String[]    certs,
+        String[]    crls,
+        int         index,
+        String      mesStart,
+        String      mesEnd)
+        throws Exception
+    {
+        try
+        {
+            doTest(trustAnchor, certs, crls);
+            
+            fail("path accepted when should be rejected");
+        }
+        catch (CertPathValidatorException e)
+        {
+            assertEquals(index, e.getIndex());
+            assertTrue(e.getMessage().startsWith(mesStart));
+            assertTrue(e.getMessage().endsWith(mesEnd));
+        }
+    }
     
     private PKIXCertPathValidatorResult doTest(
         String      trustAnchor,
@@ -672,7 +696,8 @@ public class NistCertPathTest
         
         params.addCertStore(store);
         params.setRevocationEnabled(true);
-        
+        params.setDate(new GregorianCalendar(2010, 1, 1).getTime());
+
         if (policies != null)
         {
             params.setExplicitPolicyRequired(true);
@@ -732,6 +757,7 @@ public class NistCertPathTest
         }
 
         builderParams.addCertStore(store);
+        builderParams.setDate(new GregorianCalendar(2010, 1, 1).getTime());
 
         try
         {
@@ -804,13 +830,13 @@ public class NistCertPathTest
         throws Exception
     {
         X509Certificate cert = loadCert(trustAnchorName);
-        byte[]          extBytes = cert.getExtensionValue(X509Extensions.NameConstraints.getId());
+        byte[]          extBytes = cert.getExtensionValue(X509Extension.nameConstraints.getId());
         
         if (extBytes != null)
         {
             ASN1Encodable extValue = X509ExtensionUtil.fromExtensionValue(extBytes);
             
-            return new TrustAnchor(cert, extValue.getDEREncoded());
+            return new TrustAnchor(cert, extValue.toASN1Primitive().getEncoded(ASN1Encoding.DER));
         }
         
         return new TrustAnchor(cert, null);
