@@ -32,7 +32,8 @@ import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 import org.bouncycastle.asn1.ASN1Encodable;
-import org.bouncycastle.asn1.x509.X509Extensions;
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.x509.X509Extension;
 import org.bouncycastle.cms.SignerInformation;
 import org.bouncycastle.cms.test.CMSTestUtil;
 import org.bouncycastle.i18n.ErrorBundle;
@@ -412,13 +413,13 @@ public class SignedMailValidatorTest extends TestCase
         if (cert != null)
         {
             byte[] ncBytes = cert
-                    .getExtensionValue(X509Extensions.NameConstraints.getId());
+                    .getExtensionValue(X509Extension.nameConstraints.getId());
 
             if (ncBytes != null)
             {
                 ASN1Encodable extValue = X509ExtensionUtil
                         .fromExtensionValue(ncBytes);
-                return new TrustAnchor(cert, extValue.getDEREncoded());
+                return new TrustAnchor(cert, extValue.toASN1Primitive().getEncoded(ASN1Encoding.DER));
             }
             return new TrustAnchor(cert, null);
         }
