@@ -1,7 +1,6 @@
 package org.bouncycastle.ocsp.test;
 
 import java.io.ByteArrayInputStream;
-import java.io.FileOutputStream;
 import java.math.BigInteger;
 import java.security.KeyPair;
 import java.security.Security;
@@ -70,8 +69,7 @@ public class OCSPTest
             + "r07NEadxM3HQkt0aX5XYEl8eRoifwqYAI9h0ziZfTNes8elNfb3DoPPjqq6V"
             + "mMg0f0iMS4W8LjNPorjRB+kIosa1deAGPhq0eJ8yr0/s2QR2/WFD5P4aXc8I"
             + "KWleklnIImS3zqiPrq6tl2Bm8DZj7vXlTOwmraSQxUwzCKwYob1yGvNOUQTq"
-            + "pG6jxn7jgDawHU1+WjWQe4Q34/pWeGLysxTraMa+Ug9kPe+jy/qRX2xwvKBZ"
-            + "====");
+            + "pG6jxn7jgDawHU1+WjWQe4Q34/pWeGLysxTraMa+Ug9kPe+jy/qRX2xwvKBZ");
 
     byte[] testResp2 = Base64.decode(
         "MIII1QoBAKCCCM4wggjKBgkrBgEFBQcwAQEEggi7MIIItzCBjqADAgEAoSMw"
@@ -124,8 +122,7 @@ public class OCSPTest
             + "tO8yWWl+xWIuxKoAO8a0Rh97TyYfAj4++GIm43b2zIvRXEWAytjz7rXUMwRC"
             + "1ipRQwSA9gyw2y0s8emV/VwJQXsTe9xtDqlEC67b90V/BgL/jxck5E8yrY9Z"
             + "gNxlOgcqscObisAkB5I6GV+dfa+BmZrhSJ/bvFMUrnFzjLFvZp/9qiK11r5K"
-            + "A5oyOoNv0w+8bbtMNEc1"
-            + "====");
+            + "A5oyOoNv0w+8bbtMNEc1");
 
     /**
      * extra version number encoding.
@@ -244,9 +241,7 @@ public class OCSPTest
         // basic request generation
         //
         OCSPReqGenerator gen = new OCSPReqGenerator();
-
-        gen.addRequest(
-            new CertificateID(CertificateID.HASH_SHA1, testCert, BigInteger.valueOf(1)));
+        gen.addRequest(id);
 
         OCSPReq req = gen.generate();
 
@@ -634,6 +629,11 @@ public class OCSPTest
         //
         CertificateID id = new CertificateID(CertificateID.HASH_SHA1, testCert, BigInteger.valueOf(1));
 
+        //
+        // general id value for our test issuer cert and a serial number and the default provider
+        //
+        id = new CertificateID(CertificateID.HASH_SHA1, testCert, BigInteger.valueOf(1), null);
+
         //
         // basic request generation
         //
