@@ -1,10 +1,21 @@
 package org.bouncycastle.openpgp.test;
 
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.security.NoSuchProviderException;
+import java.security.Security;
+import java.security.SignatureException;
+import java.util.Date;
+
 import org.bouncycastle.bcpg.CompressionAlgorithmTags;
 import org.bouncycastle.bcpg.HashAlgorithmTags;
 import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
 import org.bouncycastle.bcpg.SignatureSubpacketTags;
 import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
+import org.bouncycastle.bcpg.sig.KeyFlags;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.openpgp.PGPLiteralData;
@@ -23,19 +34,10 @@ import org.bouncycastle.openpgp.PGPSignatureSubpacketGenerator;
 import org.bouncycastle.openpgp.PGPSignatureSubpacketVector;
 import org.bouncycastle.openpgp.PGPV3SignatureGenerator;
 import org.bouncycastle.util.encoders.Base64;
+import org.bouncycastle.util.io.Streams;
 import org.bouncycastle.util.test.SimpleTest;
 import org.bouncycastle.util.test.UncloseableOutputStream;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.security.NoSuchProviderException;
-import java.security.Security;
-import java.security.SignatureException;
-import java.util.Date;
-
 public class PGPSignatureTest
     extends SimpleTest
 {
@@ -262,7 +264,7 @@ public class PGPSignatureTest
             
             fail("failed to detect non-key signature.");
         }
-        catch (IllegalStateException e)
+        catch (PGPException e)
         {
             // expected
         }
@@ -297,6 +299,11 @@ public class PGPSignatureTest
         {
             fail("found wrong number of hashed packets in override test");
         }
+
+        if (!hashedPcks.hasSubpacket(SignatureSubpacketTags.CREATION_TIME))
+        {
+            fail("hasSubpacket test for creation time failed");
+        }
         
         if (!hashedPcks.getSignatureCreationTime().equals(new Date(0L)))
         {
@@ -394,6 +401,46 @@ public class PGPSignatureTest
         testMissingSubpackets(nullPacketsSubKeyBinding);
         
         testMissingSubpackets(generateV3BinarySig(pgpPrivKey, PublicKeyAlgorithmTags.DSA, HashAlgorithmTags.SHA1));
+
+        // keyflags
+        testKeyFlagsValues();
+    }
+
+    private void testKeyFlagsValues()
+    {
+        checkValue(KeyFlags.CERTIFY_OTHER, 0x01);
+        checkValue(KeyFlags.SIGN_DATA, 0x02);
+        checkValue(KeyFlags.ENCRYPT_COMMS, 0x04);
+        checkValue(KeyFlags.ENCRYPT_STORAGE, 0x08);
+        checkValue(KeyFlags.SPLIT, 0x10);
+        checkValue(KeyFlags.AUTHENTICATION, 0x20);
+        checkValue(KeyFlags.SHARED, 0x80);
+
+        // yes this actually happens
+        checkValue(new byte[] { 4, 0, 0, 0 }, 0x04);
+        checkValue(new byte[] { 4, 0, 0 }, 0x04);
+        checkValue(new byte[] { 4, 0 }, 0x04);
+        checkValue(new byte[] { 4 }, 0x04);
+    }
+
+    private void checkValue(int flag, int value)
+    {
+        KeyFlags f = new KeyFlags(true, flag);
+
+        if (f.getFlags() != value)
+        {
+            fail("flag value mismatch");
+        }
+    }
+
+    private void checkValue(byte[] flag, int value)
+    {
+        KeyFlags f = new KeyFlags(true, flag);
+
+        if (f.getFlags() != value)
+        {
+            fail("flag value mismatch");
+        }
     }
 
     private void testMissingSubpackets(byte[] signature) 
@@ -408,10 +455,7 @@ public class PGPSignatureTest
             if (obj instanceof PGPLiteralData)
             {
                 InputStream in = ((PGPLiteralData)obj).getDataStream();
-                while (in.read() >= 0)
-                {
-                    // do nothing
-                }
+                Streams.drain(in);
             }
         }
         
@@ -425,6 +469,11 @@ public class PGPSignatureTest
             {
                 fail("key expiration time not zero for missing subpackets");
             }
+
+            if (!sig.hasSubpackets())
+            {
+                fail("hasSubpackets() returns false with packets");
+            }
         }
         else
         {
@@ -436,6 +485,11 @@ public class PGPSignatureTest
             {
                 fail("unhashed sub packets found when none expected");
             }
+
+            if (sig.hasSubpackets())
+            {
+                fail("hasSubpackets() returns true with no packets");
+            }
         }
     }
 
@@ -544,7 +598,14 @@ public class PGPSignatureTest
         
         lGen.close();
     
-        sGen.generate().encode(bOut);
+        PGPSignature sig = sGen.generate();
+
+        if (sig.getCreationTime().getTime() == 0)
+        {
+            fail("creation time not set in v4 signature");
+        }
+
+        sig.encode(bOut);
     
         verifySignature(bOut.toByteArray(), hashAlgorithm, pubKey, canonicalData);
     }
@@ -632,8 +693,15 @@ public class PGPSignatureTest
         
         lGen.close();
     
-        sGen.generate().encode(bOut);
-    
+        PGPSignature sig = sGen.generate();
+
+        if (sig.getCreationTime().getTime() == 0)
+        {
+            fail("creation time not set in v3 signature");
+        }
+
+        sig.encode(bOut);
+
         verifySignature(bOut.toByteArray(), hashAlgorithm, pubKey, canonicalData);
     }
     
