@@ -1,14 +1,5 @@
 package org.bouncycastle.openssl.test;
 
-import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
-import org.bouncycastle.asn1.cms.ContentInfo;
-import org.bouncycastle.jce.provider.BouncyCastleProvider;
-import org.bouncycastle.jce.spec.ECNamedCurveParameterSpec;
-import org.bouncycastle.openssl.PEMReader;
-import org.bouncycastle.openssl.PEMWriter;
-import org.bouncycastle.openssl.PasswordFinder;
-import org.bouncycastle.util.test.SimpleTest;
-
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -17,6 +8,7 @@ import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.io.Reader;
+import java.math.BigInteger;
 import java.security.KeyPair;
 import java.security.KeyPairGenerator;
 import java.security.PrivateKey;
@@ -24,9 +16,20 @@ import java.security.PublicKey;
 import java.security.SecureRandom;
 import java.security.Security;
 import java.security.Signature;
+import java.security.cert.X509Certificate;
 import java.security.interfaces.DSAPrivateKey;
+import java.security.interfaces.RSAPrivateCrtKey;
 import java.security.interfaces.RSAPrivateKey;
 
+import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
+import org.bouncycastle.asn1.cms.ContentInfo;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.bouncycastle.jce.spec.ECNamedCurveParameterSpec;
+import org.bouncycastle.openssl.PEMReader;
+import org.bouncycastle.openssl.PEMWriter;
+import org.bouncycastle.openssl.PasswordFinder;
+import org.bouncycastle.util.test.SimpleTest;
+
 /**
  * basic class for reading test.pem - the password is "secret"
  */
@@ -77,7 +80,7 @@ public class ReaderTest
             if (o instanceof KeyPair)
             {
                 //pair = (KeyPair)o;
-        
+
                 //System.out.println(pair.getPublic());
                 //System.out.println(pair.getPrivate());
             }
@@ -86,13 +89,24 @@ public class ReaderTest
                 //System.out.println(o.toString());
             }
         }
-        
+
+        // test bogus lines before begin are ignored.
+        pemRd = openPEMResource("extratest.pem", pGet);
+
+        while ((o = pemRd.readObject()) != null)
+        {
+            if (!(o instanceof X509Certificate))
+            {
+                fail("wrong object found");
+            }
+        }
+
         //
         // pkcs 7 data
         //
         pemRd = openPEMResource("pkcs7.pem", null);
-        ContentInfo d = (ContentInfo)pemRd.readObject();    
-            
+        ContentInfo d = (ContentInfo)pemRd.readObject();
+
         if (!d.getContentType().equals(CMSObjectIdentifiers.envelopedData))
         {
             fail("failed envelopedData check");
@@ -138,30 +152,30 @@ public class ReaderTest
         // writer/parser test
         //
         KeyPairGenerator      kpGen = KeyPairGenerator.getInstance("RSA", "BC");
-        
+
         pair = kpGen.generateKeyPair();
-        
+
         keyPairTest("RSA", pair);
-        
+
         kpGen = KeyPairGenerator.getInstance("DSA", "BC");
         kpGen.initialize(512, new SecureRandom());
         pair = kpGen.generateKeyPair();
-        
+
         keyPairTest("DSA", pair);
-        
+
         //
         // PKCS7
         //
         ByteArrayOutputStream bOut = new ByteArrayOutputStream();
         PEMWriter             pWrt = new PEMWriter(new OutputStreamWriter(bOut));
-        
+
         pWrt.writeObject(d);
-        
+
         pWrt.close();
-        
+
         pemRd = new PEMReader(new InputStreamReader(new ByteArrayInputStream(bOut.toByteArray())));
-        d = (ContentInfo)pemRd.readObject();    
-        
+        d = (ContentInfo)pemRd.readObject();
+
         if (!d.getContentType().equals(CMSObjectIdentifiers.envelopedData))
         {
             fail("failed envelopedData recode check");
@@ -185,6 +199,50 @@ public class ReaderTest
         doOpenSslRsaTest("rc2_40_cbc");
         doOpenSslDsaTest("rc2_64_cbc");
         doOpenSslRsaTest("rc2_64_cbc");
+
+        doDudPasswordTest("7fd98", 0, "corrupted stream - out of bounds length found");
+        doDudPasswordTest("ef677", 1, "corrupted stream - out of bounds length found");
+        doDudPasswordTest("800ce", 2, "unknown tag 26 encountered");
+        doDudPasswordTest("b6cd8", 3, "DEF length 81 object truncated by 56");
+        doDudPasswordTest("28ce09", 4, "DEF length 110 object truncated by 28");
+        doDudPasswordTest("2ac3b9", 5, "DER length more than 4 bytes: 11");
+        doDudPasswordTest("2cba96", 6, "DEF length 100 object truncated by 35");
+        doDudPasswordTest("2e3354", 7, "DEF length 42 object truncated by 9");
+        doDudPasswordTest("2f4142", 8, "DER length more than 4 bytes: 14");
+        doDudPasswordTest("2fe9bb", 9, "DER length more than 4 bytes: 65");
+        doDudPasswordTest("3ee7a8", 10, "DER length more than 4 bytes: 57");
+        doDudPasswordTest("41af75", 11, "unknown tag 16 encountered");
+        doDudPasswordTest("1704a5", 12, "corrupted stream detected");
+        doDudPasswordTest("1c5822", 13, "unknown object in getInstance: org.bouncycastle.asn1.DERUTF8String");
+        doDudPasswordTest("5a3d16", 14, "corrupted stream detected");
+        doDudPasswordTest("8d0c97", 15, "corrupted stream detected");
+        doDudPasswordTest("bc0daf", 16, "corrupted stream detected");
+        doDudPasswordTest("aaf9c4d",17, "corrupted stream - out of bounds length found");
+
+        doNoPasswordTest();
+
+        // encrypted private key test
+        pGet = new Password("password".toCharArray());
+        pemRd = openPEMResource("enckey.pem", pGet);
+
+        RSAPrivateCrtKey privKey = (RSAPrivateCrtKey)pemRd.readObject();
+
+        if (!privKey.getPublicExponent().equals(new BigInteger("10001", 16)))
+        {
+            fail("decryption of private key data check failed");
+        }
+
+        // general PKCS8 test
+        pGet = new Password("password".toCharArray());
+        pemRd = openPEMResource("pkcs8test.pem", pGet);
+
+        while ((privKey = (RSAPrivateCrtKey)pemRd.readObject()) != null)
+        {
+            if (!privKey.getPublicExponent().equals(new BigInteger("10001", 16)))
+            {
+                fail("decryption of private key data check failed");
+            }
+        }
     }
 
     private void keyPairTest(
@@ -297,6 +355,58 @@ public class ReaderTest
         }
     }
 
+    private void doDudPasswordTest(String password, int index, String message)
+    {
+        // illegal state exception check - in this case the wrong password will
+        // cause an underlying class cast exception.
+        try
+        {
+            PasswordFinder pGet = new Password(password.toCharArray());
+
+            PEMReader pemRd = openPEMResource("test.pem", pGet);
+            Object o;
+
+            while ((o = pemRd.readObject()) != null)
+            {
+            }
+
+            fail("issue not detected: " + index);
+        }
+        catch (IOException e)
+        {
+            if (e.getCause() != null && !e.getCause().getMessage().equals(message))
+            {
+               e.printStackTrace();
+               fail("issue " + index + " exception thrown, but wrong message");
+            }
+            else if (e.getCause() == null && !e.getMessage().equals(message))
+            {
+                               e.printStackTrace();
+               fail("issue " + index + " exception thrown, but wrong message");
+            }
+        }
+    }
+
+    private void doNoPasswordTest()
+        throws IOException
+    {
+        PasswordFinder pGet = new Password("".toCharArray());
+
+        PEMReader pemRd = openPEMResource("smimenopw.pem", pGet);
+        Object o;
+        PrivateKey key = null;
+
+        while ((o = pemRd.readObject()) != null)
+        {
+             key = (PrivateKey)o;
+        }
+
+        if (key == null)
+        {
+            fail("private key not detected");
+        }
+    }
+
     public static void main(
         String[]    args)
     {
